#include "common.h"

//double Get_distance(LOCATION, LOCATION);

double Get_LCS_theta(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi);
double Get_LCS_pi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi);
MatrixXd Get_distance_angular(double a, double b, double c, double x, double y, double z);


void LINK::Configuration(int _self_ms_idx)
{
	self_ms_idx= _self_ms_idx;

	UE_Initial_Setting();
	/*
	INPUT:
	OUTPUT: incar_loss, power_per_sc, UE_ICI, UE_PATH_LOSS
	Notes: (outdoor to car loss)
	*/


	Get_signal_interference();
	/*
	INPUT: pathloss, antgain
	OUTPUT: Signal power, interference power
	Notes: Large Scale  SINRϱ  interference  κ
	*/

	Get_adj_SECTORS();
	/*
	INPUT: static gain
	OUTPUT: adj_sector
	Notes:  ms   index 3 ã
	*/

	

	SINR = 10. * log10(pow(10., signal / 10.) / (pow(10., interference / 10.) + pow(10., (noise + MS_noisefig) / 10.)));

}



void LINK::Get_signal_interference()
{

	double strongest_signal;
	double pathloss;
	double antgain;
	double max_array_factor_gain = 0;
	double UE_max_array_factor_gain = 0;
	double random_array_factor_gain = 0;  /// for interference
	double UE_random_array_factor_gain = 0;
	double RMS_delay;
	double AOA_spread;
	double AOD_spread;
	double distance;
	double UE_antenna_gain = 0.;
	interference = 0.;
	double RSRP_antgain = 0;
	double MAX_RSRP_antgain = 0.;
	double random_RSRP_antgain = 0;
	//link_array_factor[] = 0;

	if (TYPE == 11) //// 5G InH
	{
		for (int bs_idx = 0; bs_idx < num_BS; bs_idx++)
		{
			pathloss = channel[bs_idx][self_ms_idx].pathloss;
			RMS_delay = channel[bs_idx][self_ms_idx].RMS_delay_spread;
			AOA_spread = channel[bs_idx][self_ms_idx].circular_angle_spread_AOA;
			AOD_spread = channel[bs_idx][self_ms_idx].circular_angle_spread_AOD;
			distance = channel[bs_idx][self_ms_idx].distance;

			//pathloss = pathloss + ms[self_ms_idx].LSPs[bs_idx](0);   ////// Add Shadow Fading
			pathloss = pathloss + channel[bs_idx][self_ms_idx].sigma_SF * randnum.n();   ////// Add Shadow Fading

			//antgain = 0.;   //omnidirectional case



			//UE_antenna_gain = Get_MS_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, 0);   ///// MAX antgain - combined gain;
			//Get_RX_SmallScale_antgain(bs_idx, 0);



			// ׳ ҋ  LOS_AOA, LOS_ZOA  ٲ۴

			antgain = Get_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, 1,0,0);   ///// MAX antgain - combined gain
			//UE_antenna_gain = Get_MS_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, 1);   ///// MAX antgain - combined gain;
			//antgain = antgain + UE_antenna_gain;

			//Get_TX_SmallScale_antgain(bs_idx, 0, 0, 0);
			//RSRP_antgain = Get_RSRP(bs_idx, self_ms_idx);




			//max_array_factor_gain = Get_arrayfactor(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, 1);



			//signal = bs_maxpower + antgain + max_array_factor_gain - pathloss;
			signal = bs_maxpower + antgain - pathloss;
			//signal = bs_maxpower + RSRP_antgain - pathloss;

			//signal = 10*log10( pow(10, bs_maxpower/10) * pow(10, RSRP_antgain/10) * pow(10, pathloss/10) );

			if (bs_idx == 0)
			{
				strongest_signal = signal;
				self_bs_idx = ms[self_ms_idx].nearest_bs_idx;
				self_sector_idx = 0;
				link_pathloss = pathloss;
				link_antgain = RSRP_antgain;
				link_RMS_delay_spread = RMS_delay;
				link_AOA_spread = AOA_spread;
				link_AOD_spread = AOD_spread;
				link_distance = distance;

				// indoor num_BS = num_sector , bs = sector ׳  

				_sector_in_control = self_bs_idx;

			}
			else if (signal > strongest_signal)
			{
				interference += pow(10., strongest_signal / 10.);
				strongest_signal = signal;
				self_bs_idx = ms[self_ms_idx].nearest_bs_idx;
				self_sector_idx = 0;
				link_pathloss = pathloss;
				link_antgain = RSRP_antgain;
				link_RMS_delay_spread = RMS_delay;
				link_AOA_spread = AOA_spread;
				link_AOD_spread = AOD_spread;
				link_distance = distance;


				_sector_in_control = self_bs_idx;

			}
			else
			{
				interference += pow(10., signal / 10.);
			}
		}

	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	else if (TYPE == 12)
	{
		
		for (int bs_idx = 0; bs_idx < num_BS; bs_idx++)
		{

			pathloss = channel[bs_idx][self_ms_idx].pathloss; //channel  pathloss´ // self_ms_idx   19BS  
			RMS_delay = channel[bs_idx][self_ms_idx].RMS_delay_spread;
			AOA_spread = channel[bs_idx][self_ms_idx].circular_angle_spread_AOA;
			AOD_spread = channel[bs_idx][self_ms_idx].circular_angle_spread_AOD;
			distance = channel[bs_idx][self_ms_idx].distance;
			//cout << distance << endl;

			//pathloss = pathloss + ms[self_ms_idx].LSPs[bs_idx](0) + incar_loss + Otoi_loss;   ////// Add Shadow Fading

			//pathloss = pathloss + incar_loss + Otoi_loss + channel[bs_idx][self_ms_idx].sigma_SF * randnum.n();
			//pathloss = pathloss + incar_loss + Otoi_loss + 8 * randnum.n();
			pathloss = pathloss + incar_loss + Otoi_loss;


			for (int sector_idx = 1; sector_idx <= 3; sector_idx++)
			{
				/*
				UE_antenna_gain = Get_MS_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx);   ///// MAX antgain - combined gain;
				Get_RX_SmallScale_antgain(bs_idx, sector_idx);


				VectorXd tilt_azimuth_angle_LCS(6);  //// pi
				VectorXd tilt_zenith_angle_LCS(2);  /// theta

				tilt_azimuth_angle_LCS << -5 * pi / 16, -3 * pi / 16, -pi / 16, pi / 16, 3 * pi / 16, 5 * pi / 16;
				tilt_zenith_angle_LCS << 5 * pi / 8, 7 * pi / 8;
				//// tilt  ۿ ޴°ɷ ٲ

				int aa = (int)(tilt_azimuth_angle_LCS.size() * randnum.u());
				int zz = (int)(tilt_zenith_angle_LCS.size() * randnum.u());

				for (int a = 0; a < tilt_azimuth_angle_LCS.size(); a++)
				{
					for (int z = 0; z < tilt_zenith_angle_LCS.size(); z++)
					{
						antgain = Get_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx, tilt_azimuth_angle_LCS(a), tilt_zenith_angle_LCS(z));   ///// MAX antgain - combined gain
						Get_TX_SmallScale_antgain(bs_idx, sector_idx, tilt_azimuth_angle_LCS(a), tilt_zenith_angle_LCS(z));
						RSRP_antgain = Get_RSRP(bs_idx, self_ms_idx);





						if (a == aa && z == zz)
						{
							random_RSRP_antgain = RSRP_antgain;
						}


						//cout << "tilt_azimuth_angle_LCS.size = " << tilt_azimuth_angle_LCS.size() << endl;

						if (a == 0 && z == 0)
						{
							MAX_RSRP_antgain = RSRP_antgain;
						}
						else if (MAX_RSRP_antgain < RSRP_antgain)
						{
							MAX_RSRP_antgain = RSRP_antgain;
						}
						else
						{
							MAX_RSRP_antgain = MAX_RSRP_antgain;
						}
					}
				}
				*/


				//antgain = 0.; // for small scale calibration


				// BS antenna element
				antgain = Get_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx, 0, 0);
				max_array_factor_gain = Get_arrayfactor(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx ,sector_idx);
				random_array_factor_gain = Get_random_arrayfactor(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx);

				
				// MS antenna element
				if (Frequency < 6000000000)
				{
					// Get signal power
					signal = bs_maxpower + antgain + max_array_factor_gain - pathloss;
				}
				else
				{
					UE_antenna_gain = Get_MS_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx);
					UE_max_array_factor_gain = Get_MS_arrayfactor(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx);
					UE_random_array_factor_gain = Get_random_MS_arrayfactor(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx);

					// Get signal power
					signal = bs_maxpower + antgain + UE_antenna_gain + max_array_factor_gain + UE_max_array_factor_gain - pathloss;
				}
				

				// Get signal power
				signal = bs_maxpower + antgain + max_array_factor_gain - pathloss;
				//signal = bs_maxpower + antgain  - pathloss;
				//signal = bs_maxpower + MAX_RSRP_antgain - pathloss;


				if (bs_idx == 0 && sector_idx == 1)
				{
					strongest_signal = signal;
					link_prev_MAX_RSRP = max_array_factor_gain;
					//link_prev_UE_MAX_RSRP = UE_max_array_factor_gain;
					self_bs_idx = bs_idx;
					self_sector_idx = sector_idx;
					link_pathloss = pathloss;
					link_coupling_loss = pathloss - antgain;
					//link_pathloss = pathloss - ms[self_ms_idx].LSPs[bs_idx](0);
					link_antgain = antgain;
					serving_bs = channel[bs_idx][self_ms_idx].self_bs;
					link_RMS_delay_spread = RMS_delay;
					link_AOA_spread = AOA_spread;
					link_AOD_spread = AOD_spread;
					link_distance = channel[bs_idx][self_ms_idx].distance;

					_sector_in_control = self_bs_idx * 3 + (self_sector_idx - 1);


				}
				else if (signal > strongest_signal)
				{

					//interference += pow(10., (strongest_signal - link_prev_MAX_RSRP - link_prev_UE_MAX_RSRP + random_array_factor_gain + UE_random_array_factor_gain) / 10.);
					interference += pow(10., (strongest_signal - link_prev_MAX_RSRP + random_array_factor_gain) / 10.);
					//interference += pow(10., (strongest_signal) / 10.);
					strongest_signal = signal;
					link_prev_MAX_RSRP = max_array_factor_gain;
					//link_prev_UE_MAX_RSRP = UE_max_array_factor_gain;
					self_bs_idx = bs_idx; // serving BS idx
					self_sector_idx = sector_idx;
					link_pathloss = pathloss;
					link_coupling_loss = pathloss - antgain;
					//link_pathloss = pathloss - ms[self_ms_idx].LSPs[bs_idx](0);
					link_antgain = antgain;
					serving_bs = channel[bs_idx][self_ms_idx].self_bs;
					link_RMS_delay_spread = RMS_delay;
					link_AOA_spread = AOA_spread;
					link_AOD_spread = AOD_spread;
					link_distance = channel[bs_idx][self_ms_idx].distance;

					_sector_in_control = self_bs_idx * 3 + (self_sector_idx - 1);


				}
				else
				{
					//interference += pow(10., (strongest_signal - link_prev_MAX_RSRP - link_prev_UE_MAX_RSRP + random_array_factor_gain + UE_random_array_factor_gain) / 10.);
					interference += pow(10., (strongest_signal - link_prev_MAX_RSRP  + random_array_factor_gain) / 10.);
					//interference += pow(10., (signal) / 10.);
				}
			}

			
		}
	}
	else if (TYPE == 13)
	{

		for (int bs_idx = 0; bs_idx < num_BS; bs_idx++)
		{

			pathloss = channel[bs_idx][self_ms_idx].pathloss; //channel  pathloss´ // self_ms_idx   19BS  
			RMS_delay = channel[bs_idx][self_ms_idx].RMS_delay_spread;
			AOA_spread = channel[bs_idx][self_ms_idx].circular_angle_spread_AOA;
			AOD_spread = channel[bs_idx][self_ms_idx].circular_angle_spread_AOD;
			distance = channel[bs_idx][self_ms_idx].distance;
			//cout << distance << endl;

			//pathloss = pathloss + ms[self_ms_idx].LSPs[bs_idx](0) + incar_loss + Otoi_loss;   ////// Add Shadow Fading

			pathloss = pathloss + incar_loss + Otoi_loss + channel[bs_idx][self_ms_idx].sigma_SF * randnum.n();
			//pathloss = pathloss + incar_loss + Otoi_loss + 8 * randnum.n();
			//pathloss = pathloss + incar_loss + Otoi_loss;


			for (int sector_idx = 1; sector_idx <= 3; sector_idx++)
			{

				antgain = Get_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, bs_idx, sector_idx,0,0);   ///// MAX antgain - combined gain
				//UE_antenna_gain = Get_MS_antgain(channel[bs_idx][self_ms_idx].self_bs, channel[bs_idx][self_ms_idx].self_ms, sector_idx);   ///// MAX antgain - combined gain;

				//antgain = antgain + UE_antenna_gain;


				//Get_TX_SmallScale_antgain(bs_idx, sector_idx);
				//Get_RX_SmallScale_antgain(bs_idx, sector_idx);



				//antgain = 0.; // for small scale calibration

				signal = bs_maxpower + antgain - pathloss;

				/*
				cout << "pathloss = " << pathloss << endl;
				cout << "antgain = " << antgain << endl;
				cout << "signal = " << signal << endl;
				getchar();
				*/

				if (bs_idx == 0 && sector_idx == 1)
				{
					strongest_signal = signal;
					self_bs_idx = bs_idx;
					self_sector_idx = sector_idx;
					link_pathloss = pathloss;
					link_coupling_loss = pathloss - antgain;
					//link_pathloss = pathloss - ms[self_ms_idx].LSPs[bs_idx](0);
					link_antgain = antgain;
					serving_bs = channel[bs_idx][self_ms_idx].self_bs;
					link_RMS_delay_spread = RMS_delay;
					link_AOA_spread = AOA_spread;
					link_AOD_spread = AOD_spread;
					link_distance = channel[bs_idx][self_ms_idx].distance;

					_sector_in_control = self_bs_idx * 3 + (self_sector_idx - 1);


				}
				else if (signal > strongest_signal)
				{

					interference += pow(10., (strongest_signal) / 10.);
					strongest_signal = signal;
					self_bs_idx = bs_idx; // serving BS idx
					self_sector_idx = sector_idx;
					link_pathloss = pathloss;
					link_coupling_loss = pathloss - antgain;
					//link_pathloss = pathloss - ms[self_ms_idx].LSPs[bs_idx](0);
					link_antgain = antgain;
					serving_bs = channel[bs_idx][self_ms_idx].self_bs;
					link_RMS_delay_spread = RMS_delay;
					link_AOA_spread = AOA_spread;
					link_AOD_spread = AOD_spread;
					link_distance = channel[bs_idx][self_ms_idx].distance;

					_sector_in_control = self_bs_idx * 3 + (self_sector_idx - 1);


				}
				else
				{
					interference += pow(10., (signal) / 10.);
				}
			}


		}
	}
		



	str_signal = strongest_signal;
	signal = strongest_signal;
	interference = 10. * log10(interference);

}

void LINK::Get_adj_SECTORS() //num_sector ε ڿ ߾ static ,  ã
{
	LS_gain = new double[num_SECTORS];      //LS gain antgain - (pathloss + shadowing) sorting Ѱ
	static_gain = new double[num_SECTORS];  //antgain - (pathloss + shadowing)
	adj_sector = new int[num_compute_coef]; //coeff ŭ sector ε 

	double pathlosss;
	double antgainn;

	
	if (TYPE == 11) //// 5G InH
	{
		//NO USE
	}

	else
	{
		/////////////////////////////////////
		///LS gain GET
		/////////////////////////////////////
		for (int bs_idx = 0; bs_idx < num_BS; bs_idx++)
		{
			pathlosss = channel[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]][self_ms_idx].pathloss; /// pathloss

			pathlosss = pathlosss + ms[self_ms_idx].LSPs[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]](0);   ////// Add Shadow Fading

			for (int sector_idx = 1; sector_idx <= 3; sector_idx++)
			{

				antgainn = Get_antgain(channel[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]][self_ms_idx].self_bs, channel[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]][self_ms_idx].self_ms, bs_idx, sector_idx,0,0);

				//signal = bs_maxpower + antgain - pathloss;

				LS_gain[ wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx] * 3 + (sector_idx - 1)] = antgainn - pathlosss; //sorting Ѱ
				static_gain[ wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx] * 3 + (sector_idx - 1)] = antgainn - pathlosss; //static gain num_SECTORS  ߾ Ѱ
			}
		}
		///////////////////////////////////////////////////////

		///////////////////////////////////
		//LS gain sorting  ///////////////
		//////////////////////////////////
		for (int i = 1; i < num_SECTORS; i++)
		{
			double temp;
			int j;

			temp = LS_gain[i]; //LSgain antgain - (shadowing + pathloss)
			for (j = i; j > 0; j--)
			{
				if (LS_gain[j - 1] < temp)
					LS_gain[j] = LS_gain[j - 1];
				else break;
			}
			LS_gain[j] = temp;
		} //LS_gain ūź  ۰ sorting, [0]  ũ []  , LS_gain[0]  ū pathloss   -> 
		//////////////////////////////////////


		for (int bs_idx = 0; bs_idx < num_BS; bs_idx++)
		{
			pathlosss = channel[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]][self_ms_idx].pathloss; //channel  pathloss´

			pathlosss = pathlosss + ms[self_ms_idx].LSPs[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]](0);   ////// Add Shadow Fading

			for (int sector_idx = 1; sector_idx <= 3; sector_idx++)
			{
				antgainn = Get_antgain(channel[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]][self_ms_idx].self_bs, channel[wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx]][self_ms_idx].self_ms, bs_idx, sector_idx,0,0);
				///////////////////////////////////////////////////////////////////////////////////////////////////////
				for (int idx = 0; idx < num_compute_coef; idx++)
				{
					if ((int)(1000000 * LS_gain[idx]) == (int)(1000000 * (antgainn - pathlosss)))
					{
						adj_sector[idx] = wrap_mat[ms[self_ms_idx].nearest_bs_idx][bs_idx] * 3 + (sector_idx - 1); //ϴ sector_idx ãƼ  // bs_idx*3 + (sector_idx - 1) ϸ num_SECTORS .
					}
				}
				//_sector_in_control = adj_sector[0]; //[0]    = sector_IN_control
				//////////////////////////////////////////////////////////////////////////////////////////////////////
			}
		}

		//////////   sector sector_in_control ϵ TEST /////////////

		
		//int idx = self_ms_idx / 10;

		//_sector_in_control = idx;

		//cout << "_sector_in_control = " << _sector_in_control << endl;
		//getchar();
		
		/////////////////////////////////////////////////////////////////////////////////////////////

	}
}


void LINK::UE_Initial_Setting(void) 
{


	//// Car penetration loss  //////////////////////////////////////////

  if (TYPE == 12) // Dense Urban 
  {
	  if (ms[self_ms_idx].Indoor == 1)   ///// Indoor
	  {
		  incar_loss = 0.;
	  }
	  else if (ms[self_ms_idx].Indoor == 0)                              ////// outdoor
	  {
		  incar_loss = 9. + 5. * randnum.n();
	  }
	  else
	  {
		  cout << "Channel Model Type ERROR!! - in_car_loss !!" << endl;
		  getchar();

	  }

  }
  else if (TYPE == 13) // Rural
  {
	  if (ms[self_ms_idx].Indoor == 1)   ///// Indoor
	  {
		  incar_loss = 0.;
	  }
	  else                              ////// outdoor
	  {
		  if (Configuration_Type == 2)     /////// Rural configuration C = 40% outdoor pedestrian, 20% outdoor incar
		  {
			  if (randnum.u() < (0.4/0.6))  //// Pedestrian
			  {
				  incar_loss = 0.;
			  }
			  else                   //// In Car
			  {
				  incar_loss = 9. + 5. * randnum.n();
			  }
		  }
		  else    /// 50% All outdoor
		  {
			  incar_loss = 9. + 5. * randnum.n();
		  }
	  }
  }
  else //// Indoor case
  {
    incar_loss = 0.;
  }


  ///////////////////////////// Building penetration loss /////////////////////////////////

  double glass_L    = 2 + 0.2 * Frequency / (1000000000);      ///// Standard multi-pane glass , Frequency = GHz
  double IRRglass_L = 23 + 0.3 * Frequency / (1000000000);     ///// Infrared Reflective(IRR) glass , Frequency = GHz
  double concrete_L = 5 + 4 * Frequency / (1000000000);        ///// Concrete glass , Frequency = GHz
  double wood_L     = 4.85 + 0.12 * Frequency / (1000000000);  ///// Wood glass , Frequency = GHz

  double indoor_L;

  if (TYPE == 12) // Dense Urban UMa
  {
	  //cout << "ms[self_ms_idx].Indoor = " << ms[self_ms_idx].Indoor << endl;
	  //getchar();


	  if (ms[self_ms_idx].Indoor == 0)   ///// Outdoor
	  {
		  Otoi_loss = 0;
	  }
	  else                               /////// Indoor
	  {
		  if (Channel_Model_Type == 1)   ///// Model B
		  {
			  if (randnum.u() > 0.2) //// 80% Low Loss  O2I
			  {
				  /////// Low-loss model

				  Otoi_loss = 5 - 10 * log10(0.3 * pow(10, -1 * (glass_L / 10)) + 0.7 * pow(10, -1 * (concrete_L / 10)));
				  indoor_L = 0.5 * MIN((25 * randnum.u()), (25 * randnum.u()));

				  Otoi_loss = Otoi_loss + indoor_L + 4.4 * randnum.n();

			  }
			  else           ////// 20& High Loss O2I
			  {
				  /////// High-loss model

				  Otoi_loss = 5 - 10 * log10(0.7 * pow(10, -1 * (IRRglass_L / 10)) + 0.3 * pow(10, -1 * (concrete_L / 10)));
				  indoor_L = 0.5 * MIN((25 * randnum.u()), (25 * randnum.u()));

				  Otoi_loss = Otoi_loss + indoor_L + 6.5 * randnum.n();
			  }
		  }
		  else if (Channel_Model_Type == 0)  //// Model A 
		  {
			  if (Frequency > 6000000000.)  //// above 6GHz
			  {
				  if (randnum.u() > 0.2) //// 80% Low Loss  O2I
				  {
					  /////// Low-loss model

					  Otoi_loss = 5 - 10 * log10(0.3 * pow(10, -1 * (glass_L / 10)) + 0.7 * pow(10, -1 * (concrete_L / 10)));
					  indoor_L = 0.5 * MIN((25 * randnum.u()), (25 * randnum.u()));

					  Otoi_loss = Otoi_loss + indoor_L + 4.4 * randnum.n();
				  }
				  else
				  {
					  /////// High-loss model

					  Otoi_loss = 5 - 10 * log10(0.7 * pow(10, -1 * (IRRglass_L / 10)) + 0.3 * pow(10, -1 * (concrete_L / 10)));
					  indoor_L = 0.5 * MIN((25 * randnum.u()), (25 * randnum.u()));

					  Otoi_loss = Otoi_loss + indoor_L + 6.5 * randnum.n();
				  }
			  }
			  else                         //// below 6GHz
			  {
				  Otoi_loss = 20.;
				  indoor_L = 0.5 * ms[self_ms_idx].ms_d_in;

				 // cout << "Otoi_loss = " << Otoi_loss << endl;
				 // cout << "indoor_L = " << indoor_L << endl;

				  Otoi_loss = Otoi_loss + indoor_L;

				 // cout << "Otoi_loss 222 = " << Otoi_loss << endl;
				 // getchar();

			  }
		  }
		  else
		  {
			  cout << "Channel Model Type ERROR!! - Otoi Loss" << endl;
		  }
	  }
  }
  else if (TYPE == 13) // Rural
  {
	  if (ms[self_ms_idx].Indoor == 0)   ///// Outdoor
	  {
		  Otoi_loss = 0;
	  }
	  else                               /////// Indoor
	  {
		  if (Channel_Model_Type == 1)   ///// Model B
		  {
			  /////// 100% Low-loss model
			  Otoi_loss = 5 - 10 * log10(0.3 * pow(10, -1 * (glass_L / 10)) + 0.7 * pow(10, -1 * (concrete_L / 10)));
			  indoor_L = 0.5 * MIN((10 * randnum.u()), (10 * randnum.u()));

			  Otoi_loss = Otoi_loss + indoor_L + 4.4 * randnum.n();

		  }
		  else if (Channel_Model_Type == 0)  //// Model A 
		  {
			  if (Frequency > 6000000000)  //// above 6GHz
			  {
				  /////// 100% Low-loss model
				  Otoi_loss = 5 - 10 * log10(0.3 * pow(10, -1 * (glass_L / 10)) + 0.7 * pow(10, -1 * (concrete_L / 10)));
				  indoor_L = 0.5 * MIN((10 * randnum.u()), (10 * randnum.u()));

				  Otoi_loss = Otoi_loss + indoor_L + 4.4 * randnum.n();
			  }
			  else                         //// below 6GHz
			  {
				  Otoi_loss = 10;
				  indoor_L = 0.5 * (10 * randnum.u());

				  Otoi_loss = Otoi_loss + indoor_L;
			  }
		  }
		  else
		  {
			  cout << "Channel Model Type ERROR!! - Otoi Loss" << endl;
		  }
	  }
  }
  else   ///// Indoor case
  {
	  Otoi_loss = 0.;
  }


  // initial setting
  _throughput = 0 ;
  CQI_offset = 0;
  num_added = 0;
  num_re_tx =    0;
  ACK = true;

  //H_m = NULL;
  //HI_m = NULL;
}


double LINK::Get_angle(LOCATION interferer, LOCATION victim, LOCATION interfere_rx  )
{
	double angle = 0;
	double vector1_x, vector1_y, vector2_x, vector2_y;
	double acos_input = 0;
	

	vector1_x = victim.x - interferer.x;  /// BS->MS vector
	vector1_y = victim.y - interferer.y;
	vector2_x = interfere_rx.x - interferer.x;  /// Boresight vector
	vector2_y = interfere_rx.y - interferer.y;
	acos_input = (vector1_x*vector2_x + vector1_y*vector2_y)
		/ (sqrt(vector1_x*vector1_x + vector1_y*vector1_y)*sqrt(vector2_x*vector2_x + vector2_y*vector2_y));

	if (acos_input >= 1)
	{
		angle = 0;
	}
	else
	{
		angle = acos(acos_input);

		if ((victim.y-interfere_rx.y)<0.)
		{
			angle = -angle;
		}

	}

	return angle * 180. / pi;

}


double LINK::Get_antgain(LOCATION interferer,LOCATION victim, int _bs_idx, int sector_idx, double tilt_a, double tilt_z)  ////// LOS gain BS
{
	// interferer = BS
	// victim = MS

	LOCATION vector;

	vector.x = interferer.x + cos((30 + 120 * (sector_idx - 1))*pi / 180.);
	vector.y = interferer.y + sin((30 + 120 * (sector_idx - 1))*pi / 180.);
		
	if (TYPE == 11)   /// Indoor
	{

		/*
		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));
		//double h_angle = 0.;    ///// NO 3 sector
		//double h_angle = Get_angle(interferer, victim, vector);
		//double v_angle = (atan2((bs_height - ms_height), distance)*180. / pi) + 90.;
		double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOD_GCS;  //// pi in GCS
		double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOD_GCS;  ///// theta in GCS

		//double h_antgain = 0.;   ///// NO 3 sector
		double h_antgain = -MIN(12. * (h_angle_pi / 90.)*(h_angle_pi / 90.), 25.);
		double v_antgain = -MIN(12. * ((v_angle_theta - 90.) / 90.) * ((v_angle_theta - 90.) / 90.), 25.);

		return max_antgain - MIN(-(h_antgain + v_antgain), 25.);
		*/

		///////////////////////////////////////////////
		///////////////////////////////////////////////   GCS-> LCS test //////////////////////////////////////////
		///////////////////////////////////////////////

		
		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));
		//double h_angle_pi = 0.;  //// pi in GCS
		//double v_angle_theta = ((atan2((bs_height - ms_height), distance)*180. / pi) + 90.) * (pi / 180.);  ///// theta in GCS
		double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOD_GCS * (pi / 180.);  //// pi in GCS
		double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOD_GCS * (pi / 180.);  ///// theta in GCS


		double v_angle_theta_LCS = 0.;
		double h_angle_pi_LCS = 0.;

		double v_angle_theta_LCS_temp = 0.;
		double h_angle_pi_LCS_temp = 0.;

		/*
		/////  GCS -> LCS with mechanical down tile angle 
		/////  TR 36.814 / A.2.1.6.2 / equation (1),(2)
		double mechanical_down_tilt_beta = 180. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		v_angle_theta_LCS = acos(cos(h_angle_pi)*sin(v_angle_theta)*sin(mechanical_down_tilt_beta) + cos(v_angle_theta)*cos(mechanical_down_tilt_beta));
		complex <double> h_angle_pi_LCS_complex(cos(h_angle_pi)*sin(v_angle_theta)*cos(mechanical_down_tilt_beta) - cos(v_angle_theta)*sin(mechanical_down_tilt_beta), sin(h_angle_pi)*sin(v_angle_theta));
		h_angle_pi_LCS = arg(h_angle_pi_LCS_complex);
		*/

		/////GCS -> LCS , use alpha, beta, gamma
		///// TR 36.873
		double alpha = 0;
		//double beta = 180. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
		double gamma = 0.;
		double zeta_1 = -pi / 4;
		double zeta_2 = pi / 4;

		v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
		h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

		//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
		//h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);

		double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 90.)*(h_angle_pi_LCS_temp * (180. / pi) / 90.), 25.);
		//double h_antgain = 0.;
		double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.) * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.), 25.);

		double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 25.);


		/// Polarized antennas Model-2
		/// 36.873 -> 7.1.1
		double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));


		double F_theta_GCS = A * cos(zeta_2);
		double F_pi_GCS = A * sin(zeta_2);
		//// -> F_tx_


		//cout << "F_theta_GCS = " << F_theta_GCS << endl;
		//cout << "F_pi_GCS = " << F_pi_GCS << endl;


		//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
		//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
		//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

		/// number of TXRU per TRxP
		/*
		double Mp = 4;
		double Np = 4;
		double P = 2;
		double Mg = 1;
		double Ng = 1;


		double dH = 0.5 * Wavelength;
		double dV = 0.5 * Wavelength;

		double K = BS_M / Mp;    /////   4 / 4 = 1
		double L = BS_N / Np;   //////   4/4 = 1

		complex <double>jay(0, 1);


		//VectorXcd Vertical_weight   (K*L);   //// for theta
		//VectorXcd Horizontal_weight (K*L);   //// for pi

		double F_theta_temp_real = 0.;
		double F_theta_temp_imag = 0.;
		double F_pi_temp_real = 0.;
		double F_pi_temp_imag = 0.;

		/// theta, Vertical
		for (int k = 0; k < K; k++)
		{
			for (int l = 0; l < L; l++)
			{
				VectorXcd wei(1, 1);

				//wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)k * dV * cos(tilt_z));
				///wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)l * dH * sin(tilt_z)*cos(tilt_a));
				wei(0, 0) = 0.;

				MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, dV * k);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

				VectorXcd ant_co(1, 1);

				ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

				F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));  
				F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

				F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
				F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));


				//cout << "F_theta_temp_real = " << F_theta_temp_real << endl;
				//cout << "F_theta_temp_imag = " << F_theta_temp_imag << endl;
				//cout << "F_pi_temp_real = " << F_pi_temp_real << endl;
				//cout << "F_pi_temp_imag = " << F_pi_temp_imag << endl;

				//getchar();

			}

		}



		/// F, antenna gain for RSRP
		complex <double> F_tx_theta = (F_theta_temp_real, F_theta_temp_imag);
		complex <double> F_tx_pi = (F_pi_temp_real, F_pi_temp_imag);


		//TransmitterAntennaGainXLOS_theta = F_tx_theta;
		//TransmitterAntennaGainXLOS_pi = F_tx_pi;

		//// 1to1 mapping ̷ ص ҵ
		TransmitterAntennaGainXLOS_theta = F_theta_GCS;
		TransmitterAntennaGainXLOS_pi = F_pi_GCS;
		*/





		///// ̰ 0 ° ߸Ȱ................................
		//cout << TransmitterAntennaGainXLOS_pi << endl;
		//cout << TransmitterAntennaGainXLOS_theta << endl;
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////







		return combined_antenna_gain;

		

	}
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	else if (TYPE == 12)   ////// Dense Urban 
	{
		
		/*
		double ms_height_in_channel = 0;
		ms_height_in_channel = ms[self_ms_idx].MS_HEIGHT_FINAL;

		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));
		double h_angle_pi = Get_angle(interferer, victim, vector);  //// pi in GCS
		double v_angle_theta = ((atan2((bs_height - ms_height_in_channel), distance)*180. / pi) + 90.);  ///// theta in GCS


		double h_antgain = -MIN(12. * (h_angle_pi / 65.)*(h_angle_pi / 65.), 30.);
		double v_antgain = -MIN(12. * ((v_angle_theta - 90.) / 65.) * ((v_angle_theta - 90.) / 65.), 30.);

		return max_antgain - MIN(-(h_antgain + v_antgain), 30.);
		*/

		///////////////////////////////////////////////
		///////////////////////////////////////////////   GCS-> LCS test //////////////////////////////////////////
		///////////////////////////////////////////////

		
		double ms_height_in_channel = 0;
		ms_height_in_channel = ms[self_ms_idx].MS_HEIGHT_FINAL;

		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));
		//double h_angle_pi = Get_angle(interferer, victim, vector) * (pi / 180.);  //// pi in GCS
		//double v_angle_theta = ((atan2((bs_height - ms_height_in_channel), distance)*180. / pi) + 90.) * (pi / 180.);  ///// theta in GCS
		double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOD_GCS * (pi / 180.);  //// pi in GCS
		double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOD_GCS * (pi / 180.);  ///// theta in GCS


		double v_angle_theta_LCS = 0.;
		double h_angle_pi_LCS = 0.;

		double v_angle_theta_LCS_temp = 0.;
		double h_angle_pi_LCS_temp = 0.;

		/*
		/////  GCS -> LCS with mechanical down tile angle 
		/////  TR 36.814 / A.2.1.6.2 / equation (1),(2)
		double mechanical_down_tilt_beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		v_angle_theta_LCS = acos(cos(h_angle_pi)*sin(v_angle_theta)*sin(mechanical_down_tilt_beta) + cos(v_angle_theta)*cos(mechanical_down_tilt_beta));
		complex <double> h_angle_pi_LCS_complex(cos(h_angle_pi)*sin(v_angle_theta)*cos(mechanical_down_tilt_beta) - cos(v_angle_theta)*sin(mechanical_down_tilt_beta), sin(h_angle_pi)*sin(v_angle_theta));
		h_angle_pi_LCS = arg(h_angle_pi_LCS_complex);
		*/

		/////GCS -> LCS , use alpha, beta, gamma
		///// TR 36.873
		double alpha;
		//double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
		double gamma = 0.;
		if (sector_idx == 1)
		{
			alpha = 30. * (pi / 180.);
		}
		else if (sector_idx == 2)
		{
			alpha = 150. * (pi / 180.);
		}
		else
		{
			alpha = -90. * (pi / 180.);
			//alpha = 270. * (pi / 180.);
		}
		double zeta_1 = -pi / 4;
		double zeta_2 = pi / 4;

		v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
		h_angle_pi_LCS_temp    = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

		//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
		//h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);


		double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 65.)*(h_angle_pi_LCS_temp * (180. / pi) / 65.), 30.);
		double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.) * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.), 30.);

		double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 30.);


		/// Polarized antennas Model-2
		/// 36.873 -> 7.1.1
		double A= sqrt( pow(10.0, (combined_antenna_gain / 10.0) ) );

		double F_theta_GCS = A * cos(zeta_2);
		double F_pi_GCS    = A * sin(zeta_2);
		//// -> F_tx_

		//double B = randnum.n();


		//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
		//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
		//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

		/// number of TXRU per TRxP
		/*
		double Mp = 2;
		double Np = 1;
		double P = 2;
		double Mg = 1;
		double Ng = 1;
		

		double dH = 0.5 * Wavelength;
		double dV = 0.8 * Wavelength;

		double K = BS_M / Mp;    /////   8/2 = 4
		double L = BS_N / Np;   //////   8/1 = 8

		complex <double>jay(0, 1);


		//VectorXcd Vertical_weight   (K*L);   //// for theta
		//VectorXcd Horizontal_weight (K*L);   //// for pi

		double F_theta_temp_real = 0.;
		double F_theta_temp_imag = 0.;
		double F_pi_temp_real = 0.;
		double F_pi_temp_imag = 0.;
		

		/// theta, Vertical
		/// pi, Horizontal
		for (int k = 0; k < K; k++)
		{
			for (int l = 0; l < L; l++)
			{

				VectorXcd K_wei(1, 1);
				VectorXcd L_wei(1, 1);

				K_wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)k * dV * cos(tilt_z));
				L_wei(0, 0) = (-1.0 * jay * (2. * pi * (1 / Wavelength) * (double)l * dH * sin(tilt_z)*cos(tilt_a)));

				//MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);
				MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, dV * k);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

				VectorXcd ant_co(1, 1);

				ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

				F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(K_wei(0, 0)) * exp(ant_co(0, 0))));
				F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(K_wei(0, 0)) * exp(ant_co(0, 0))));

				F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(L_wei(0, 0)) * exp(ant_co(0, 0))));
				F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(L_wei(0, 0)) * exp(ant_co(0, 0))));

			}
		}



		/// F, antenna gain for RSRP
		complex <double> F_tx_theta = (F_theta_temp_real, F_theta_temp_imag);
		complex <double> F_tx_pi = (F_pi_temp_real, F_pi_temp_imag);


		TransmitterAntennaGainXLOS_theta = F_tx_theta;
		TransmitterAntennaGainXLOS_pi = F_tx_pi;
		*/
		
		////////// tilt  ʿ ...ۿ ޵..................................................................... 


		/*
		BS_N  N_H
		BS_M  N_V
		*/





		/*
		VectorXcd W[12];

		for (int size = 0; size < 12; size++)
		{
			W[size].resize(BS_M*BS_N);
		}

		


		for (int m = 0; m < BS_N; m++)
		{
			for (int n = 0; n < BS_M; n++)
			{
				int vector_idx = m * BS_M + n;
				V(vector_idx) = exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS) + (m + 1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS)*sin(h_angle_pi_LCS)));
			}
		}


		for (int pi_idx = 0; pi_idx < tilt_azimuth_angle_LCS.size(); pi_idx++)
		{
			for (int theta_idx = 0; theta_idx < tilt_zenith_angle_LCS.size(); theta_idx++)
			{
				for (int m = 0; m < BS_N; m++)
				{
					for (int n = 0; n < BS_M; n++)
					{
						int weighting_factor_idx = pi_idx * tilt_zenith_angle_LCS.size() + theta_idx;

						int vector_idx = m * BS_M + n;

						W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
						//W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*(-1*(n+1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
					}
				}
			}
		}

		double array_factor[12];
		double max_array_factor = 0;

		for (int weighting_factor_idx = 0; weighting_factor_idx < 12; weighting_factor_idx++)
		{
			complex <double> W_Tilde = V.transpose() * W[weighting_factor_idx];


			array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));

			link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];


			if (weighting_factor_idx == 0)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else if (array_factor[weighting_factor_idx] > max_array_factor)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else
			{
				max_array_factor = max_array_factor;
			}
			
			cout << "weighting_factor_idx = " << weighting_factor_idx << endl;
			cout << "W_Tilde = " << W_Tilde << endl;
			cout << "array_factor[weighting_factor_idx] = " << array_factor[weighting_factor_idx] << endl;
			cout << "max_array_factor = " << max_array_factor << endl;
			getchar();
			
		}
		*/
		

		return combined_antenna_gain;
		
		///////////////////////////////////////////////
		///////////////////////////////////////////////   GCS-> LCS test //////////////////////////////////////////
		///////////////////////////////////////////////

	}
		else if (TYPE == 13)   ////// Rural
		{

			/*
			double ms_height_in_channel = 0;
			ms_height_in_channel = ms[self_ms_idx].MS_HEIGHT_FINAL;

			double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));
			double h_angle_pi = Get_angle(interferer, victim, vector);  //// pi in GCS
			double v_angle_theta = ((atan2((bs_height - ms_height_in_channel), distance)*180. / pi) + 90.);  ///// theta in GCS


			double h_antgain = -MIN(12. * (h_angle_pi / 65.)*(h_angle_pi / 65.), 30.);
			double v_antgain = -MIN(12. * ((v_angle_theta - 90.) / 65.) * ((v_angle_theta - 90.) / 65.), 30.);

			return max_antgain - MIN(-(h_antgain + v_antgain), 30.);
			*/

			///////////////////////////////////////////////
			///////////////////////////////////////////////   GCS-> LCS test //////////////////////////////////////////
			///////////////////////////////////////////////
			///////////////////////////////////////////////
			///////////////////////////////////////////////   GCS-> LCS test //////////////////////////////////////////
			///////////////////////////////////////////////


			double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));
			//double h_angle_pi = 0.;  //// pi in GCS
			//double v_angle_theta = ((atan2((bs_height - ms_height), distance)*180. / pi) + 90.) * (pi / 180.);  ///// theta in GCS
			double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOD_GCS * (pi / 180.);  //// pi in GCS
			double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOD_GCS * (pi / 180.);  ///// theta in GCS

			double v_angle_theta_LCS_temp = 0.;
			double h_angle_pi_LCS_temp = 0.;

			double v_angle_theta_LCS = 0.;
			double h_angle_pi_LCS = 0.;

			/*
			/////  GCS -> LCS with mechanical down tile angle
			/////  TR 36.814 / A.2.1.6.2 / equation (1),(2)
			double mechanical_down_tilt_beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
			v_angle_theta_LCS = acos(cos(h_angle_pi)*sin(v_angle_theta)*sin(mechanical_down_tilt_beta) + cos(v_angle_theta)*cos(mechanical_down_tilt_beta));
			complex <double> h_angle_pi_LCS_complex(cos(h_angle_pi)*sin(v_angle_theta)*cos(mechanical_down_tilt_beta) - cos(v_angle_theta)*sin(mechanical_down_tilt_beta), sin(h_angle_pi)*sin(v_angle_theta));
			h_angle_pi_LCS = arg(h_angle_pi_LCS_complex);
			*/

			/////GCS -> LCS , use alpha, beta, gamma
			///// TR 36.873
			double alpha;
			//double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
			double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
			double gamma = 0.;
			if (sector_idx == 1)
			{
				alpha = 30. * (pi / 180.);
			}
			else if (sector_idx == 2)
			{
				alpha = 150. * (pi / 180.);
			}
			else
			{
				alpha = -90. * (pi / 180.);
			}
			double zeta_1 = -pi / 4;
			double zeta_2 = pi / 4;

			v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
			h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

			//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
			//h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);


			double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 65.)*(h_angle_pi_LCS_temp * (180. / pi) / 65.), 30.);
			//double h_antgain = 0.;
			double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.) * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.), 30.);

			double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 30.);


			/// Polarized antennas Model-2
			/// 36.873 -> 7.1.1
			double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));



			double F_theta_GCS = A * cos(zeta_2);
			double F_pi_GCS = A * sin(zeta_2);
			//// -> F_tx_


			return combined_antenna_gain;


		}

	else
	{
		return 0;
	}

}


double LINK::Get_arrayfactor(LOCATION interferer, LOCATION victim, int _bs_idx , int sector_idx)
{
	// interferer = BS
	// victim = MS

	LOCATION vector;

	vector.x = interferer.x + cos((30 + 120 * (sector_idx - 1))*pi / 180.);
	vector.y = interferer.y + sin((30 + 120 * (sector_idx - 1))*pi / 180.);

	if (TYPE == 11)   /// Indoor
	{
		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));



		double h_angle_pi = 0.;    ///// NO 3 sector
		//double h_angle_pi = Get_angle(interferer, victim, vector) * (pi / 180.);  //// pi in GCS
		double v_angle_theta = ((atan2((bs_height - ms_height), distance)*180. / pi) + 90.) * (pi / 180.);  ///// theta in GCS


		double mechanical_down_tilt_beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		double v_angle_theta_LCS = 0.;
		double h_angle_pi_LCS = 0.;


		/////  GCS -> LCS with mechanical down tile angle
		/////  TR 36.814 / A.2.1.6.2 / equation (1),(2)

		v_angle_theta_LCS = acos(cos(h_angle_pi)*sin(v_angle_theta)*sin(mechanical_down_tilt_beta) + cos(v_angle_theta)*cos(mechanical_down_tilt_beta));

		complex <double> h_angle_pi_LCS_complex(cos(h_angle_pi)*sin(v_angle_theta)*cos(mechanical_down_tilt_beta) - cos(v_angle_theta)*sin(mechanical_down_tilt_beta), sin(h_angle_pi)*sin(v_angle_theta));
		h_angle_pi_LCS = arg(h_angle_pi_LCS_complex);



		//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
		//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
		//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

		/// number of TXRU per TRxP
		/*
		double Mp = 2;
		double Np = 1;
		double P = 2;
		double Mg = 1;
		double Ng = 1;
		*/

		double dH = 0.5 * Wavelength;
		double dV = 0.5 * Wavelength;

		//double K = BS_M / Mp;    /////   8/2 = 4
		//double L = BS_N / Np;   //////   8/1 = 8

		complex <double>jay(0, 1);


		VectorXd azimuth_angle_LCS(1);
		VectorXd zenith_angle_LCS(1);

		azimuth_angle_LCS << 0;
		zenith_angle_LCS << 90 * (pi/180.);


		VectorXcd V(BS_M*BS_N);
		VectorXcd W(BS_M*BS_N);




		/*
		BS_N  N_H
		BS_M  N_V
		*/


		for (int m = 0; m < BS_N; m++)
		{
			for (int n = 0; n < BS_M; n++)
			{
				int vector_idx = m * BS_M + n;
				V(vector_idx) = exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS) + (m + 1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS)*sin(h_angle_pi_LCS)));

				cout << "m = " << m << endl;
				cout << "n = " << n << endl;
				cout << "cos(v_angle_theta_LCS) = " << cos(v_angle_theta_LCS) << endl;
				cout << "sin(v_angle_theta_LCS) = " << sin(v_angle_theta_LCS) << endl;
				cout << "sin(h_angle_pi_LCS) = " << sin(h_angle_pi_LCS) << endl;
				cout << "V(vector_idx) = " << V(vector_idx) << endl;
				getchar();
			}
		}



		for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
		{
			for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
			{
				for (int m = 0; m < BS_N; m++)
				{
					for (int n = 0; n < BS_M; n++)
					{
						int vector_idx = m * BS_M + n;

						W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));

						//W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*(-1*(n+1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));

					}
				}
			}
		}

		double array_factor[12];
		double max_array_factor = 0;

		for (int weighting_factor_idx = 0; weighting_factor_idx < 12; weighting_factor_idx++)
		{
			complex <double> W_Tilde = V.transpose() * W;


			array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));

			link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];


			if (weighting_factor_idx == 0)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else if (array_factor[weighting_factor_idx] > max_array_factor)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else
			{
				max_array_factor = max_array_factor;
			}

			/*
			cout << "weighting_factor_idx = " << weighting_factor_idx << endl;
			cout << "W_Tilde = " << W_Tilde << endl;
			cout << "array_factor[weighting_factor_idx] = " << array_factor[weighting_factor_idx] << endl;
			cout << "max_array_factor = " << max_array_factor << endl;
			getchar();
			*/
		}



		return max_array_factor;   /////  max array factor return

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	}
	else if (TYPE == 12 || TYPE == 13)   ////// Dense Urban & Rural
	{


		///////////////////////////////////////////////
		///////////////////////////////////////////////   GCS-> LCS //////////////////////////////////////////
		///////////////////////////////////////////////
		/*
		double v_angle_theta_LCS = 0.;
		double h_angle_pi_LCS = 0.;
		double v_angle_theta_LCS_temp = 0.;
		double h_angle_pi_LCS_temp = 0.;
		
		double ms_height_in_channel = 0;
		ms_height_in_channel = ms[self_ms_idx].MS_HEIGHT_FINAL;

		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));

		double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOD_GCS * (pi / 180.);  //// pi in GCS
		double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOD_GCS * (pi / 180.);  ///// theta in GCS

		/////GCS -> LCS , use alpha, beta, gamma
		///// TR 36.873
		double alpha;
		//double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
		double gamma = 0.;
		if (sector_idx == 1)
		{
			alpha = 30. * (pi / 180.);
		}
		else if (sector_idx == 2)
		{
			alpha = 150. * (pi / 180.);
		}
		else
		{
			alpha = -90. * (pi / 180.);
		}
		double zeta_1 = -pi / 4;
		double zeta_2 = pi / 4;

		v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
		h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);


		double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 65.)*(h_angle_pi_LCS_temp * (180. / pi) / 65.), 30.);
		double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.) * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.), 30.);
		*/



		//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
		//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
		//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

		/// number of TXRU per TRxP
		/*
		double Mp = 2;
		double Np = 1;
		double P = 2;
		double Mg = 1;
		double Ng = 1;
		*/

		double dH = 0.5 * Wavelength;
		double dV = 0.8 * Wavelength;

		//double K = BS_M / Mp;    /////   8/2 = 4
		//double L = BS_N / Np;   //////   8/1 = 8

		complex <double>jay(0, 1);


		VectorXd azimuth_angle_LCS(6);
		VectorXd zenith_angle_LCS(2);

		azimuth_angle_LCS << -5 * pi / 16, -3 * pi / 16, -pi / 16, pi / 16, 3 * pi / 16, 5 * pi / 16;
		zenith_angle_LCS << 5 * pi / 8, 7 * pi / 8;


		VectorXcd V(BS_M*BS_N);
		//VectorXcd W(BS_M*BS_N);
		VectorXcd W[12];

		for (int size = 0; size < 12; size++)
		{
			W[size].resize(BS_M*BS_N);
		}

		/*
		BS_N  N_H
		BS_M  N_V
		*/


		for (int m = 0; m < BS_N; m++)
		{
			for (int n = 0; n < BS_M; n++)
			{
				int vector_idx = m * BS_M + n;
				//V(vector_idx) = exp(jay * 2. * pi*((n+1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS_temp) + (m+1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS_temp)*sin(h_angle_pi_LCS_temp)));
				V(vector_idx) = 1;
			}
		}


		for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
		{
			for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
			{
				for (int m = 0; m < BS_N; m++)
				{
					for (int n = 0; n < BS_M; n++)
					{
						int weighting_factor_idx = pi_idx * zenith_angle_LCS.size() + theta_idx;

						int vector_idx = m * BS_M + n;

						//W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n+1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
						W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*(-1*(n+1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
					}
				}
			}
		}
		
		double array_factor[12];
		double max_array_factor = 0;

		for (int weighting_factor_idx = 0; weighting_factor_idx < 12; weighting_factor_idx++)
		{
			complex <double> W_Tilde = V.transpose() * W[weighting_factor_idx];


			array_factor[weighting_factor_idx] = 10*log10( pow(abs(W_Tilde),2) );

			link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];


			if (weighting_factor_idx == 0)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else if (array_factor[weighting_factor_idx] > max_array_factor)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else
			{
				max_array_factor = max_array_factor;
			}
			/*
			cout << "weighting_factor_idx = " << weighting_factor_idx << endl;
			cout << "W_Tilde = " << W_Tilde << endl;
			cout << "array_factor[weighting_factor_idx] = " << array_factor[weighting_factor_idx] << endl;
			cout << "max_array_factor = " << max_array_factor << endl;
			getchar();
			*/
		}



		return max_array_factor;   /////  max array factor return

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	}

	else
	{
		return 0;
	}

}

double LINK::Get_random_arrayfactor(LOCATION interferer, LOCATION victim, int _bs_idx, int sector_idx)
{
	// interferer = BS
	// victim = MS

	LOCATION vector;

	vector.x = interferer.x + cos((30 + 120 * (sector_idx - 1))*pi / 180.);
	vector.y = interferer.y + sin((30 + 120 * (sector_idx - 1))*pi / 180.);

	if (TYPE == 11)   /// Indoor
	{
		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));



		double h_angle_pi = 0.;    ///// NO 3 sector
		//double h_angle_pi = Get_angle(interferer, victim, vector) * (pi / 180.);  //// pi in GCS
		double v_angle_theta = ((atan2((bs_height - ms_height), distance)*180. / pi) + 90.) * (pi / 180.);  ///// theta in GCS


		double mechanical_down_tilt_beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		double v_angle_theta_LCS = 0.;
		double h_angle_pi_LCS = 0.;


		/////  GCS -> LCS with mechanical down tile angle
		/////  TR 36.814 / A.2.1.6.2 / equation (1),(2)

		v_angle_theta_LCS = acos(cos(h_angle_pi)*sin(v_angle_theta)*sin(mechanical_down_tilt_beta) + cos(v_angle_theta)*cos(mechanical_down_tilt_beta));

		complex <double> h_angle_pi_LCS_complex(cos(h_angle_pi)*sin(v_angle_theta)*cos(mechanical_down_tilt_beta) - cos(v_angle_theta)*sin(mechanical_down_tilt_beta), sin(h_angle_pi)*sin(v_angle_theta));
		h_angle_pi_LCS = arg(h_angle_pi_LCS_complex);



		//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
		//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
		//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

		/// number of TXRU per TRxP
		/*
		double Mp = 2;
		double Np = 1;
		double P = 2;
		double Mg = 1;
		double Ng = 1;
		*/

		double dH = 0.5 * Wavelength;
		double dV = 0.5 * Wavelength;

		//double K = BS_M / Mp;    /////   8/2 = 4
		//double L = BS_N / Np;   //////   8/1 = 8

		complex <double>jay(0, 1);


		VectorXd azimuth_angle_LCS(1);
		VectorXd zenith_angle_LCS(1);

		azimuth_angle_LCS << 0;
		zenith_angle_LCS << 90 * (pi / 180.);


		VectorXcd V(BS_M*BS_N);
		VectorXcd W(BS_M*BS_N);




		/*
		BS_N  N_H
		BS_M  N_V
		*/


		for (int m = 0; m < BS_N; m++)
		{
			for (int n = 0; n < BS_M; n++)
			{
				int vector_idx = m * BS_M + n;
				V(vector_idx) = exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS) + (m + 1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS)*sin(h_angle_pi_LCS)));

				cout << "m = " << m << endl;
				cout << "n = " << n << endl;
				cout << "cos(v_angle_theta_LCS) = " << cos(v_angle_theta_LCS) << endl;
				cout << "sin(v_angle_theta_LCS) = " << sin(v_angle_theta_LCS) << endl;
				cout << "sin(h_angle_pi_LCS) = " << sin(h_angle_pi_LCS) << endl;
				cout << "V(vector_idx) = " << V(vector_idx) << endl;
				getchar();
			}
		}



		for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
		{
			for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
			{
				for (int m = 0; m < BS_N; m++)
				{
					for (int n = 0; n < BS_M; n++)
					{
						int vector_idx = m * BS_M + n;

						W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));

						//W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*(-1*(n+1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));

					}
				}
			}
		}

		double array_factor[12];
		double max_array_factor = 0;

		for (int weighting_factor_idx = 0; weighting_factor_idx < 12; weighting_factor_idx++)
		{
			complex <double> W_Tilde = V.transpose() * W;


			array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));

			link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];


			if (weighting_factor_idx == 0)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else if (array_factor[weighting_factor_idx] > max_array_factor)
			{
				max_array_factor = array_factor[weighting_factor_idx];
			}
			else
			{
				max_array_factor = max_array_factor;
			}

			/*
			cout << "weighting_factor_idx = " << weighting_factor_idx << endl;
			cout << "W_Tilde = " << W_Tilde << endl;
			cout << "array_factor[weighting_factor_idx] = " << array_factor[weighting_factor_idx] << endl;
			cout << "max_array_factor = " << max_array_factor << endl;
			getchar();
			*/
		}



		return max_array_factor;   /////  max array factor return

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	}
	else if (TYPE == 12 || TYPE == 13)   ////// Dense Urban & Rural
	{


		///////////////////////////////////////////////
		///////////////////////////////////////////////   GCS-> LCS //////////////////////////////////////////
		///////////////////////////////////////////////
		/*
		double v_angle_theta_LCS = 0.;
		double h_angle_pi_LCS = 0.;
		double v_angle_theta_LCS_temp = 0.;
		double h_angle_pi_LCS_temp = 0.;

		double ms_height_in_channel = 0;
		ms_height_in_channel = ms[self_ms_idx].MS_HEIGHT_FINAL;

		double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));

		double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOD_GCS * (pi / 180.);  //// pi in GCS
		double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOD_GCS * (pi / 180.);  ///// theta in GCS

		/////GCS -> LCS , use alpha, beta, gamma
		///// TR 36.873
		double alpha;
		//double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
		double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
		double gamma = 0.;
		if (sector_idx == 1)
		{
			alpha = 30. * (pi / 180.);
		}
		else if (sector_idx == 2)
		{
			alpha = 150. * (pi / 180.);
		}
		else
		{
			alpha = -90. * (pi / 180.);
		}
		double zeta_1 = -pi / 4;
		double zeta_2 = pi / 4;

		v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
		h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

		//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
		//h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);


		double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 65.)*(h_angle_pi_LCS_temp * (180. / pi) / 65.), 30.);
		double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.) * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.), 30.);
		*/



		//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
		//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
		//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

		/// number of TXRU per TRxP
		/*
		double Mp = 2;
		double Np = 1;
		double P = 2;
		double Mg = 1;
		double Ng = 1;
		*/

		double dH = 0.5 * Wavelength;
		double dV = 0.8 * Wavelength;

		//double K = BS_M / Mp;    /////   8/2 = 4
		//double L = BS_N / Np;   //////   8/1 = 8

		complex <double>jay(0, 1);


		VectorXd azimuth_angle_LCS(6);
		VectorXd zenith_angle_LCS(2);

		azimuth_angle_LCS << -5 * pi / 16, -3 * pi / 16, -pi / 16, pi / 16, 3 * pi / 16, 5 * pi / 16;
		zenith_angle_LCS << 5 * pi / 8, 7 * pi / 8;


		VectorXcd V(BS_M*BS_N);
		//VectorXcd W(BS_M*BS_N);
		VectorXcd W[12];

		for (int size = 0; size < 12; size++)
		{
			W[size].resize(BS_M*BS_N);
		}

		/*
		BS_N  N_H
		BS_M  N_V
		*/


		for (int m = 0; m < BS_N; m++)
		{
			for (int n = 0; n < BS_M; n++)
			{
				int vector_idx = m * BS_M + n;
				//V(vector_idx) = exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS_temp) + (m + 1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS_temp)*sin(h_angle_pi_LCS_temp)));
				V(vector_idx) = 1;
			}
		}


		for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
		{
			for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
			{
				for (int m = 0; m < BS_N; m++)
				{
					for (int n = 0; n < BS_M; n++)
					{
						int weighting_factor_idx = pi_idx * zenith_angle_LCS.size() + theta_idx;

						int vector_idx = m * BS_M + n;

						//W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
						W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*(-1*(n+1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
					}
				}
			}
		}

		double array_factor[12];
		double random_array_factor = 0;

		int random_number = (int)12 * randnum.u();

		for (int weighting_factor_idx = 0; weighting_factor_idx < 12; weighting_factor_idx++)
		{
			complex <double> W_Tilde = V.transpose() * W[weighting_factor_idx];


			array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));

			link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];

			//cout << "weighting_factor_idx = " << weighting_factor_idx << endl;
			//cout << "random number = " << random_number << endl;
			//getchar();

			if (weighting_factor_idx == random_number)
			{
				random_array_factor = array_factor[weighting_factor_idx];
			}

			/*
			cout << "weighting_factor_idx = " << weighting_factor_idx << endl;
			cout << "W_Tilde = " << W_Tilde << endl;
			cout << "array_factor[weighting_factor_idx] = " << array_factor[weighting_factor_idx] << endl;
			cout << "max_array_factor = " << max_array_factor << endl;
			getchar();
			*/
		}



		return random_array_factor;   /////  max array factor return

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	}

	else
	{
		return 0;
	}

}

double LINK::Get_MS_arrayfactor(LOCATION interferer, LOCATION victim, int _bs_idx, int sector_idx)
{
	if (Frequency < 6000000000) /// Below 6GHz
	{
		return 0;
	}
	else    //// above 6GHz
	{

		if (TYPE == 11)   /// Indoor
		{
			double dH = 0.5 * Wavelength;
			double dV = 0.5 * Wavelength;

			complex <double>jay(0, 1);

			VectorXd azimuth_angle_LCS(1);
			VectorXd zenith_angle_LCS(1);

			azimuth_angle_LCS << 0;
			zenith_angle_LCS << 90 * (pi / 180.);

			VectorXcd V(BS_M*BS_N);
			VectorXcd W(BS_M*BS_N);

			for (int m = 0; m < BS_N; m++)
			{
				for (int n = 0; n < BS_M; n++)
				{
					int vector_idx = m * BS_M + n;
					//V(vector_idx) = exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS) + (m + 1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS)*sin(h_angle_pi_LCS)));
					V(vector_idx) = 1;

				}
			}

			for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
			{
				for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
				{
					for (int m = 0; m < BS_N; m++)
					{
						for (int n = 0; n < BS_M; n++)
						{
							int vector_idx = m * BS_M + n;

							//W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
							W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*(-1 * (n + 1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));

						}
					}
				}
			}

			double array_factor[12];
			double max_array_factor = 0;

			for (int weighting_factor_idx = 0; weighting_factor_idx < 12; weighting_factor_idx++)
			{
				complex <double> W_Tilde = V.transpose() * W;
				array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));
				link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];

				if (weighting_factor_idx == 0)
				{
					max_array_factor = array_factor[weighting_factor_idx];
				}
				else if (array_factor[weighting_factor_idx] > max_array_factor)
				{
					max_array_factor = array_factor[weighting_factor_idx];
				}
				else
				{
					max_array_factor = max_array_factor;
				}
			}

			return max_array_factor;   /////  max array factor return

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		}
		else if (TYPE == 12)   ////// Dense Urban & Rural
		{
			double dH = 0.5 * Wavelength;
			double dV = 0.5 * Wavelength;

			complex <double>jay(0, 1);

			VectorXd azimuth_angle_LCS(4);
			VectorXd zenith_angle_LCS(2);

			azimuth_angle_LCS << -3 * pi / 8, -pi / 8, pi / 8, 3 * pi / 8;
			zenith_angle_LCS << pi / 4, 3 * pi / 4;

			VectorXcd V(MS_M*MS_N);
			VectorXcd W[8];

			for (int size = 0; size < 8; size++)
			{
				W[size].resize(MS_M*MS_N);
			}

			for (int m = 0; m < MS_N; m++)
			{
				for (int n = 0; n < MS_M; n++)
				{
					int vector_idx = m * MS_M + n;
					//V(vector_idx) = exp(jay * 2. * pi*((n+1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS_temp) + (m+1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS_temp)*sin(h_angle_pi_LCS_temp)));
					V(vector_idx) = 1;
				}
			}

			for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
			{
				for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
				{
					for (int m = 0; m < MS_N; m++)
					{
						for (int n = 0; n < MS_M; n++)
						{
							int weighting_factor_idx = pi_idx * zenith_angle_LCS.size() + theta_idx;

							int vector_idx = m * MS_M + n;

							//W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n+1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
							W[weighting_factor_idx](vector_idx) = (1 / sqrt(MS_N*MS_M)) * exp(jay * 2. * pi*(-1 * (n + 1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
						}
					}
				}
			}

			double array_factor[8];
			double max_array_factor = 0;

			for (int weighting_factor_idx = 0; weighting_factor_idx < 8; weighting_factor_idx++)
			{
				complex <double> W_Tilde = V.transpose() * W[weighting_factor_idx];
				array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));
				link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];

				if (weighting_factor_idx == 0)
				{
					max_array_factor = array_factor[weighting_factor_idx];
				}
				else if (array_factor[weighting_factor_idx] > max_array_factor)
				{
					max_array_factor = array_factor[weighting_factor_idx];
				}
				else
				{
					max_array_factor = max_array_factor;
				}
			}
			return max_array_factor;   /////  max array factor return

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


		}

		else
		{
			return 0;
		}
	}
}

double LINK::Get_random_MS_arrayfactor(LOCATION interferer, LOCATION victim, int _bs_idx, int sector_idx)
{
	if (Frequency < 6000000000) /// Below 6GHz
	{
		return 0;
	}
	else    //// above 6GHz
	{
		if (TYPE == 11)   /// Indoor
		{
			double dH = 0.5 * Wavelength;
			double dV = 0.5 * Wavelength;

			complex <double>jay(0, 1);
			VectorXd azimuth_angle_LCS(1);
			VectorXd zenith_angle_LCS(1);

			azimuth_angle_LCS << 0;
			zenith_angle_LCS << 90 * (pi / 180.);

			VectorXcd V(BS_M*BS_N);
			VectorXcd W(BS_M*BS_N);

			for (int m = 0; m < BS_N; m++)
			{
				for (int n = 0; n < BS_M; n++)
				{
					int vector_idx = m * BS_M + n;
					//V(vector_idx) = exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS) + (m + 1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS)*sin(h_angle_pi_LCS)));
					V(vector_idx) = 1;
				}
			}



			for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
			{
				for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
				{
					for (int m = 0; m < BS_N; m++)
					{
						for (int n = 0; n < BS_M; n++)
						{
							int vector_idx = m * BS_M + n;

							//W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n + 1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));

							W(vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*(-1 * (n + 1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));

						}
					}
				}
			}

			double array_factor[12];
			double max_array_factor = 0;

			for (int weighting_factor_idx = 0; weighting_factor_idx < 12; weighting_factor_idx++)
			{
				complex <double> W_Tilde = V.transpose() * W;


				array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));

				link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];


				if (weighting_factor_idx == 0)
				{
					max_array_factor = array_factor[weighting_factor_idx];
				}
				else if (array_factor[weighting_factor_idx] > max_array_factor)
				{
					max_array_factor = array_factor[weighting_factor_idx];
				}
				else
				{
					max_array_factor = max_array_factor;
				}
			}

			return max_array_factor;   /////  max array factor return

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		}
		else if (TYPE == 12)   ////// Dense Urban & Rural
		{

			double dH = 0.5 * Wavelength;
			double dV = 0.5 * Wavelength;

			complex <double>jay(0, 1);

			VectorXd azimuth_angle_LCS(4);
			VectorXd zenith_angle_LCS(2);

			azimuth_angle_LCS << -3 * pi / 8, -pi / 8, pi / 8, 3 * pi / 8;
			zenith_angle_LCS << pi / 4, 3 * pi / 4;

			VectorXcd V(MS_M*MS_N);
			VectorXcd W[8];
			for (int size = 0; size < 8; size++)
			{
				W[size].resize(MS_M*MS_N);
			}

			for (int m = 0; m < MS_N; m++)
			{
				for (int n = 0; n < MS_M; n++)
				{
					int vector_idx = m * MS_M + n;
					//V(vector_idx) = exp(jay * 2. * pi*((n+1 - 1)*(dV / Wavelength)*cos(v_angle_theta_LCS_temp) + (m+1 - 1)*(dH / Wavelength)*sin(v_angle_theta_LCS_temp)*sin(h_angle_pi_LCS_temp)));
					V(vector_idx) = 1;
				}
			}


			for (int pi_idx = 0; pi_idx < azimuth_angle_LCS.size(); pi_idx++)
			{
				for (int theta_idx = 0; theta_idx < zenith_angle_LCS.size(); theta_idx++)
				{
					for (int m = 0; m < MS_N; m++)
					{
						for (int n = 0; n < MS_M; n++)
						{
							int weighting_factor_idx = pi_idx * zenith_angle_LCS.size() + theta_idx;

							int vector_idx = m * MS_M + n;

							//W[weighting_factor_idx](vector_idx) = (1 / sqrt(BS_N*BS_M)) * exp(jay * 2. * pi*((n+1 - 1)*(dV / Wavelength)*sin(zenith_angle_LCS(theta_idx)) - (m+1 - 1)*(dH / Wavelength)*cos(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
							W[weighting_factor_idx](vector_idx) = (1 / sqrt(MS_N*MS_M)) * exp(jay * 2. * pi*(-1 * (n + 1 - 1)*(dV / Wavelength)*cos(zenith_angle_LCS(theta_idx)) - (m + 1 - 1)*(dH / Wavelength)*sin(zenith_angle_LCS(theta_idx))*sin(azimuth_angle_LCS(pi_idx))));
						}
					}
				}
			}

			double array_factor[8];
			double random_array_factor = 0;
			int random_number = (int)8 * randnum.u();

			for (int weighting_factor_idx = 0; weighting_factor_idx < 8; weighting_factor_idx++)
			{
				complex <double> W_Tilde = V.transpose() * W[weighting_factor_idx];
				array_factor[weighting_factor_idx] = 10 * log10(pow(abs(W_Tilde), 2));
				link_array_factor[weighting_factor_idx] = array_factor[weighting_factor_idx];

				if (weighting_factor_idx == random_number)
				{
					random_array_factor = array_factor[weighting_factor_idx];
				}
			}

			return random_array_factor;   /////  max array factor return

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		}

		else
		{
			return 0;
		}
	}
}



double LINK::Get_MS_antgain(LOCATION interferer, LOCATION victim, int _bs_idx, int sector_idx)
{
	if (Frequency < 6000000000)  		///below 6GHz
	{
		return 0;
	}
	else   ////// above 6GHz
	{
		LOCATION vector;

		vector.x = interferer.x + cos((30 + 120 * (sector_idx - 1))*pi / 180.);
		vector.y = interferer.y + sin((30 + 120 * (sector_idx - 1))*pi / 180.);

		if (TYPE == 11)   /// Indoor
		{


			//double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));

			double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOA_GCS * (pi / 180.);  //// pi in GCS
			double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOA_GCS * (pi / 180.);  ///// theta in GCS

			double v_angle_theta_LCS = 0.;
			double h_angle_pi_LCS = 0.;

			double v_angle_theta_LCS_temp = 0.;
			double h_angle_pi_LCS_temp = 0.;


			/////GCS -> LCS , use alpha, beta, gamma
			///// TR 36.873
			double alpha = 360 * randnum.u() * (pi / 180.);
			//double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
			double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
			double gamma = 0.;
			double zeta_1 = 0;
			double zeta_2 = pi / 2;

			v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
			h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

			//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
			//h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);


			double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 90.)*(h_angle_pi_LCS_temp * (180. / pi) / 90.), 25.);
			double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.) * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.), 25.);

			double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 25.);

			/*
			/// Polarized antennas Model-2
			/// 36.873 -> 7.1.1
			double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));

			double F_theta_GCS = A * cos(zeta_2);
			double F_pi_GCS = A * sin(zeta_2);
			//// -> F_tx_



			//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
			//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
			//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

			/// number of TXRU per UE

			double Mp = 1;
			double Np = 2;
			double P = 2;
			double Mg = 1;
			double Ng = 1;


			double dH = 0.5 * Wavelength;
			double dV = -1; /// N/A

			double K = MS_M / Mp;    /////
			double L = MS_N / Np;   //////

			complex <double>jay(0, 1);


			//VectorXd tilt_azimuth_angle_LCS(6);  //// pi
			//VectorXd tilt_zenith_angle_LCS(2);  /// theta

			//tilt_azimuth_angle_LCS << -5 * pi / 16, -3 * pi / 16, -pi / 16, pi / 16, 3 * pi / 16, 5 * pi / 16;
			//tilt_zenith_angle_LCS << 5 * pi / 8, 7 * pi / 8;



			//VectorXcd Vertical_weight(K*L);   //// for theta
			//VectorXcd Horizontal_weight(K*L);   //// for pi

			double F_theta_temp_real = 0.;
			double F_theta_temp_imag = 0.;
			double F_pi_temp_real = 0.;
			double F_pi_temp_imag = 0.;

			/// theta, Vertical
			for (int k = 0; k < K; k++)
			{
			for (int l = 0; l < L; l++)
			{
			VectorXcd wei(1, 1);

			wei(0, 0) = 0.;

			MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, 0.0);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

			VectorXcd ant_co(1, 1);

			ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

			F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
			F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

			F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
			F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));
			}


			}



			/// F, antenna gain for RSRP
			complex <double> F_rx_theta = (F_theta_temp_real, F_theta_temp_imag);
			complex <double> F_rx_pi = (F_pi_temp_real, F_pi_temp_imag);


			//ReceiverAntennaGainXLOS_theta = F_rx_theta;
			//ReceiverAntennaGainXLOS_pi = F_rx_pi;

			ReceiverAntennaGainXLOS_theta = F_theta_GCS;
			ReceiverAntennaGainXLOS_pi = F_pi_GCS;
			*/


			return combined_antenna_gain;

		}
		else if (TYPE == 12)   ////// Dense Urban & Rural
		{

			///////////////////////////////////////////////
			///////////////////////////////////////////////   GCS-> LCS test //////////////////////////////////////////
			///////////////////////////////////////////////


			//double ms_height_in_channel = 0;
			//ms_height_in_channel = ms[self_ms_idx].MS_HEIGHT_FINAL;

			//double distance = sqrt((interferer.x - victim.x)*(interferer.x - victim.x) + (interferer.y - victim.y)*(interferer.y - victim.y));

			double h_angle_pi = channel[_bs_idx][self_ms_idx].LOS_AOA_GCS * (pi / 180.);  //// pi in GCS
			double v_angle_theta = channel[_bs_idx][self_ms_idx].LOS_ZOA_GCS * (pi / 180.);  ///// theta in GCS

			double v_angle_theta_LCS = 0.;
			double h_angle_pi_LCS = 0.;

			double v_angle_theta_LCS_temp = 0.;
			double h_angle_pi_LCS_temp = 0.;


			/////GCS -> LCS , use alpha, beta, gamma
			///// TR 36.873
			double alpha = 360 * randnum.u() * (pi / 180.);
			//double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
			double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
			double gamma = 0.;
			double zeta_1 = 0;
			double zeta_2 = pi / 2;

			v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
			h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

			double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 90.)*(h_angle_pi_LCS_temp * (180. / pi) / 90.), 25.);
			double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 90.) * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 90.), 25.);

			double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 25.);


			//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
			//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
			//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////
			/*

			/// Polarized antennas Model-2
			/// 36.873 -> 7.1.1
			double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));

			double F_theta_GCS = A * cos(zeta_2);
			double F_pi_GCS = A * sin(zeta_2);
			//// -> F_tx_


			/// number of TXRU per UE

			double Mp = 1;
			double Np = 2;
			double P = 2;
			double Mg = 1;
			double Ng = 1;


			double dH = 0.5 * Wavelength;
			double dV = -1; /// N/A

			double K = MS_M / Mp;    /////
			double L = MS_N / Np;   //////

			complex <double>jay(0, 1);


			//VectorXd tilt_azimuth_angle_LCS(6);  //// pi
			//VectorXd tilt_zenith_angle_LCS(2);  /// theta

			//tilt_azimuth_angle_LCS << -5 * pi / 16, -3 * pi / 16, -pi / 16, pi / 16, 3 * pi / 16, 5 * pi / 16;
			//tilt_zenith_angle_LCS << 5 * pi / 8, 7 * pi / 8;



			//VectorXcd Vertical_weight(K*L);   //// for theta
			//VectorXcd Horizontal_weight(K*L);   //// for pi

			double F_theta_temp_real = 0.;
			double F_theta_temp_imag = 0.;
			double F_pi_temp_real = 0.;
			double F_pi_temp_imag = 0.;


			/// theta, Vertical
			for (int k = 0; k < K; k++)
			{
			for (int l = 0; l < L; l++)
			{
			VectorXcd wei(1, 1);

			wei(0, 0) = 0.;

			MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, 0.0);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

			VectorXcd ant_co(1, 1);

			ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

			F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
			F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

			F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
			F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));
			}


			}

			/// F, antenna gain for RSRP
			complex <double> F_rx_theta = (F_theta_temp_real, F_theta_temp_imag);
			complex <double> F_rx_pi = (F_pi_temp_real, F_pi_temp_imag);


			ReceiverAntennaGainXLOS_theta = F_rx_theta;
			ReceiverAntennaGainXLOS_pi = F_rx_pi;
			*/


			return combined_antenna_gain;

			///////////////////////////////////////////////
			///////////////////////////////////////////////   GCS-> LCS test //////////////////////////////////////////
			///////////////////////////////////////////////

		}

		else
		{
			return 0;
		}
	}
}

void LINK::Get_TX_SmallScale_antgain(int _bs_idx, int sector_idx, double tilt_a, double tilt_z)
{
	if (TYPE == 11)   /// Indoor
	{
		SmallScale_TX_AntennaGainXLOS_theta.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);
		SmallScale_TX_AntennaGainXLOS_pi.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);

		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < MAX_NUM_RAYS; j++)
			{
				SmallScale_TX_AntennaGainXLOS_theta(i, j) = 0.;
				SmallScale_TX_AntennaGainXLOS_pi(i, j) = 0.;
			}
		}


		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < channel[_bs_idx][self_ms_idx].NUM_RAY_per_ClusterNUM[i]; j++)
			{

				double h_angle_pi = channel[_bs_idx][self_ms_idx].AOD[i] + channel[_bs_idx][self_ms_idx].cluster_ASD * channel[_bs_idx][self_ms_idx].offset_angle[i][j];
				double v_angle_theta = channel[_bs_idx][self_ms_idx].ZOD[i] + (3 / 8) * pow(10, channel[_bs_idx][self_ms_idx].mu_ZSD) * channel[_bs_idx][self_ms_idx].offset_angle[i][j];

				h_angle_pi = h_angle_pi * (pi / 180.);
				v_angle_theta = v_angle_theta * (pi / 180.);

				double v_angle_theta_LCS = 0.;
				double h_angle_pi_LCS = 0.;

				double v_angle_theta_LCS_temp = 0.;
				double h_angle_pi_LCS_temp = 0.;


				/////GCS -> LCS , use alpha, beta, gamma
				///// TR 36.873
				double alpha = 0.;
				//double beta = 180. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
				double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
				double gamma = 0.;

				double zeta_1 = -pi / 4;
				double zeta_2 = pi / 4;


				v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
				h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

				//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
				//h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);

				double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 90.)*(h_angle_pi_LCS_temp * (180. / pi) / 90.), 25.);
				double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.) * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.), 25.);

				double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 25.);


				/// Polarized antennas Model-2
				/// 36.873 -> 7.1.1
				double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));


				double F_theta_GCS = A * cos(zeta_2);
				double F_pi_GCS = A * sin(zeta_2);
				//// -> F_tx_




				//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
				//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
				//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

				/// number of TXRU per TRxP

				double Mp = 4;
				double Np = 4;
				double P = 2;
				double Mg = 1;
				double Ng = 1;


				double dH = 0.5 * Wavelength;
				double dV = 0.5 * Wavelength;

				double K = BS_M / Mp;    /////   8/2 = 4
				double L = BS_N / Np;   //////   8/1 = 8

				complex <double>jay(0, 1);



				//VectorXcd Vertical_weight(K*L);   //// for theta
				//VectorXcd Horizontal_weight(K*L);   //// for pi

				double F_theta_temp_real = 0.;
				double F_theta_temp_imag = 0.;
				double F_pi_temp_real = 0.;
				double F_pi_temp_imag = 0.;

				/// theta, Vertical
				for (int k = 0; k < K; k++)
				{
					for (int l = 0; l < L; l++)
					{
						VectorXcd wei(1, 1);

						//wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)k * dV * cos(tilt_z));
						//wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)l * dH * sin(tilt_z)*cos(tilt_a));
						wei(0, 0) = 0.;

						MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, dV * k);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

						VectorXcd ant_co(1, 1);

						ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

						F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
						F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

						F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
						F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

					}

				}



				/// F, antenna gain for RSRP
				complex <double> F_tx_theta = (F_theta_temp_real, F_theta_temp_imag);
				complex <double> F_tx_pi = (F_pi_temp_real, F_pi_temp_imag);


				//SmallScale_TX_AntennaGainXLOS_theta(i, j) = F_tx_theta;
				//SmallScale_TX_AntennaGainXLOS_pi(i, j) = F_tx_pi;


				//// 1 to 1 mapping 
				SmallScale_TX_AntennaGainXLOS_theta(i, j) = F_theta_GCS;
				SmallScale_TX_AntennaGainXLOS_pi(i, j) = F_pi_GCS;

				////////// tilt  ʿ ...ۿ ޵..................................................................... 


			}
		}
	}
	else if (TYPE == 12 || TYPE == 13)   ////// Dense Urban & Rural
	{

		SmallScale_TX_AntennaGainXLOS_theta.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);
		SmallScale_TX_AntennaGainXLOS_pi.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);

		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < MAX_NUM_RAYS; j++)
			{
				SmallScale_TX_AntennaGainXLOS_theta(i, j) = 0.;
				SmallScale_TX_AntennaGainXLOS_pi(i, j) = 0.;
			}
		}


		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < channel[_bs_idx][self_ms_idx].NUM_RAY_per_ClusterNUM[i]; j++)
			{

				double h_angle_pi    = channel[_bs_idx][self_ms_idx].AOD[i] + channel[_bs_idx][self_ms_idx].cluster_ASD * channel[_bs_idx][self_ms_idx].offset_angle[i][j];
				double v_angle_theta = channel[_bs_idx][self_ms_idx].ZOD[i] + (3 / 8) * pow(10, channel[_bs_idx][self_ms_idx].mu_ZSD) * channel[_bs_idx][self_ms_idx].offset_angle[i][j];

				h_angle_pi = h_angle_pi * (pi / 180.);
				v_angle_theta = v_angle_theta * (pi / 180.);

				double v_angle_theta_LCS = 0.;
				double h_angle_pi_LCS = 0.;

				double v_angle_theta_LCS_temp = 0.;
				double h_angle_pi_LCS_temp = 0.;


				/////GCS -> LCS , use alpha, beta, gamma
				///// TR 36.873
				double alpha;
				//double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
				double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
				double gamma = 0.;
				if (sector_idx == 1)
				{
					alpha = 30. * (pi / 180.);
				}
				else if (sector_idx == 2)
				{
					alpha = 150. * (pi / 180.);
				}
				else
				{
					alpha = -90. * (pi / 180.);
					//alpha = 270. * (pi / 180.);
				}
				double zeta_1 = -pi / 4;
				double zeta_2 = pi / 4;


				v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
				h_angle_pi_LCS_temp    = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

				//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
				//h_angle_pi_LCS    = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);

				double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 65.)*(h_angle_pi_LCS_temp * (180. / pi) / 65.), 30.);
				double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.) * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.), 30.);

				double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 30.);


				/// Polarized antennas Model-2
				/// 36.873 -> 7.1.1
				double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));


				double F_theta_GCS = A * cos(zeta_2);
				double F_pi_GCS = A * sin(zeta_2);
				//// -> F_tx_




				//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
				//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
				//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

				/// number of TXRU per TRxP

				double Mp = 2;
				double Np = 1;
				double P = 2;
				double Mg = 1;
				double Ng = 1;


				double dH = 0.5 * Wavelength;
				double dV = 0.8 * Wavelength;

				double K = BS_M / Mp;    /////   8/2 = 4
				double L = BS_N / Np;   //////   8/1 = 8

				complex <double>jay(0, 1);



				//VectorXcd Vertical_weight(K*L);   //// for theta
				//VectorXcd Horizontal_weight(K*L);   //// for pi

				double F_theta_temp_real = 0.;
				double F_theta_temp_imag = 0.;
				double F_pi_temp_real = 0.;
				double F_pi_temp_imag = 0.;

				/*
				/// theta, Vertical
				for (int k = 0; k < K; k++)
				{
					VectorXcd wei(1, 1);

					wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)k * dV * cos(tilt_z));

					MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

					VectorXcd ant_co(1, 1);

					ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

					F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
					F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));


				}
				/// pi, Horizontal
				for (int l = 0; l < L; l++)
				{
					VectorXcd wei(1, 1);

					wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)l * dH * sin(tilt_z)*cos(tilt_a));

					MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, 0.0);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

					VectorXcd ant_co(1, 1);

					ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

					F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
					F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));


				}
				*/
				/// theta, Vertical
				for (int k = 0; k < K; k++)
				{
					for (int l = 0; l < L; l++)
					{
						VectorXcd K_wei(1, 1);
						VectorXcd L_wei(1, 1);

						K_wei(0, 0) = -1.0 * jay * (2. * pi * (1 / Wavelength) * (double)k * dV * cos(tilt_z)) ;
						L_wei(0, 0) = (-1.0 * jay * (2. * pi * (1 / Wavelength) * (double)l * dH * sin(tilt_z)*cos(tilt_a)));

						MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, dV * k);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

						VectorXcd ant_co(1, 1);

						ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

						F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(K_wei(0, 0)) * exp(ant_co(0, 0))));
						F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(K_wei(0, 0)) * exp(ant_co(0, 0))));

						F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(L_wei(0, 0)) * exp(ant_co(0, 0))));
						F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(L_wei(0, 0)) * exp(ant_co(0, 0))));

					}


				}



				/// F, antenna gain for RSRP
				complex <double> F_tx_theta = (F_theta_temp_real, F_theta_temp_imag);
				complex <double> F_tx_pi = (F_pi_temp_real, F_pi_temp_imag);


				SmallScale_TX_AntennaGainXLOS_theta(i,j) = F_tx_theta;
				SmallScale_TX_AntennaGainXLOS_pi(i,j) = F_tx_pi;

				////////// tilt  ʿ ...ۿ ޵..................................................................... 


			}
		}
	}

	else
	{
		cout << "TX small scale antgain ERROR!" << endl;
		getchar();
	}

}

void LINK::Get_RX_SmallScale_antgain(int _bs_idx, int sector_idx)
{
	if (TYPE == 11)   /// Indoor
	{
		SmallScale_RX_AntennaGainXLOS_theta.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);
		SmallScale_RX_AntennaGainXLOS_pi.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);

		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < MAX_NUM_RAYS; j++)
			{
				SmallScale_RX_AntennaGainXLOS_theta(i, j) = 0.;
				SmallScale_RX_AntennaGainXLOS_pi(i, j) = 0.;
			}
		}


		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < channel[_bs_idx][self_ms_idx].NUM_RAY_per_ClusterNUM[i]; j++)
			{
				double h_angle_pi = channel[_bs_idx][self_ms_idx].AOA[i] + channel[_bs_idx][self_ms_idx].cluster_ASA * channel[_bs_idx][self_ms_idx].offset_angle[i][j];
				double v_angle_theta = channel[_bs_idx][self_ms_idx].ZOA[i] + channel[_bs_idx][self_ms_idx].cluster_ZSA * channel[_bs_idx][self_ms_idx].offset_angle[i][j];


				h_angle_pi = h_angle_pi * (pi / 180.);
				v_angle_theta = v_angle_theta * (pi / 180.);

				double v_angle_theta_LCS = 0.;
				double h_angle_pi_LCS = 0.;

				double v_angle_theta_LCS_temp = 0.;
				double h_angle_pi_LCS_temp = 0.;

				/*
				/////  GCS -> LCS with mechanical down tile angle
				/////  TR 36.814 / A.2.1.6.2 / equation (1),(2)
				double mechanical_down_tilt_beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
				v_angle_theta_LCS = acos(cos(h_angle_pi)*sin(v_angle_theta)*sin(mechanical_down_tilt_beta) + cos(v_angle_theta)*cos(mechanical_down_tilt_beta));
				complex <double> h_angle_pi_LCS_complex(cos(h_angle_pi)*sin(v_angle_theta)*cos(mechanical_down_tilt_beta) - cos(v_angle_theta)*sin(mechanical_down_tilt_beta), sin(h_angle_pi)*sin(v_angle_theta));
				h_angle_pi_LCS = arg(h_angle_pi_LCS_complex);
				*/

				/////GCS -> LCS , use alpha, beta, gamma
				///// TR 36.873
				double alpha = 360 * randnum.u() * (pi / 180.);
				double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
				//double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
				double gamma = 0.;
				double zeta_1 = 0;
				double zeta_2 = pi / 2;


				v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
				h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

				//v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
				//h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);


				double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 90.)*(h_angle_pi_LCS_temp * (180. / pi) / 90.), 25.);
				double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.) * ((v_angle_theta_LCS_temp * (180. / pi) - 180.) / 90.), 25.);

				double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 25.);


				/// omni-directional
				combined_antenna_gain = 0.;


				/// Polarized antennas Model-2
				/// 36.873 -> 7.1.1
				double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));

				double F_theta_GCS = A * cos(zeta_2);
				double F_pi_GCS = A * sin(zeta_2);
				//// -> F_tx_




				//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
				//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
				//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

				/// number of TXRU per TRxP

				double Mp = 1;
				double Np = 2;
				double P = 2;
				double Mg = 1;
				double Ng = 1;


				double dH = 0.5 * Wavelength;
				double dV = 0.;

				double K = MS_M / Mp;    /////   8/2 = 4
				double L = MS_N / Np;   //////   8/1 = 8

				complex <double>jay(0, 1);


				//VectorXd tilt_azimuth_angle_LCS(6);  //// pi
				//VectorXd tilt_zenith_angle_LCS(2);  /// theta

				//tilt_azimuth_angle_LCS << -5 * pi / 16, -3 * pi / 16, -pi / 16, pi / 16, 3 * pi / 16, 5 * pi / 16;
				//tilt_zenith_angle_LCS << 5 * pi / 8, 7 * pi / 8;



				//VectorXcd Vertical_weight(K*L);   //// for theta
				//VectorXcd Horizontal_weight(K*L);   //// for pi

				double F_theta_temp_real = 0.;
				double F_theta_temp_imag = 0.;
				double F_pi_temp_real = 0.;
				double F_pi_temp_imag = 0.;

				/// theta, Vertical
				for (int k = 0; k < K; k++)
				{
					for (int l = 0; l < L; l++)
					{
						VectorXcd wei(1, 1);

						wei(0, 0) = 0.;

						MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, 0.0);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

						VectorXcd ant_co(1, 1);

						ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

						F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
						F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

						F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
						F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

					}


				}

				/// F, antenna gain for RSRP
				complex <double> F_rx_theta = (F_theta_temp_real, F_theta_temp_imag);
				complex <double> F_rx_pi = (F_pi_temp_real, F_pi_temp_imag);



				//SmallScale_TX_AntennaGainXLOS_theta(i, j) = F_tx_theta;
				//SmallScale_TX_AntennaGainXLOS_pi(i, j) = F_tx_pi;


				//SmallScale_RX_AntennaGainXLOS_theta(i, j) = F_rx_theta;
				//SmallScale_RX_AntennaGainXLOS_pi(i, j) = F_rx_pi;

				SmallScale_RX_AntennaGainXLOS_theta(i, j) = F_theta_GCS;
				SmallScale_RX_AntennaGainXLOS_pi(i, j) = F_pi_GCS;
			}
		}
	}
	else if (TYPE == 12 || TYPE == 13)   ////// Dense Urban & Rural
	{

		SmallScale_RX_AntennaGainXLOS_theta.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);
		SmallScale_RX_AntennaGainXLOS_pi.resize(channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff, MAX_NUM_RAYS);

		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < MAX_NUM_RAYS; j++)
			{
				SmallScale_RX_AntennaGainXLOS_theta(i, j) = 0.;
				SmallScale_RX_AntennaGainXLOS_pi(i, j) = 0.;
			}
		}


		for (int i = 0; i < channel[_bs_idx][self_ms_idx].NUM_PATH_for_channelcoeff; i++)
		{
			for (int j = 0; j < channel[_bs_idx][self_ms_idx].NUM_RAY_per_ClusterNUM[i]; j++)
			{
				double h_angle_pi = channel[_bs_idx][self_ms_idx].AOA[i] + channel[_bs_idx][self_ms_idx].cluster_ASA * channel[_bs_idx][self_ms_idx].offset_angle[i][j];
				double v_angle_theta = channel[_bs_idx][self_ms_idx].ZOA[i] + channel[_bs_idx][self_ms_idx].cluster_ZSA * channel[_bs_idx][self_ms_idx].offset_angle[i][j];


				h_angle_pi = h_angle_pi * (pi / 180.);
				v_angle_theta = v_angle_theta * (pi / 180.);

				double v_angle_theta_LCS = 0.;
				double h_angle_pi_LCS = 0.;

				double v_angle_theta_LCS_temp = 0.;
				double h_angle_pi_LCS_temp = 0.;

				/*
				/////  GCS -> LCS with mechanical down tile angle
				/////  TR 36.814 / A.2.1.6.2 / equation (1),(2)
				double mechanical_down_tilt_beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
				v_angle_theta_LCS = acos(cos(h_angle_pi)*sin(v_angle_theta)*sin(mechanical_down_tilt_beta) + cos(v_angle_theta)*cos(mechanical_down_tilt_beta));
				complex <double> h_angle_pi_LCS_complex(cos(h_angle_pi)*sin(v_angle_theta)*cos(mechanical_down_tilt_beta) - cos(v_angle_theta)*sin(mechanical_down_tilt_beta), sin(h_angle_pi)*sin(v_angle_theta));
				h_angle_pi_LCS = arg(h_angle_pi_LCS_complex);
				*/

				/////GCS -> LCS , use alpha, beta, gamma
				///// TR 36.873
				double alpha = 360 * randnum.u() * (pi / 180.);
				double beta = 90. * (pi / 180.);   ///// sin 90 = 1  cos 90 = 0
				//double beta = 0.;   ///// sin 90 = 1  cos 90 = 0
				double gamma = 0.;
				double zeta_1 = 0;
				double zeta_2 = pi / 2;


				v_angle_theta_LCS_temp = Get_LCS_theta(alpha, beta, gamma, v_angle_theta, h_angle_pi);
				h_angle_pi_LCS_temp = Get_LCS_pi(alpha, beta, gamma, v_angle_theta, h_angle_pi);

				v_angle_theta_LCS = Get_LCS_theta(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);
				h_angle_pi_LCS = Get_LCS_pi(0, 0, zeta_2, v_angle_theta_LCS_temp, h_angle_pi_LCS_temp);


				double h_antgain = -MIN(12. * (h_angle_pi_LCS_temp * (180. / pi) / 65.)*(h_angle_pi_LCS_temp * (180. / pi) / 65.), 30.);
				double v_antgain = -MIN(12. * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.) * ((v_angle_theta_LCS_temp * (180. / pi) - 90.) / 65.), 30.);

				double combined_antenna_gain = max_antgain - MIN(-(h_antgain + v_antgain), 30.);


				/// omni-directional
				combined_antenna_gain = 0.;


				/// Polarized antennas Model-2
				/// 36.873 -> 7.1.1
				double A = sqrt(pow(10.0, (combined_antenna_gain / 10.0)));

				double F_theta_GCS = A * cos(zeta_2);
				double F_pi_GCS = A * sin(zeta_2);
				//// -> F_tx_




				//////////////////////////////////////// 2D DFT decoder                                                                  ////////////////////////////////
				//////////////////////////////////////// RT-170019 Appendix 1                                                            ////////////////////////////////
				//////////////////////////////////////// TR 37.840 V12.1.0, Table 5.4.4.2-3 Composite array pattern for multiple column  ////////////////////////////////

				/// number of TXRU per TRxP
				
				double Mp = 1;
				double Np = 2;
				double P = 2;
				double Mg = 1;
				double Ng = 1;


				double dH = 0.5 * Wavelength;
				double dV = 0.;

				double K = MS_M / Mp;    /////   8/2 = 4
				double L = MS_N / Np;   //////   8/1 = 8

				complex <double>jay(0, 1);


				//VectorXd tilt_azimuth_angle_LCS(6);  //// pi
				//VectorXd tilt_zenith_angle_LCS(2);  /// theta

				//tilt_azimuth_angle_LCS << -5 * pi / 16, -3 * pi / 16, -pi / 16, pi / 16, 3 * pi / 16, 5 * pi / 16;
				//tilt_zenith_angle_LCS << 5 * pi / 8, 7 * pi / 8;



				//VectorXcd Vertical_weight(K*L);   //// for theta
				//VectorXcd Horizontal_weight(K*L);   //// for pi

				double F_theta_temp_real = 0.;
				double F_theta_temp_imag = 0.;
				double F_pi_temp_real = 0.;
				double F_pi_temp_imag = 0.;

				/*
				/// theta, Vertical
				for (int k = 0; k < K; k++)
				{
					VectorXcd wei(1, 1);

					wei(0, 0) = 0.;

					MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, 0.0);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

					VectorXcd ant_co(1, 1);

					ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

					F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
					F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));


				}



				/// pi, Horizontal
				for (int l = 0; l < L; l++)
				{
					VectorXcd wei(1, 1);

					wei(0, 0) = 0.;

					MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, 0.0);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

					VectorXcd ant_co(1, 1);

					ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

					F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
					F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));


				}
				*/
				/// theta, Vertical
				for (int k = 0; k < K; k++)
				{
					for (int l = 0; l < L; l++)
					{
						VectorXcd wei(1, 1);

						wei(0, 0) = 0.;

						MatrixXd Rd = Get_distance_angular(alpha, beta, gamma, 0.0, dH * l, 0.0);  //// Rd = Get_distance_angular(alpha, beta, gamma, 0.0, 0.0, dV * k);

						VectorXcd ant_co(1, 1);

						ant_co(0, 0) = jay * (2. * pi * (1 / Wavelength) * (sin(v_angle_theta)*cos(h_angle_pi)*Rd(0, 0) + sin(v_angle_theta)*sin(h_angle_pi)*Rd(1, 0) + cos(v_angle_theta)*Rd(2, 0)));

						F_theta_temp_real = F_theta_temp_real + (F_theta_GCS * (1.0 / sqrt(K)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
						F_theta_temp_imag = F_theta_temp_imag + (F_theta_GCS * (1.0 / sqrt(K)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

						F_pi_temp_real = F_pi_temp_real + (F_pi_GCS * (1.0 / sqrt(L)) * real(exp(wei(0, 0)) * exp(ant_co(0, 0))));
						F_pi_temp_imag = F_pi_temp_imag + (F_pi_GCS * (1.0 / sqrt(L)) * imag(exp(wei(0, 0)) * exp(ant_co(0, 0))));

					}
				}




				/// F, antenna gain for RSRP
				complex <double> F_rx_theta = (F_theta_temp_real, F_theta_temp_imag);
				complex <double> F_rx_pi = (F_pi_temp_real, F_pi_temp_imag);



				//SmallScale_TX_AntennaGainXLOS_theta(i, j) = F_tx_theta;
				//SmallScale_TX_AntennaGainXLOS_pi(i, j) = F_tx_pi;

				
				SmallScale_RX_AntennaGainXLOS_theta(i, j) = F_rx_theta;
				SmallScale_RX_AntennaGainXLOS_pi(i, j) = F_rx_pi;
			}
		}
	}

	else
	{
		cout << "RX small scale antgain ERROR!" << endl;
		getchar();
	}
}


double LINK::Get_RSRP(int _bs_idx, int _ms_idx)
{
	complex<double> Big_PI_LOS             (0. , (360 * randnum.u() - 180.0)*pi / 180.0);

	complex<double> Big_pi_NLOS_thetatheta (0, (360 * randnum.u() - 180.0)*pi / 180.0);
	complex<double> Big_pi_NLOS_thetapi    (0, (360 * randnum.u() - 180.0)*pi / 180.0);
	complex<double> Big_pi_NLOS_pitheta    (0, (360 * randnum.u() - 180.0)*pi / 180.0);
	complex<double> Big_pi_NLOS_pipi       (0, (360 * randnum.u() - 180.0)*pi / 180.0);

	double K_linear = channel[_bs_idx][_ms_idx].K_linear;
	double XPR = pow(10.0, (channel[_bs_idx][_ms_idx].XPR) / 10.0);

	double alpha_zero = 0.;
	complex <double> alpha_zero_temp = 0.;
	double alpha_nmup = 0.;
	complex <double> alpha_nmup_temp = 0.;

	alpha_zero_temp = sqrt(K_linear / (K_linear + 1)) * (ReceiverAntennaGainXLOS_theta * exp(Big_PI_LOS) * TransmitterAntennaGainXLOS_theta - ReceiverAntennaGainXLOS_pi * exp(Big_PI_LOS) * TransmitterAntennaGainXLOS_pi);

	alpha_zero = pow(abs(alpha_zero_temp), 2);

	for (int i = 0; i < channel[_bs_idx][_ms_idx].NUM_PATH_for_channelcoeff; i++)
	{
		for (int j = 0; j < MAX_NUM_RAYS; j++)
		{
			alpha_nmup_temp = sqrt(channel[_bs_idx][_ms_idx].power[i] / (20 * (K_linear + 1))) * (
				(SmallScale_RX_AntennaGainXLOS_theta(i, j) * exp(Big_pi_NLOS_thetatheta) + SmallScale_RX_AntennaGainXLOS_pi(i, j)*sqrt(1 / XPR)*exp(Big_pi_NLOS_pitheta)) * SmallScale_TX_AntennaGainXLOS_theta(i, j) +
				(SmallScale_RX_AntennaGainXLOS_theta(i, j) * sqrt(1 / XPR) * exp(Big_pi_NLOS_thetapi) + SmallScale_RX_AntennaGainXLOS_pi(i, j)*exp(Big_pi_NLOS_pipi)) * SmallScale_TX_AntennaGainXLOS_pi(i, j)
				);

			alpha_nmup = alpha_nmup + pow(abs(alpha_nmup_temp), 2);
		}
	}


	//RSRP_antgain = 10 * log10(alpha_zero + alpha_nmup);

	return 10 * log10(alpha_zero + alpha_nmup);

	//cout << "alpha_nmup = " << 10*log10(alpha_zero + alpha_nmup) << endl;
	//getchar();


}

double Get_LCS_theta(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi)
{
	return acos( cos(beta)*cos(gamma)*cos(GCS_theta) + (sin(beta)*cos(gamma)*cos(GCS_pi-alpha) - sin(gamma)*sin(GCS_pi-alpha))*sin(GCS_theta));
}

double Get_LCS_pi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi)
{

	// arg(x+iy) = atan2(y,x)


	double real;
	double imag;

	real = (cos(beta)*sin(GCS_theta)*cos(GCS_pi - alpha) - sin(beta)*cos(GCS_theta));
	imag = (cos(beta)*sin(gamma)*cos(GCS_theta) + (sin(beta)*sin(gamma)*cos(GCS_pi - alpha) + cos(gamma)*sin(GCS_pi - alpha))*sin(GCS_theta));

	return atan2(imag,real);

}


MatrixXd Get_distance_angular(double a, double b, double c, double x, double y, double z)
{
	///// TR 36.873 page 9

	MatrixXd Rd(3, 1);

	Rd(0, 0) = cos(a)*cos(b) * x + cos(a)*sin(b)*sin(c) - sin(a)*cos(c) * y + cos(a)*sin(b)*cos(c) + sin(a)*sin(c) * z;
	Rd(1, 0) = sin(a)*cos(b) * x + sin(a)*sin(b)*sin(c) + cos(a)*cos(c) * y + sin(a)*sin(b)*cos(c) - cos(a)*sin(c) * z;
	Rd(2, 0) = -sin(b)		 * x + cos(b)*sin(c)					    * y + cos(b)*cos(c)						   * z;

	return Rd;
}

