#include "common.h"

double normal(double mu, double sigma);
double Get_LCS_theta(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi);
double Get_LCS_pi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi);
double Get_cos_psi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi);
double Get_sin_psi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi);

void CHANNEL::Set_channel(double _distance, bool _Indoor, int _bs_idx, int _ms_idx, LOCATION _bs_location, LOCATION _ms_location)
{
	distance = _distance;
	Indoor = _Indoor;

	self_bs_idx = _bs_idx;
	self_ms_idx = _ms_idx;
	bs_location = _bs_location;
	ms_location = _ms_location;


	Set_LOS_Prob();
	Set_PATHLOSS();
	//Set_SHADOWFADING();
	Set_Channel_Parameters();






}

void CHANNEL::Set_SmallScaleParameter(int _bs_idx, int _ue_idx)
{

	K_dB = 0;
	K_linear = 0;
	P_1LOS = 0;

	if (channel[_bs_idx][_ue_idx].Propagation == LOS_propagation)
	{
		_DS = ms[_ue_idx].LSPs[_bs_idx](LOS_DS);
		K_dB = ms[_ue_idx].LSPs[_bs_idx](LOS_K);
		ASA = ms[_ue_idx].LSPs[_bs_idx](LOS_ASA);
		ASD = ms[_ue_idx].LSPs[_bs_idx](LOS_ASD);
		ZSA = ms[_ue_idx].LSPs[_bs_idx](LOS_ZSA);
		ZSD = ms[_ue_idx].LSPs[_bs_idx](LOS_ZSD);
		K_linear = pow(10, K_dB / 10);
		P_1LOS = K_linear / (K_linear + 1);
		//mu_XPR = XPR_mean[LOS];
		//sigma_XPR = XRP_std[LOS];
	}
	else if (channel[_bs_idx][_ue_idx].Propagation == NLOS_propagation)
	{
		_DS = ms[_ue_idx].LSPs[_bs_idx](NLOS_DS);
		ASA = ms[_ue_idx].LSPs[_bs_idx](NLOS_ASA);
		ASD = ms[_ue_idx].LSPs[_bs_idx](NLOS_ASD);
		ZSA = ms[_ue_idx].LSPs[_bs_idx](NLOS_ZSA);
		ZSD = ms[_ue_idx].LSPs[_bs_idx](NLOS_ZSD);
		//mu_XPR = XPR_mean[NLOS];
		//sigma_XPR = XRP_std[NLOS];
	}
	else if (channel[_bs_idx][_ue_idx].Propagation == OUT2IN_propagation)
	{
		_DS = ms[_ue_idx].LSPs[_bs_idx](OUT2IN_DS);
		ASA = ms[_ue_idx].LSPs[_bs_idx](OUT2IN_ASA);
		ASD = ms[_ue_idx].LSPs[_bs_idx](OUT2IN_ASD);
		ZSA = ms[_ue_idx].LSPs[_bs_idx](OUT2IN_ZSA);
		ZSD = ms[_ue_idx].LSPs[_bs_idx](OUT2IN_ZSD);
		//mu_XPR = XPR_mean[OUT2IN];
		//sigma_XPR = XRP_std[OUT2IN];
	}

	Set_DELAY();
	Set_POWER();
	Find_Strong2Clusters();
	Set_AOAAOD(_bs_idx, _ue_idx);
	Set_ZOAZOD(_bs_idx, _ue_idx);


	/*
	if (CH_CAL == 0 || CH_CAL == 2)       // For Pre-Calculate
	{
	Set_AOAAOD_LOS();
	Set_ZOAZOD_LOS();

	Set_AOAAOD_NLOS();
	Set_ZOAZOD_NLOS();

	Set_AOAAOD();
	Set_ZOAZOD();
	}
	else
	{
	Set_AOAAOD();
	Set_ZOAZOD();
	}
	*/

	Set_InitialPhase();
	Set_SUBCLUSTER();
	Set_W_matrix();

	/*
	Set_RMS_delay_spread(); // for calibration
	Set_circular_angle_spread();
	*/
}

void CHANNEL::Allocate_memory()
{
	delay = new double[MAX_NUM_CLUSTERS];
	delay_LOS = new double[MAX_NUM_CLUSTERS];
	power = new double[MAX_NUM_CLUSTERS];
	power_LOS = new double[MAX_NUM_CLUSTERS];
	power_NLOS = new double[MAX_NUM_CLUSTERS];   /// LOS, NLOS separate for pre-calc
	AOA = new double[MAX_NUM_CLUSTERS];
	AOD = new double[MAX_NUM_CLUSTERS];
	ZOD = new double[MAX_NUM_CLUSTERS];
	ZOA = new double[MAX_NUM_CLUSTERS];
	NUM_RAY_per_ClusterNUM = new double[MAX_NUM_CLUSTERS];

	random_phase_vv = new double *[MAX_NUM_CLUSTERS];
	random_phase_vh = new double *[MAX_NUM_CLUSTERS];
	random_phase_hv = new double *[MAX_NUM_CLUSTERS];
	random_phase_hh = new double *[MAX_NUM_CLUSTERS];

	offset_angle = new double *[MAX_NUM_CLUSTERS];

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		random_phase_vv[i] = new double[MAX_NUM_RAYS];
		random_phase_vh[i] = new double[MAX_NUM_RAYS];
		random_phase_hv[i] = new double[MAX_NUM_RAYS];
		random_phase_hh[i] = new double[MAX_NUM_RAYS];

		offset_angle[i] = new double[MAX_NUM_RAYS];

	}


}

void CHANNEL::Delete_memory()
{

	delete [] delay;
	delete [] delay_LOS;
	delete [] power;
	delete [] power_LOS;
	delete [] power_NLOS;   /// LOS, NLOS separate for pre-calc
	delete [] AOA;
	delete [] AOD;
	delete [] ZOD;
	delete [] ZOA;
	delete [] NUM_RAY_per_ClusterNUM;


	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		delete [] random_phase_vv[i];
		delete [] random_phase_vh[i];
		delete [] random_phase_hv[i];
		delete [] random_phase_hh[i];

		delete [] offset_angle[i];
	}

	delete [] random_phase_vv;
	delete [] random_phase_vh;
	delete [] random_phase_hv;
	delete [] random_phase_hh;

	delete [] offset_angle;
}


void CHANNEL::Set_LOS_Prob()
{

	//////////////////////////////
	///
	/// LOS Prob
	///
	//////////////////////////////////

	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 46


	if (TYPE == 11) // InH
	{
		if (distance <= 5)
		{
			LOS_prob = 1;
		}
		else if (distance > 5. && distance < 49.)
		{
			LOS_prob = exp(-(distance - 5.) / 70.8);
		}
		else if (distance >= 49.)
		{
			LOS_prob = exp(-(distance - 49.) / 211.7) * 0.54;
		}

		double P = randnum.u();

		if (P < LOS_prob)
		{
			LOS = true;   ///  LOS
			Propagation = LOS_propagation;
		}
		else
		{
			LOS = false;   ///  NLOS
			Propagation = NLOS_propagation;
		}


		ms[self_ms_idx].MS_HEIGHT_FINAL = ms_height;
		ms[self_ms_idx].floor_idx = 0;


		//LOS = true;   ///  LOS
		//Propagation = LOS_propagation;
		//cout << "LOS = true" << endl;

	}
	else if (TYPE == 12) // Dense_Urban 
	{
		double n = 0;
		double N = floor((5. * randnum.u()) + 4.); // 4 ~ 8
		n = floor(((N)* randnum.u()) + 1.); // 1 ~ N

		ms_height_in = 3 * (n - 1) + 1.5;


		//////////////////////////////////////// UMi
		if (Configuration_Type == 3)  
		{
			///////////////////////////////// outdoor
			if (Indoor == false)
			{
				if (distance <= 18.)
				{
					LOS_prob = 1;
				}
				else if (distance > 18.)
				{
					LOS_prob = (18. / distance) + exp(-1 * distance / 36.) * (1. - (18. / distance));
				}

				double P = randnum.u();
				if (P < LOS_prob)
				{
					LOS = true;
					Propagation = LOS_propagation;
				}
				else
				{
					LOS = false;
					Propagation = NLOS_propagation;
				}

				ms[self_ms_idx].MS_HEIGHT_FINAL = ms_height_out;
				ms[self_ms_idx].floor_idx = 0;

			}
			///////////////////////////////////////// Indoor
			else if (Indoor == true)
			{

				double d_in;
				double d_out;

				d_in = MIN(25. * randnum.u(), 25. * randnum.u());  //// UMa, UMi = 0~25, RMa = 0~10 
				d_out = distance - d_in;

				if (distance <= 18.)
				{
					LOS_prob = 1;
				}
				else if (distance > 18.)
				{
					LOS_prob = (18. / d_out) + exp(-1 * d_out / 36.) * (1. - (18. / d_out));
				}

				double P = randnum.u();
				if (P < LOS_prob)
				{
					LOS = true;
					Propagation = LOS_propagation;
				}
				else
				{
					LOS = false;
					Propagation = NLOS_propagation;
				}

				ms[self_ms_idx].MS_HEIGHT_FINAL = ms_height_in; 
				ms[self_ms_idx].floor_idx = 0;

			}
			else
			{
				cout << "Indoor state ERROR! - LOS Prob" << endl;
			}

			////////////////////////////////// Set OUT2IN case /////////////////////////////
			if (Indoor == true)
			{
				Propagation = OUT2IN_propagation;   //// OUT2IN  
			}


			//LOS = false;
			///Propagation = NLOS_propagation;
			//cout << "LOS = false" << endl;
		}
		//////////////////////////////////////// UMa 
		else  
		{
			/////////////////////// outdoor
			if (Indoor == false) 
			{
				double C = 0;

				if (ms_height_out <= 13)
				{
					C = 0;
				}
				else
				{
					C = pow(((ms_height_out - 13.) / 10.), 1.5);
				}

				if (distance <= 18.)
				{
					LOS_prob = 1;
				}
				else if (distance > 18.)
				{
					LOS_prob = ((18. / distance) + exp(-1 * distance / 63.) * (1 - (18. / distance))) * (1 + C * (5 / 4) * pow((distance / 100), 3) * exp(-1 * distance / 150.));
				}

				double P = randnum.u();

				if (P < LOS_prob)
				{
					LOS = true;   /// LOS
					Propagation = LOS_propagation;
				}
				else
				{
					LOS = false;   /// NLOS
					Propagation = NLOS_propagation;
				}

				ms[self_ms_idx].MS_HEIGHT_FINAL = ms_height_out;
				ms[self_ms_idx].floor_idx = 0;
				ms[self_ms_idx].ms_d_in = 0.;

			}
			///////////////////////////// Indoor
			else if (Indoor == true)  
			{

				double d_in;
				double d_out;

				d_in = 25. * randnum.u();  //// UMa, UMi = 0~25, RMa = 0~10 
				d_out = distance - d_in;

				double C = 0;

				if (ms_height_in <= 13)
				{
					C = 0;
				}
				else
				{
					C = pow(((ms_height_in - 13.) / 10.), 1.5);
				}

				if (d_out <= 18)
				{
					LOS_prob = 1;
				}
				else if (d_out > 18)
				{
					LOS_prob = ((18. / d_out) + exp(-1 * d_out / 63.) * (1 - (18. / d_out))) * (1 + C * (5 / 4) * pow(d_out / 100, 3) * exp(-1 * d_out / 150.));
				}

				double P = randnum.u();
				if (P < LOS_prob)
				{
					LOS = true;
					Propagation = LOS_propagation;
				}
				else
				{
					LOS = false;
					Propagation = NLOS_propagation;
				}

				ms[self_ms_idx].MS_HEIGHT_FINAL = ms_height_in;
				ms[self_ms_idx].floor_idx = int(n) - 1;
				ms[self_ms_idx].ms_d_in = d_in;

			}
			else
			{
				cout << "Indoor state ERROR! - LOS Prob" << endl;
			}


			/////////////////////// Set OUT2IN case //////////////
			if (Indoor == true)
			{
				Propagation = OUT2IN_propagation;   //// OUT2IN  
			}

			/// For SSP test

			//LOS = true;
			//Propagation = LOS_propagation;
			//cout << "LOS = true" << endl;

			//LOS = true;
			//Propagation = LOS_propagation;
			//cout << "LOS = true" << endl;
		}



	}
	else if (TYPE == 13) // RMa
	{
		///////////////////////////////// outdoor
		if (Indoor == false) 
		{
			if (distance <= 10)
			{
				LOS_prob = 1;
			}
			else if (distance > 10.)
			{
				LOS_prob = exp(-1 * (distance - 10.) / 1000.);
			}

			double P = randnum.u();
			if (P < LOS_prob)
			{
				LOS = true;
				Propagation = LOS_propagation;
			}
			else
			{
				LOS = false;
				Propagation = NLOS_propagation;
			}

			ms[self_ms_idx].MS_HEIGHT_FINAL = ms_height_out;
			ms[self_ms_idx].floor_idx = 0;

		}
		///////////////////////////////////////// Indoor
		else if (Indoor == true)  
		{

			double d_in;
			double d_out;

			d_in = MIN(10. * randnum.u(), 10. * randnum.u());  //// UMa, UMi = 0~25, RMa = 0~10 
			d_out = distance - d_in;

			if (distance <= 10)
			{
				LOS_prob = 1;
			}
			else if (distance > 10.)
			{
				LOS_prob = exp(-1*(d_out - 10.) / 1000.);
			}

			double P = randnum.u();
			if (P < LOS_prob)
			{
				LOS = true;
				Propagation = LOS_propagation;
			}
			else
			{
				LOS = false;
				Propagation = NLOS_propagation;
			}

			ms[self_ms_idx].MS_HEIGHT_FINAL = ms_height_in; // 1.5
			ms[self_ms_idx].floor_idx = 0;

		}
		else
		{
			cout << "Indoor state ERROR! - LOS Prob" << endl;
		}

		////////////////////////////////// Set OUT2IN case /////////////////////////////
		if (Indoor == true)
		{
			Propagation = OUT2IN_propagation;   //// OUT2IN  
		}


		//LOS = false;
		//Propagation = NLOS_propagation;
		//cout << "LOS = false" << endl;
	}
}

void CHANNEL::Set_PATHLOSS()
{
	////////////////////////////// PathLoss ////////////////////////////////
	//PathLoss
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 39 ~ 

	/////////////////////////////////////////// InH
	if (TYPE == 11) 
	{
		double distance_3d;
		distance_3d = sqrt(distance * distance + (bs_height - ms_height) * (bs_height - ms_height)); //3D distance

		//////////////////////////////////////// Channel Model A
		if (Channel_Model_Type == 0) 
		{
			//////////////////////////////////// LOS
			if (LOS == 1)  
			{
				if (Frequency <= 6 * pow(10,9))  ///// 0.5GHz <= fc <= 6GHz
				{
					sigma_SF = 3.;
					pathloss = 16.9 * log10(distance_3d) + 32.8 + 20 * log10(Frequency / 1000000000.);
				}
				else  // 6GHz < fc <= 100 GHz
				{
					sigma_SF = 3.;
					pathloss = 32.4 + 17.3 * log10(distance_3d) + 20 * log10(Frequency / 1000000000.);
				}
			}
			/////////////////////////////////// NLOS
			else if (LOS == 0)  
			{
				if (Frequency <= 6 * pow(10,9))  ///// 0.5GHz <= fc <= 6GHz
				{
					sigma_SF = 4.;
					pathloss = 43.3 * log10(distance_3d) + 11.5 + 20 * log10(Frequency / 1000000000.);
				}
				else  // 6GHz < fc <= 100 GHz
				{
					sigma_SF = 8.03;
					pathloss = 43.3 * log10(distance_3d) + 11.5 + 20 * log10(Frequency / 1000000000.);
					pathloss_2 = 38.3 * log10(distance_3d) + 17.3 + 24.9 * log10(Frequency / 1000000000.);
					pathloss = MAX(pathloss, pathloss_2);
				}
			}
		}
		//////////////////////////////////////// Channel Model B
		else if (Channel_Model_Type == 1) 
		{
			//////////////////////////////////// LOS
			if (LOS == 1)  
			{
				sigma_SF = 3.;
				pathloss = 17.3 * log10(distance_3d) + 32.4 + 20 * log10(Frequency / 1000000000.);

			}
			//////////////////////////////////// NLOS
			else if (LOS == 0)  
			{
				sigma_SF = 8.03;
				pathloss = 17.3 * log10(distance_3d) + 32.4 + 20 * log10(Frequency / 1000000000.);
				pathloss_2 = 38.3 * log10(distance_3d) + 17.3 + 24.9 * log10(Frequency / 1000000000.);
				pathloss = MAX(pathloss, pathloss_2);
			}
		}
	}
	////////////////////////////////////////////////////// Dense_Urban 
	else if (TYPE == 12) 
	{
		double n = 0;
		double N = floor((5. * randnum.u()) + 4.); // 4 ~ 8
		n = floor(((N)* randnum.u()) + 1.); // 1 ~ N

		ms_height_in = 3 * (n - 1) + 1.5;

		double ms_height_in_channel = 0;

		if (Indoor == 1) 
		{
			//ms_height_in_channel = ms_height_in;
			ms_height_in_channel = ms[self_ms_idx].MS_HEIGHT_FINAL;
		}
		else if (Indoor == 0) 
		{
			ms_height_in_channel = ms_height_out;
		}


		////////////////////////////////////////////////////// UMi
		if (Configuration_Type == 3)
		{
			double g;
			double C;
			double hE;

			if (distance > 18.)
			{
				g = (5 / 4) * pow((distance / 100), 3) * exp(-1 * distance / 150);
			}
			else
			{
				g = 0;
			}

			if (ms_height_in_channel < 13)
			{
				C = 0;
			}
			else
			{
				C = pow((ms_height_in_channel - 13.) / 10., 1.5) * g;
			}


			if (randnum.u() < (1 / (1 + C)))
			{
				hE = 1;

			}
			else
			{

				hE = floor((ms_height_in_channel - 1.5 - 9.) / 3. * randnum.u()) * 3. + 12.;
			}

			double d_BP = 4 * (ms_height_in_channel - hE)*(bs_height - hE)*Frequency / light_speed;
			double distance_3d;
			distance_3d = sqrt(distance * distance + (bs_height - ms_height_in_channel) * (bs_height - ms_height_in_channel)); //3D distance

			/////////////////////////////////////////////// Channel Model A
			if (Channel_Model_Type == 0)
			{
				//////////////////////////////// LOS
				if (LOS == 1)
				{

					if (Frequency <= 6 * pow(10, 9))  ///// 0.5GHz <= fc <= 6GHz
					{
						if (distance <= d_BP)
						{
							sigma_SF = 3.;
							pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
						}
						else if (d_BP < distance)
						{
							sigma_SF = 3.;
							pathloss = 40. * log10(distance_3d) + 28. + 20. * log10(Frequency / 1000000000.) - 9. * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
						}
					}
					else// 6GHz < fc <= 100 GHz
					{
						if (distance <= d_BP)
						{
							sigma_SF = 4.;
							pathloss = 32.4 + 21. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
						}
						else if (d_BP < distance)
						{
							sigma_SF = 4.;
							pathloss = 40. * log10(distance_3d) + 32.4 + 20. * log10(Frequency / 1000000000.) - 9.5 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
						}
					}
				}
				/////////////////////////////// NLOS
				else if (LOS == 0)
				{
					if (Frequency <= 6 * pow(10, 9))  ///// 0.5GHz <= fc <= 6GHz
					{
						if (distance <= d_BP)
						{
							sigma_SF = 3.;
							pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
						}
						else if (d_BP < distance)
						{
							sigma_SF = 3.;
							pathloss = 40. * log10(distance_3d) + 28. + 20. * log10(Frequency / 1000000000.) - 9. * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
						}
						pathloss_2 = 36.7 * log10(distance_3d) + 22.7 + 26. * log10(Frequency / 1000000000.) - 0.3*(ms_height_in_channel - 1.5);

						pathloss = MAX(pathloss, pathloss_2);
					}
					else  // 6GHz < fc <= 100 GHz
					{
						if (distance <= d_BP)
						{
							sigma_SF = 4.;
							pathloss = 32.4 + 21. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
						}
						else if (d_BP < distance)
						{
							sigma_SF = 4.;
							pathloss = 40. * log10(distance_3d) + 32.4 + 20. * log10(Frequency / 1000000000.) - 9.5 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
						}
						pathloss_2 = 35.3 * log10(distance_3d) + 22.4 + 21.3 * log10(Frequency / 1000000000.) - 0.3*(ms_height_in_channel - 1.5);

						pathloss = MAX(pathloss, pathloss_2);
					}
				}

			}
			////////////////////////////////////////////////////////////// Channel Model B
			else if (Channel_Model_Type == 1)
			{
				/////////////////////////////////////////////////////// LOS
				if (LOS == 1)
				{
					if (distance <= d_BP)
					{
						sigma_SF = 4.;
						pathloss = 32.4 + 21. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
					}
					else if (d_BP < distance)
					{
						sigma_SF = 4.;
						pathloss = 40 * log10(distance_3d) + 32.4 + 20. * log10(Frequency / 1000000000.) - 9.5 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
					}
				}
				//////////////////////////////////////////////// NLOS
				else if (LOS == 0)
				{
					if (distance <= d_BP)
					{
						sigma_SF = 4.;
						pathloss = 32.4 + 21. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
					}
					else if (d_BP < distance)
					{
						sigma_SF = 4.;
						pathloss = 40 * log10(distance_3d) + 32.4 + 20. * log10(Frequency / 1000000000.) - 9.5 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
					}
					pathloss_2 = 22.4 + 35.3*log10(distance_3d) + 21.3 * log10(Frequency / 1000000000.) - 0.3*(ms_height_in_channel - 1.5);

					pathloss = MAX(pathloss, pathloss_2);
				}
			}
		}
		////////////////////////////////////////////////////// UMa
		else
		{
			double g;
			double C;
			double hE;

			if (distance > 18.)
			{
				g = (5 / 4) * pow((distance / 100), 3) * exp(-1 * distance / 150);
			}
			else
			{
				g = 0;
			}

			if (ms_height_in_channel < 13)
			{
				C = 0;
			}
			else
			{
				C = pow(((ms_height_in_channel - 13.) / 10.), 1.5) * g;
			}


			if (randnum.u() < (1 / (1 + C)))
			{
				hE = 1;

			}
			else
			{

				hE = floor((ms_height_in_channel - 1.5 - 9.) / 3. * randnum.u()) * 3. + 12.;

			}

			double d_BP = 4 * (ms_height_in_channel - hE)*(bs_height - hE)*Frequency / light_speed;
			double distance_3d;
			distance_3d = sqrt(distance * distance + (bs_height - ms_height_in_channel) * (bs_height - ms_height_in_channel)); //3D distance

			/////////////////////////////////////////////// Channel Model A
			if (Channel_Model_Type == 0)
			{
				//////////////////////////////// LOS
				if (LOS == 1)
				{
					if (distance <= d_BP)
					{
						sigma_SF = 4.;
						pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
					}
					else if (d_BP < distance)
					{
						sigma_SF = 4.;
						pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
					}
				}
				/////////////////////////////// NLOS
				else if (LOS == 0)
				{
					if (Frequency <= 6000000000.)  ///// 0.5GHz <= fc <= 6GHz
					{
						if (distance <= d_BP)
						{
							sigma_SF = 4.;
							pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
						}
						else if (d_BP < distance)
						{
							sigma_SF = 4.;
							pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
						}

						pathloss_2 = 161.04 - 7.1 * log10(20) + 7.5*log10(20) - (24.37 - 3.7*(20 / bs_height)*(20 / bs_height))*log10(bs_height) + (43.42 - 3.1*log10(bs_height))*(log10(distance_3d) - 3) + 20 * log10(Frequency / 1000000000.) - (3.2*(pow(log10(17.625), 2)) - 4.97) - 0.6*(ms_height_in_channel - 1.5);

						pathloss = MAX(pathloss, pathloss_2);


					}
					else  // 6GHz < fc <= 100 GHz
					{
						if (distance <= d_BP)
						{
							sigma_SF = 4;
							pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
						}
						else if (d_BP < distance)
						{
							sigma_SF = 4;
							pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
						}

						pathloss_2 = 13.54 + 39.08*log10(distance_3d) + 20 * log10(Frequency / 1000000000.) - 0.6*(ms_height_in_channel - 1.5);

						pathloss = MAX(pathloss, pathloss_2);
					}
				}

			}
			////////////////////////////////////////////////////////////// Channel Model B
			else if (Channel_Model_Type == 1)
			{
				/////////////////////////////////////////////////////// LOS
				if (LOS == 1)
				{
					if (distance <= d_BP)
					{
						sigma_SF = 4.;
						pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
					}
					else if (d_BP < distance)
					{
						sigma_SF = 4.;
						pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
					}
				}
				//////////////////////////////////////////////// NLOS
				else if (LOS == 0)
				{
					if (distance <= d_BP)
					{
						sigma_SF = 4.;
						pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.);
					}
					else if (d_BP < distance)
					{
						sigma_SF = 4.;
						pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2));
					}

					pathloss_2 = 13.54 + 39.08*log10(distance_3d) + 20 * log10(Frequency / 1000000000.) - 0.6*(ms_height_in_channel - 1.5);

					pathloss = MAX(pathloss, pathloss_2);
				}
			}
		}

	}
	/////////////////////////////////////////////////////////////////////////////// RMa
	else if (TYPE == 13) 
	{

		double ms_height_in_channel = 0;

		/////////////////////////////////////////////////////////////// Indoor
		if (Indoor == 1) 
		{
			ms_height_in_channel = ms_height_in;
		}
		///////////////////////////////////////////////////////// Outdoor
		else if (Indoor == 0) 
		{
			ms_height_in_channel = ms_height_out;
		}

		double d_BP = 2 * pi * (ms_height_in_channel)*(bs_height)*Frequency / light_speed;
		double distance_3d;
		distance_3d = sqrt(distance * distance + (bs_height - ms_height_in_channel) * (bs_height - ms_height_in_channel)); //3D distance

		/////////////////////////////////////////////////////////// Channel Model A
		if (Channel_Model_Type == 0) 
		{
			double hh = 5.;
			double WW = 20.;

			///////////////////////////////////LOS
			if (LOS == 1)
			{
				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss = 20 * log10(40 * pi*distance_3d*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(distance_3d) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*distance_3d;
				}
				else if (d_BP < distance && distance < 21000.)
				{
					sigma_SF = 6.;
					pathloss_2 = 20 * log10(40 * pi*d_BP*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(d_BP) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*d_BP;

					pathloss = pathloss_2 + 40 * log10(distance_3d / d_BP);
				}
			}
			//////////////////////////////// NLOS
			else if (LOS == 0)
			{
				sigma_SF = 8.;
				pathloss = 161.04 - 7.1*log10(WW) + 7.5*log10(hh) - (24.37 - 3.7*(pow((hh / bs_height), 2)))*log10(bs_height) + (43.42 - 3.1*log10(bs_height))*(log10(distance_3d) - 3) + 20 * log10(Frequency / 1000000000.) - (3.2*(log10(11.75*ms_height_in_channel))*(log10(11.75*ms_height_in_channel)) - 4.97);
			}
		}
		///////////////////////////////////////////////////// Channel Model B
		else if (Channel_Model_Type == 1) 
		{
			double hh = 5.;
			double WW = 20.;

			////////////////////////////////////////// LOS
			if (LOS == 1)
			{
				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss = 20 * log10(40 * pi*distance_3d*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(distance_3d) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*distance_3d;
				}
				else if (d_BP < distance)
				{
					sigma_SF = 6.;
					pathloss_2 = 20 * log10(40 * pi*d_BP*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(d_BP) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*d_BP;

					pathloss = pathloss_2 + 40 * log10(distance_3d / d_BP);
				}
			}
			/////////////////////////////////////////// NLOS
			else if (LOS == 0)
			{

				sigma_SF = 8.;
				pathloss = 161.04 - 7.1*log10(WW) + 7.5*log10(hh) - (24.37 - 3.7*(pow((hh / bs_height), 2)))*log10(bs_height) + (43.42 - 3.1*log10(bs_height))*(log10(distance_3d) - 3) + 20 * log10(Frequency / 1000000000.) - (3.2*(log10(11.75*ms_height_in_channel))*(log10(11.75*ms_height_in_channel)) - 4.97);

				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss_2 = 20 * log10(40 * pi*distance_3d*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(distance_3d) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*distance_3d;
				}
				else if (d_BP < distance)
				{
					sigma_SF = 6.;
					pathloss_2 = 20 * log10(40 * pi*d_BP*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(d_BP) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*d_BP;

					pathloss_2 = pathloss_2 + 40 * log10(distance_3d / d_BP);
				}
				pathloss = MAX(pathloss, pathloss_2);
			}
		}

	}
}

/*
void CHANNEL::Set_SHADOWFADING()
{

	////// IMT 2020 EVAL Page 39 ~ 

	if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
}
*/

double normal(double mu, double sigma)
{
	double U1, U2, W, mult;
	static double X1, X2;
	static int call = 0;

	if (call == 1)
	{
		call = !call;
		return (mu + sigma * (double)X2);
	}

	do
	{
		U1 = -1 + randnum.u() * 2;
		U2 = -1 + randnum.u() * 2;
		W = pow(U1, 2) + pow(U2, 2);
	} while (W >= 1 || W == 0);

	mult = sqrt((-2 * log(W)) / W);
	X1 = U1 * mult;
	X2 = U2 * mult;

	call = !call;

	return (mu + sigma * (double)X1);
}

void CHANNEL::Set_Channel_Parameters()
{

	////////////////////////// Parameter setting /////////////////////////////////////////////////////
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// M.2412-0 page 62

	//////////////////////////////////////////////////////////////////// InH
	if (TYPE == 11) 
	{
		//////////////////////////////////////////////////// Channel Model A
		if (Channel_Model_Type == 0) 
		{

			BS_d_H = 0.5 * Wavelength;
			BS_d_V = 0.5 * Wavelength;
			BS_d_gH = 0.5 * Wavelength;
			BS_d_gV = 0.5 * Wavelength;

			MS_d_H = 0.5 * Wavelength;
			MS_d_V = 0.5 * Wavelength;
			MS_d_gH = 0.5 * Wavelength;
			MS_d_gV = 0.5 * Wavelength;

			alpha = 0.;  /// bearing angle  ///// decided by sector_idx
			beta = 90.;   /// downtilt angle
			gamma = 45.;  /// slant angle

			if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
			{
				if (LOS == 1) // LOS
				{
					num_path = 15; // Number of clusters
					r_tau = 3.6;

					//K_factor = normal(7, 4);
					mu_K_factor = 7.;
					sigma_K_factor = 4.;
					K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); // linear

					mu_DS = -7.70;
					sigma_DS = 0.18;
					DS = pow(10, normal(-7.70, 0.18)); // = sigma_tau in M.2135

					mu_ASD = 1.6;
					sigma_ASD = 0.18;
					mu_ASA = 1.62;
					sigma_ASA = 0.22;

					mu_ZSA = 1.22;
					sigma_ZSA = 0.23;
					mu_ZSD = 1.02;
					sigma_ZSD = 0.41;
					mu_offset_ZOD = 0;

					cluster_DS = -1;       /// N/A
					cluster_ASD = 5;
					cluster_ASA = 8;
					cluster_ZSA = 9;

					XPR = normal(11, 4 * 4);
					cluster_shadowing = 6;

					if (NUM_TX == 1)   ///// 1x2 calibration case
					{
						d_rx = 0.5*Wavelength;
						d_tx = 10 * Wavelength;
					}
					else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
					{
						d_rx = 0.5*Wavelength;
						d_tx = 0.5 * Wavelength;
					}
				}
				else if (LOS == 0)  //// NLOS
				{
					num_path = 19; // Number of clusters
					r_tau = 3;

					K_factor = -1;       // N/A
					mu_K_factor = -1;
					sigma_K_factor = -1;

					mu_DS = -7.41;
					sigma_DS = 0.14;
					DS = pow(10, normal(-7.41, 0.14)); // = sigma_tau in M.2135

					mu_ASD = 1.62;
					sigma_ASD = 0.25;
					mu_ASA = 1.77;
					sigma_ASA = 0.16;

					mu_ZSA = 1.26;
					sigma_ZSA = 0.67;
					mu_ZSD = 1.08;
					sigma_ZSD = 0.36;
					mu_offset_ZOD = 0;

					cluster_DS = -1;       /// N/A
					cluster_ASD = 5;
					cluster_ASA = 11;
					cluster_ZSA = 9;

					XPR = normal(10, 4 * 4);
					cluster_shadowing = 3;

					if (NUM_TX == 1)   ///// 1x2 calibration case
					{
						d_rx = 0.5*Wavelength;
						d_tx = 10 * Wavelength;
					}
					else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
					{
						d_rx = 0.5*Wavelength;
						d_tx = 0.5 * Wavelength;
					}
				}
			}
			else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
			{
				if (LOS == 1) // LOS
				{
					num_path = 15; // Number of clusters
					r_tau = 3.6;

					//K_factor = normal(7, 4);
					mu_K_factor = 7.;
					sigma_K_factor = 4.;
					K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

					mu_DS = -0.01 * log10(1+(Frequency/1000000000)) - 7.692;
					sigma_DS = 0.18;
					DS = pow(10, normal(mu_DS, 0.18)); // = sigma_tau in M.2135

					mu_ASD = 1.6;
					sigma_ASD = 0.18;
					mu_ASA = -0.19 * log10(1+(Frequency/1000000000)) + 1.781;
					sigma_ASA = 0.12 * log10(1 + (Frequency / 1000000000)) + 0.119;

					mu_ZSA = -0.26 * log10(1 + (Frequency / 1000000000)) + 1.44;
					sigma_ZSA = -0.04 * log10(1 + (Frequency / 1000000000)) + 0.264;
					mu_ZSD = -1.43 * log10(1 + (Frequency / 1000000000)) + 2.228;
					sigma_ZSD = 0.13 * log10(1 + (Frequency / 1000000000)) + 0.3;
					mu_offset_ZOD = 0;

					cluster_DS = -1;       /// N/A
					cluster_ASD = 5;
					cluster_ASA = 8;
					cluster_ZSA = 9;

					XPR = normal(11, 4 * 4);
					cluster_shadowing = 6;

					if (NUM_TX == 1)   ///// 1x2 calibration case
					{
						d_rx = 0.5*Wavelength;
						d_tx = 10 * Wavelength;
					}
					else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
					{
						d_rx = 0.5*Wavelength;
						d_tx = 0.5 * Wavelength;
					}
				}
				else if (LOS == 0)  //// NLOS
				{
					num_path = 19; // Number of clusters
					r_tau = 3;

					K_factor = -1;       // N/A
					mu_K_factor = -1;
					sigma_K_factor = -1;

					mu_DS = -0.28 * log10(1 + (Frequency / 1000000000)) - 7.173;
					sigma_DS = 0.1 * log10(1 + (Frequency / 1000000000)) + 0.055;
					DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

					mu_ASD = 1.62;
					sigma_ASD = 0.25;
					mu_ASA = -0.11 * log10(1 + (Frequency / 1000000000)) + 1.863;
					sigma_ASA = 0.12 * log10(1 + (Frequency / 1000000000)) + 0.059;

					mu_ZSA = -0.15 * log10(1 + (Frequency / 1000000000)) + 1.387;
					sigma_ZSA = -0.09 * log10(1 + (Frequency / 1000000000)) + 0.746;
					mu_ZSD = 1.08;
					sigma_ZSD = 0.36;
					mu_offset_ZOD = 0;

					cluster_DS = -1;       /// N/A
					cluster_ASD = 5;
					cluster_ASA = 11;
					cluster_ZSA = 9;

					XPR = normal(10, 4 * 4);
					cluster_shadowing = 3;

					if (NUM_TX == 1)   ///// 1x2 calibration case
					{
						d_rx = 0.5*Wavelength;
						d_tx = 10 * Wavelength;
					}
					else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
					{
						d_rx = 0.5*Wavelength;
						d_tx = 0.5 * Wavelength;
					}
				}

			}
			else
			{
				cout << "Frequency ERROR! - Set channel parameter" << endl;
			}
			

		}
		//////////////////////////////////////////////////// Channel Model B
		else if (Channel_Model_Type == 1) 
		{
			BS_d_H = 0.5 * Wavelength;
			BS_d_V = 0.5 * Wavelength;
			BS_d_gH = 0.5 * Wavelength;
			BS_d_gV = 0.5 * Wavelength;

			MS_d_H = 0.5 * Wavelength;
			MS_d_V = 0.5 * Wavelength;
			MS_d_gH = 0.5 * Wavelength;
			MS_d_gV = 0.5 * Wavelength;

			alpha = 0.;  /// bearing angle  ///// decided by sector_idx
			beta = 90.;   /// downtilt angle
			gamma = 45.;  /// slant angle

			if (Frequency >= 500000000 && Frequency <= 100000000000) //// 0.5GHz <= fc <= 100GHz
			{
				if (LOS == 1) // LOS
				{
					num_path = 15; // Number of clusters
					r_tau = 3.6;

					//K_factor = normal(7, 4);
					mu_K_factor = 7.;
					sigma_K_factor = 4.;
					K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

					mu_DS = -0.01 * log10(1 + (Frequency / 1000000000)) - 7.692;
					sigma_DS = 0.18;
					DS = pow(10, normal(mu_DS, 0.18)); // = sigma_tau in M.2135

					mu_ASD = 1.6;
					sigma_ASD = 0.18;
					mu_ASA = -0.19 * log10(1 + (Frequency / 1000000000)) + 1.781;
					sigma_ASA = 0.12 * log10(1 + (Frequency / 1000000000)) + 0.119;

					mu_ZSA = -0.26 * log10(1 + (Frequency / 1000000000)) + 1.44;
					sigma_ZSA = -0.04 * log10(1 + (Frequency / 1000000000)) + 0.264;
					mu_ZSD = -1.43 * log10(1 + (Frequency / 1000000000)) + 2.228;
					sigma_ZSD = 0.13 * log10(1 + (Frequency / 1000000000)) + 0.3;
					mu_offset_ZOD = 0;

					cluster_DS = -1;       /// N/A
					cluster_ASD = 5;
					cluster_ASA = 8;
					cluster_ZSA = 9;

					XPR = normal(11, 4 * 4);
					cluster_shadowing = 6;

					if (NUM_TX == 1)   ///// 1x2 calibration case
					{
						d_rx = 0.5*Wavelength;
						d_tx = 10 * Wavelength;
					}
					else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
					{
						d_rx = 0.5*Wavelength;
						d_tx = 0.5 * Wavelength;
					}
				}
				else if (LOS == 0)  //// NLOS
				{
					num_path = 19; // Number of clusters
					r_tau = 3;

					K_factor = -1;       // N/A
					mu_K_factor = -1;
					sigma_K_factor = -1;

					mu_DS = -0.28 * log10(1 + (Frequency / 1000000000)) - 7.173;
					sigma_DS = 0.1 * log10(1 + (Frequency / 1000000000)) + 0.055;
					DS = pow(10, normal(-7.41, 0.14)); // = sigma_tau in M.2135

					mu_ASD = 1.62;
					sigma_ASD = 0.25;
					mu_ASA = -0.11 * log10(1 + (Frequency / 1000000000)) + 1.863;
					sigma_ASA = 0.12 * log10(1 + (Frequency / 1000000000)) + 0.059;

					mu_ZSA = -0.15 * log10(1 + (Frequency / 1000000000)) + 1.387;
					sigma_ZSA = -0.09 * log10(1 + (Frequency / 1000000000)) + 0.746;
					mu_ZSD = 1.08;
					sigma_ZSD = 0.36;
					mu_offset_ZOD = 0;

					cluster_DS = -1;       /// N/A
					cluster_ASD = 5;
					cluster_ASA = 11;
					cluster_ZSA = 9;

					XPR = normal(10, 4 * 4);
					cluster_shadowing = 3;

					if (NUM_TX == 1)   ///// 1x2 calibration case
					{
						d_rx = 0.5*Wavelength;
						d_tx = 10 * Wavelength;
					}
					else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
					{
						d_rx = 0.5*Wavelength;
						d_tx = 0.5 * Wavelength;
					}
				}
			}
			else
			{
				cout << "Frequency ERROR! - Set channel parameter" << endl;
			}
		}

	}
	//////////////////////////////////////////////////////////////////////// Dense_Urban
	else if (TYPE == 12) 
	{
		////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////// UMi
		if (Configuration_Type == 3)
		{
			/////////////////////////////////////////////////////////////////////// Channel Model A
			if (Channel_Model_Type == 0)
			{
				BS_d_H = 0.5 * Wavelength;
				BS_d_V = 0.5 * Wavelength;
				BS_d_gH = 0.5 * Wavelength;
				BS_d_gV = 0.5 * Wavelength;

				MS_d_H = 0.5 * Wavelength;
				MS_d_V = 0.5 * Wavelength;
				MS_d_gH = 0.5 * Wavelength;
				MS_d_gV = 0.5 * Wavelength;

				alpha = 0.;  /// bearing angle  ///// decided by sector_idx
				beta = 90.;   /// downtilt angle
				gamma = 45.;  /// slant angle

				if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
				{
					if (Propagation == 1) // LOS
					{
						num_path = 12; // Number of clusters
						r_tau = 3.2;

						//K_factor = normal(9, 5);
						mu_K_factor = 9.;
						sigma_K_factor = 5;
						K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

						mu_DS = -7.19;
						sigma_DS = 0.40;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.2;
						sigma_ASD = 0.43;
						mu_ASA = 1.75;
						sigma_ASA = 0.19;

						mu_ZSA = 0.6;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * abs(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height) + 0.75);
						sigma_ZSD = 0.4;
						mu_offset_ZOD = 0;

						cluster_DS = -1;       /// N/A
						cluster_ASD = 3;
						cluster_ASA = 17;
						cluster_ZSA = 7;

						XPR = normal(9, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 0)  //// NLOS
					{
						num_path = 19; // Number of clusters
						r_tau = 3;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.89;
						sigma_DS = 0.54;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.41;
						sigma_ASD = 0.17;
						mu_ASA = 1.84;
						sigma_ASA = 0.15;

						mu_ZSA = 0.88;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * MAX((ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height),0) + 0.9);
						sigma_ZSD = 0.6;
						mu_offset_ZOD = -1 * pow(10, -0.55 * log10(MAX(10, distance)) + 1.6);

						cluster_DS = -1;       /// N/A
						cluster_ASD = 10;
						cluster_ASA = 22;
						cluster_ZSA = 7;

						XPR = normal(8, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 2)   ///// OUT2IN
					{
						num_path = 12; // Number of clusters
						r_tau = 2.2;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.62;
						sigma_DS = 0.32;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.25;
						sigma_ASD = 0.42;
						mu_ASA = 1.76;
						sigma_ASA = 0.16;

						mu_ZSA = 1.01;
						sigma_ZSA = 0.43;

						if (LOS == 1)
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * abs(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height) + 0.75);
							sigma_ZSD = 0.4;
							mu_offset_ZOD = 0;
						}
						else
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * MAX((ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height), 0) + 0.9);
							sigma_ZSD = 0.6;
							mu_offset_ZOD = -1 * pow(10, -0.55 * log10(MAX(10, distance)) + 1.6);
						}

						cluster_DS = -1;       /// N/A
						cluster_ASD = 5;
						cluster_ASA = 8;
						cluster_ZSA = 3;

						XPR = normal(9, 11 * 11);
						cluster_shadowing = 4;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
				}
				else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
				{
					if (Propagation == 1) // LOS
					{
						num_path = 12; // Number of clusters
						r_tau = 3;

						//K_factor = normal(9, 3.5);
						mu_K_factor = 9.;
						sigma_K_factor = 5;
						K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

						mu_DS = -7.14 - 0.24 * log10(1 + (Frequency / 1000000000));
						sigma_DS = 0.38;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.21 - 0.05 * log10(1 + (Frequency / 1000000000));
						sigma_ASD = 0.41;
						mu_ASA = 1.73 - 0.08 * log10(1 + (Frequency / 1000000000));
						sigma_ASA = 0.28 + 0.014 * log10(1 + (Frequency / 1000000000));

						mu_ZSA = 0.73 - 0.1 * log10(1 + (Frequency / 1000000000));
						sigma_ZSA = 0.34 - 0.04 * log10(1 + (Frequency / 1000000000));
						mu_ZSD = MAX(-0.21, -14.8*(distance / 1000) + 0.01 * abs(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height) + 0.83);
						sigma_ZSD = 0.35;
						mu_offset_ZOD = 0;

						cluster_DS = 5;
						cluster_ASD = 3;
						cluster_ASA = 17;
						cluster_ZSA = 7;

						XPR = normal(9, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 0)  //// NLOS
					{
						num_path = 20; // Number of clusters
						r_tau = 2.3;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.83 - 0.24*log10(1+(Frequency / 1000000000));
						sigma_DS = 0.28 + 0.16*log10(1 + (Frequency / 1000000000));
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.53 - 0.23 * log10(1 + (Frequency / 1000000000));
						sigma_ASD = 0.33 + 0.11 * log10(1 + (Frequency / 1000000000));
						mu_ASA = 1.81 - 0.08 * log10(1 + (Frequency / 1000000000));
						sigma_ASA = 0.3 + 0.05 * log10(1 + (Frequency / 1000000000));

						mu_ZSA = 0.92 - 0.04 * log10(1 + (Frequency / 1000000000));
						sigma_ZSA = 0.41 - 0.07 * log10(1 + (Frequency / 1000000000));
						mu_ZSD = MAX(-0.5, -3.1*(distance / 1000) + 0.01 * MAX(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height,0) + 0.2);
						sigma_ZSD = 0.35;
						mu_offset_ZOD = -1 * pow(10, -1.5 * log10(MAX(10, distance)) + 3.3);

						cluster_DS = 11;
						cluster_ASD = 10;
						cluster_ASA = 22;
						cluster_ZSA = 7;

						XPR = normal(8, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 2) ///// OUT2IN
					{
						num_path = 12; // Number of clusters
						r_tau = 2.2;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.62;
						sigma_DS = 0.32;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.25;
						sigma_ASD = 0.42;
						mu_ASA = 1.76;
						sigma_ASA = 0.16;

						mu_ZSA = 1.01;
						sigma_ZSA = 0.43;
						if (LOS == 1)
						{
							mu_ZSD = MAX(-0.21, -14.8*(distance / 1000) + 0.01 * abs(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height) + 0.83);
							sigma_ZSD = 0.35;
							mu_offset_ZOD = 0;
						}
						else
						{
							mu_ZSD = MAX(-0.5, -3.1*(distance / 1000) + 0.01 * MAX(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height, 0) + 0.2);
							sigma_ZSD = 0.35;
							mu_offset_ZOD = -1 * pow(10, -1.5 * log10(MAX(10, distance)) + 3.3);
						}

						cluster_DS = 11;
						cluster_ASD = 5;
						cluster_ASA = 20;
						cluster_ZSA = 6;

						XPR = normal(9, 5 * 5);
						cluster_shadowing = 4;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}

				}
				else
				{
					cout << "Frequency ERROR! - Set channel parameter" << endl;
				}
			}
			////////////////////////////////////////////////////////// Channel Model B
			else if (Channel_Model_Type == 1)
			{
				BS_d_H = 0.5 * Wavelength;
				BS_d_V = 0.5 * Wavelength;
				BS_d_gH = 0.5 * Wavelength;
				BS_d_gV = 0.5 * Wavelength;

				MS_d_H = 0.5 * Wavelength;
				MS_d_V = 0.5 * Wavelength;
				MS_d_gH = 0.5 * Wavelength;
				MS_d_gV = 0.5 * Wavelength;

				alpha = 0.;  /// bearing angle  ///// decided by sector_idx
				beta = 90.;   /// downtilt angle
				gamma = 45.;  /// slant angle

				if (Frequency > 500000000 && Frequency <= 100000000000) /// 0.5GHz < fc <= 100GHz
				{
					if (Propagation == 1) // LOS
					{
						num_path = 12; // Number of clusters
						r_tau = 3;

						//K_factor = normal(9, 5);
						mu_K_factor = 9.;
						sigma_K_factor = 5;
						K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

						mu_DS = -7.14 - 0.24 * log10(1 + (Frequency / 1000000000));
						sigma_DS = 0.38;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.21 - 0.05 * log10(1 + (Frequency / 1000000000));
						sigma_ASD = 0.41;
						mu_ASA = 1.73 - 0.08 * log10(1 + (Frequency / 1000000000));
						sigma_ASA = 0.28 + 0.014 * log10(1 + (Frequency / 1000000000));

						mu_ZSA = 0.73 - 0.1 * log10(1 + (Frequency / 1000000000));
						sigma_ZSA = 0.34 - 0.04 * log10(1 + (Frequency / 1000000000));
						mu_ZSD = MAX(-0.21, -14.8*(distance / 1000) + 0.01 * abs(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height) + 0.83);
						sigma_ZSD = 0.35;
						mu_offset_ZOD = 0;

						cluster_DS = 5;
						cluster_ASD = 3;
						cluster_ASA = 17;
						cluster_ZSA = 7;

						XPR = normal(9, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 0)  //// NLOS
					{
						num_path = 19; // Number of clusters
						r_tau = 2.1;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.83 - 0.24 * log10(1 + (Frequency / 1000000000));
						sigma_DS = 0.28 + 0.16 * log10(1 + (Frequency / 1000000000));
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.53 - 0.23 * log10(1 + (Frequency / 1000000000));
						sigma_ASD = 0.33 + 0.11 * log10(1 + (Frequency / 1000000000));
						mu_ASA = 1.81 - 0.08 * log10(1 + (Frequency / 1000000000));
						sigma_ASA = 0.3 + 0.05 * log10(1 + (Frequency / 1000000000));

						mu_ZSA = 0.92 - 0.04 * log10(1 + (Frequency / 1000000000));
						sigma_ZSA = 0.41 - 0.07 * log10(1 + (Frequency / 1000000000));
						mu_ZSD = MAX(-0.5, -3.1*(distance / 1000) + 0.01*MAX(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height,0) + 0.2);
						sigma_ZSD = 0.35;
						mu_offset_ZOD = -1 * pow(10, -1.5 * log10(MAX(10, distance)) + 3.3);

						cluster_DS = 11;
						cluster_ASD = 5;
						cluster_ASA = 8;
						cluster_ZSA = 3;

						XPR = normal(9, 5 * 5);
						cluster_shadowing = 4;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 2) ///// OUT2IN
					{
						num_path = 12; // Number of clusters
						r_tau = 2.2;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.62;
						sigma_DS = 0.32;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.25;
						sigma_ASD = 0.42;
						mu_ASA = 1.25;
						sigma_ASA = 0.42;

						mu_ZSA = 1.01;
						sigma_ZSA = 0.43;

						if (LOS == 1)
						{
							mu_ZSD = MAX(-0.21, -14.8*(distance / 1000) + 0.01 * abs(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height) + 0.83);
							sigma_ZSD = 0.35;
							mu_offset_ZOD = 0;
						}
						else
						{
							mu_ZSD = MAX(-0.5, -3.1*(distance / 1000) + 0.01*MAX(ms[self_ms_idx].MS_HEIGHT_FINAL - bs_height, 0) + 0.2);
							sigma_ZSD = 0.35;
							mu_offset_ZOD = -1 * pow(10, -1.5 * log10(MAX(10, distance)) + 3.3);
						}

						cluster_DS = 11;
						cluster_ASD = 5;
						cluster_ASA = 8;
						cluster_ZSA = 3;

						XPR = normal(9, 5 * 5);
						cluster_shadowing = 4;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}

				}
				else
				{
					cout << "Frequency ERROR! - Set channel parameter" << endl;
				}
			}
		}
		///////////////////////////////////////////////////////////////////////////
		/////////////////////////////////////////////////////////////////////////// UMa
		else
		{
			/////////////////////////////////////////////////////////////////////// Channel Model A
			if (Channel_Model_Type == 0)
			{
				BS_d_H = 0.5 * Wavelength;
				BS_d_V = 0.5 * Wavelength;
				BS_d_gH = 0.5 * Wavelength;
				BS_d_gV = 0.5 * Wavelength;

				MS_d_H = 0.5 * Wavelength;
				MS_d_V = 0.5 * Wavelength;
				MS_d_gH = 0.5 * Wavelength;
				MS_d_gV = 0.5 * Wavelength;

				alpha = 0.;  /// bearing angle  ///// decided by sector_idx
				beta = 90.;   /// downtilt angle
				gamma = 45.;  /// slant angle

				if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
				{
					if (Propagation == 1) // LOS
					{
						num_path = 12; // Number of clusters
						r_tau = 2.5;

						//K_factor = normal(9, 3.5);
						mu_K_factor = 9.;
						sigma_K_factor = 3.5;
						K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

						mu_DS = -7.03;
						sigma_DS = 0.66;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.15;
						sigma_ASD = 0.28;
						mu_ASA = 1.81;
						sigma_ASA = 0.2;

						mu_ZSA = 0.95;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.75);
						sigma_ZSD = 0.4;
						mu_offset_ZOD = 0;

						cluster_DS = -1;       /// N/A
						cluster_ASD = 5;
						cluster_ASA = 11;
						cluster_ZSA = 7;

						XPR = normal(8, 4 * 4);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 0)  //// NLOS
					{
						num_path = 20; // Number of clusters
						r_tau = 2.3;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.44;
						sigma_DS = 0.39;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.41;
						sigma_ASD = 0.28;
						mu_ASA = 1.87;
						sigma_ASA = 0.11;

						mu_ZSA = 1.26;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.9);
						sigma_ZSD = 0.49;
						mu_offset_ZOD = -1 * pow(10, -0.62 * log10(MAX(10, distance)) + 1.93 - 0.07*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5));


						cluster_DS = -1;       /// N/A
						cluster_ASD = 2;
						cluster_ASA = 15;
						cluster_ZSA = 7;

						XPR = normal(7, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 2)   ///// OUT2IN
					{
						num_path = 12; // Number of clusters
						r_tau = 2.2;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.62;
						sigma_DS = 0.32;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.25;
						sigma_ASD = 0.42;
						mu_ASA = 1.76;
						sigma_ASA = 0.16;

						mu_ZSA = 1.01;
						sigma_ZSA = 0.43;

						if (LOS == 1)
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.75);
							sigma_ZSD = 0.4;
							mu_offset_ZOD = 0;
						}
						else
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.9);
							sigma_ZSD = 0.49;
							mu_offset_ZOD = -1 * pow(10, -0.62 * log10(MAX(10, distance)) + 1.93 - 0.07*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5));

							///// -10 -  . -10 Ǿ mu_offset_ZOD -1#IND  찡 ־... 
						}

						/*
						if (self_ms_idx == 4)
						{
							cout << "Propagation = " << Propagation << endl;
							cout << "LOS = " << LOS << endl;
							cout << "distance = " << distance << endl;
							cout << "ms[self_ms_idx].MS_HEIGHT_FINAL = " << ms[self_ms_idx].MS_HEIGHT_FINAL << endl;
							cout << "mu_offset = " << mu_offset_ZOD << endl;
							getchar();
						}
						*/

						cluster_DS = -1;       /// N/A
						cluster_ASD = 5;
						cluster_ASA = 8;
						cluster_ZSA = 3;

						XPR = normal(9, 5 * 5);
						cluster_shadowing = 4;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
				}
				else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
				{
					if (Propagation == 1) // LOS
					{
						num_path = 12; // Number of clusters
						r_tau = 2.5;

						//K_factor = normal(9, 3.5);
						mu_K_factor = 9.;
						sigma_K_factor = 3.5;
						K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

						mu_DS = -6.955 - 0.0963*log10(Frequency / 1000000000);
						sigma_DS = 0.66;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.06 + 0.1114*log10(Frequency / 1000000000);
						sigma_ASD = 0.28;
						mu_ASA = 1.81;
						sigma_ASA = 0.2;

						mu_ZSA = 0.95;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.75);
						sigma_ZSD = 0.4;
						mu_offset_ZOD = 0;

						cluster_DS = MAX(0.25, -3.4084*log10(Frequency / 1000000000) + 6.5622);
						cluster_ASD = 5;
						cluster_ASA = 11;
						cluster_ZSA = 7;

						XPR = normal(8, 4 * 4);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 0)  //// NLOS
					{
						num_path = 20; // Number of clusters
						r_tau = 2.3;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.28 - 0.204*log10(Frequency / 1000000000);
						sigma_DS = 0.39;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.5 - 0.1144*log10(Frequency / 1000000000);
						sigma_ASD = 0.28;
						mu_ASA = 2.08 - 0.27*log10(Frequency / 1000000000);
						sigma_ASA = 0.11;

						mu_ZSA = -0.3236*log10(Frequency / 1000000000) + 1.512;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.9);
						sigma_ZSD = 0.49;
						mu_offset_ZOD = 7.66*log10(Frequency / 1000000000) - 5.96 - pow(10, (0.208*log10(Frequency / 1000000000) - 0.782)*log10(MAX(25, distance)) - 0.13*log10(Frequency / 1000000000) + 2.03);

						cluster_DS = MAX(0.25, -3.4084*log10(Frequency / 1000000000) + 6.5622);
						cluster_ASD = 2;
						cluster_ASA = 15;
						cluster_ZSA = 7;

						XPR = normal(7, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 2) ///// OUT2IN
					{
						num_path = 12; // Number of clusters
						r_tau = 2.2;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.63;
						sigma_DS = 0.32;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.25;
						sigma_ASD = 0.42;
						mu_ASA = 1.76;
						sigma_ASA = 0.16;

						mu_ZSA = 1.01;
						sigma_ZSA = 0.43;
						if (LOS == 1)
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.75);
							sigma_ZSD = 0.4;
							mu_offset_ZOD = 0;
						}
						else
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.9);
							sigma_ZSD = 0.49;
							mu_offset_ZOD = 7.66*log10(Frequency / 1000000000) - 5.96 - pow(10, (0.208*log10(Frequency / 1000000000) - 0.782)*log10(MAX(25, distance)) - 0.13*log10(Frequency / 1000000000) + 2.03);
						}

						cluster_DS = 11;
						cluster_ASD = 5;
						cluster_ASA = 8;
						cluster_ZSA = 3;

						XPR = normal(9, 5 * 5);
						cluster_shadowing = 4;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}

				}
				else
				{
					cout << "Frequency ERROR! - Set channel parameter" << endl;
				}
			}
			////////////////////////////////////////////////////////// Channel Model B
			else if (Channel_Model_Type == 1)
			{
				BS_d_H = 0.5 * Wavelength;
				BS_d_V = 0.5 * Wavelength;
				BS_d_gH = 0.5 * Wavelength;
				BS_d_gV = 0.5 * Wavelength;

				MS_d_H = 0.5 * Wavelength;
				MS_d_V = 0.5 * Wavelength;
				MS_d_gH = 0.5 * Wavelength;
				MS_d_gV = 0.5 * Wavelength;

				alpha = 0.;  /// bearing angle  ///// decided by sector_idx
				beta = 90.;   /// downtilt angle
				gamma = 45.;  /// slant angle

				if (Frequency > 500000000 && Frequency <= 100000000000) /// 0.5GHz < fc <= 100GHz
				{
					if (Propagation == 1) // LOS
					{
						num_path = 12; // Number of clusters
						r_tau = 2.5;

						//K_factor = normal(9, 3.5);
						mu_K_factor = 9.;
						sigma_K_factor = 3.5;
						K_factor = pow(10, normal(mu_K_factor, sigma_K_factor) / 10); //

						mu_DS = -6.955 - 0.0963*log10(Frequency / 1000000000);
						sigma_DS = 0.66;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.06 + 0.1114*log10(Frequency / 1000000000);
						sigma_ASD = 0.28;
						mu_ASA = 1.81;
						sigma_ASA = 0.2;

						mu_ZSA = 0.95;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.75);
						sigma_ZSD = 0.4;
						mu_offset_ZOD = 0;

						cluster_DS = MAX(0.25, -3.4084*log10(Frequency / 1000000000) + 6.5622);
						cluster_ASD = 5;
						cluster_ASA = 11;
						cluster_ZSA = 7;

						XPR = normal(8, 4 * 4);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 0)  //// NLOS
					{
						num_path = 20; // Number of clusters
						r_tau = 2.3;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.28 - 0.204*log10(Frequency / 1000000000);
						sigma_DS = 0.39;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.5 - 0.1144*log10(Frequency / 1000000000);
						sigma_ASD = 0.28;
						mu_ASA = 2.08 - 0.27*log10(Frequency / 1000000000);
						sigma_ASA = 0.11;

						mu_ZSA = -0.3236*log10(Frequency / 1000000000) + 1.512;
						sigma_ZSA = 0.16;
						mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.9);
						sigma_ZSD = 0.49;
						mu_offset_ZOD = 7.66*log10(Frequency / 1000000000) - 5.96 - pow(10, (0.208*log10(Frequency / 1000000000) - 0.782)*log10(MAX(25, distance)) - 0.13*log10(Frequency / 1000000000) + 2.03 - 0.07*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5));

						cluster_DS = MAX(0.25, -3.4084*log10(Frequency / 1000000000) + 6.5622);
						cluster_ASD = 2;
						cluster_ASA = 15;
						cluster_ZSA = 7;

						XPR = normal(7, 3 * 3);
						cluster_shadowing = 3;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}
					else if (Propagation == 2) ///// OUT2IN
					{
						num_path = 12; // Number of clusters
						r_tau = 2.2;

						K_factor = -1;       // N/A
						mu_K_factor = -1;
						sigma_K_factor = -1;

						mu_DS = -6.62;
						sigma_DS = 0.32;
						DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

						mu_ASD = 1.25;
						sigma_ASD = 0.42;
						mu_ASA = 1.76;
						sigma_ASA = 0.16;

						mu_ZSA = 1.01;
						sigma_ZSA = 0.43;
						if (LOS == 1)
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.75);
							sigma_ZSD = 0.4;
							mu_offset_ZOD = 0;
						}
						else
						{
							mu_ZSD = MAX(-0.5, -2.1*(distance / 1000) - 0.01*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.9);
							sigma_ZSD = 0.49;
							mu_offset_ZOD = 7.66*log10(Frequency / 1000000000) - 5.96 - pow(10, (0.208*log10(Frequency / 1000000000) - 0.782)*log10(MAX(25, distance)) - 0.13*log10(Frequency / 1000000000) + 2.03 - 0.07*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5));
						}

						cluster_DS = 11;
						cluster_ASD = 5;
						cluster_ASA = 8;
						cluster_ZSA = 3;

						XPR = normal(9, 5 * 5);
						cluster_shadowing = 4;

						if (NUM_TX == 1)   ///// 1x2 calibration case
						{
							d_rx = 0.5*Wavelength;
							d_tx = 10 * Wavelength;
						}
						else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
						{
							d_rx = 0.5*Wavelength;
							d_tx = 0.5 * Wavelength;
						}
					}

				}
				else
				{
					cout << "Frequency ERROR! - Set channel parameter" << endl;
				}


			}
		}




		



	}
	/////////////////////////////////////////////////// RMa
	else if (TYPE == 13) 
	{
		BS_d_H = 0.5 * Wavelength;
		BS_d_V = 0.5 * Wavelength;
		BS_d_gH = 0.5 * Wavelength;
		BS_d_gV = 0.5 * Wavelength;

		MS_d_H = 0.5 * Wavelength;
		MS_d_V = 0.5 * Wavelength;
		MS_d_gH = 0.5 * Wavelength;
		MS_d_gV = 0.5 * Wavelength;

		alpha = 0.;  /// bearing angle  ///// decided by sector_idx
		beta = 90.;   /// downtilt angle
		gamma = 45.;  /// slant angle

		if (Frequency > 500000000 && Frequency <= 7000000000) /// RMm_A:0.5GHz < fc <= 6GHz, RMa_B:0.5GHz < fc <= 7GHz
		{
			if (Propagation == 1) // LOS
			{
				num_path = 11; // Number of clusters
				r_tau = 3.8;

				//K_factor = normal(7, 4);
				mu_K_factor = 7.;
				sigma_K_factor = 4.;
				K_factor = pow(10, normal(mu_K_factor, sigma_K_factor)/10 ); //

				mu_DS = -7.49;
				sigma_DS = 0.55;
				DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

				mu_ASD = 0.9;
				sigma_ASD = 0.38;
				mu_ASA = 1.52;
				sigma_ASA = 0.24;

				mu_ZSA = 0.47;
				sigma_ZSA = 0.4;
				mu_ZSD = MAX(-1, -0.17*(distance / 1000) - 0.01 * (ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.22);
				sigma_ZSD = 0.34;
				mu_offset_ZOD = 0;

				cluster_DS = -1;  /// N/A
				cluster_ASD = 2;
				cluster_ASA = 3;
				cluster_ZSA = 3;

				XPR = normal(12, 4 * 4);
				cluster_shadowing = 3;

				if (NUM_TX == 1)   ///// 1x2 calibration case
				{
					d_rx = 0.5*Wavelength;
					d_tx = 10 * Wavelength;
				}
				else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
				{
					d_rx = 0.5*Wavelength;
					d_tx = 0.5 * Wavelength;
				}
			}
			else if (Propagation == 0)  //// NLOS
			{
				num_path = 10; // Number of clusters
				r_tau = 1.7;

				K_factor = -1;       // N/A
				mu_K_factor = -1;
				sigma_K_factor = -1;

				mu_DS = -7.43;
				sigma_DS = 0.48;
				DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

				mu_ASD = 0.95;
				sigma_ASD = 0.45;
				mu_ASA = 1.52;
				sigma_ASA = 0.13;

				mu_ZSA = 0.58;
				sigma_ZSA = 0.37;
				mu_ZSD = MAX(-1, -0.19*(distance / 1000) - 0.01*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.28);
				sigma_ZSD = 0.3;
				mu_offset_ZOD = atan((35-3.5)/distance) - atan((35-1.5)/distance);

				cluster_DS = -1; /// N/A
				cluster_ASD = 2;
				cluster_ASA = 3;
				cluster_ZSA = 3;

				XPR = normal(7, 3 * 3);
				cluster_shadowing = 3;

				if (NUM_TX == 1)   ///// 1x2 calibration case
				{
					d_rx = 0.5*Wavelength;
					d_tx = 10 * Wavelength;
				}
				else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
				{
					d_rx = 0.5*Wavelength;
					d_tx = 0.5 * Wavelength;
				}
			}
			else if (Propagation == 2) ///// OUT2IN
			{
				num_path = 10; // Number of clusters
				r_tau = 1.7;

				K_factor = -1;       // N/A
				mu_K_factor = -1;
				sigma_K_factor = -1;

				mu_DS = -7.47;
				sigma_DS = 0.24;
				DS = pow(10, normal(mu_DS, sigma_DS)); // = sigma_tau in M.2135

				mu_ASD = 0.67;
				sigma_ASD = 0.18;
				mu_ASA = 1.66;
				sigma_ASA = 0.21;

				mu_ZSA = 0.93;
				sigma_ZSA = 0.22;
				mu_ZSD = MAX(-1, -0.19*(distance / 1000) - 0.01*(ms[self_ms_idx].MS_HEIGHT_FINAL - 1.5) + 0.28);
				sigma_ZSD = 0.3;
				mu_offset_ZOD = atan((35 - 3.5) / distance) - atan((35 - 1.5) / distance);
				

				cluster_DS = -1;  /// N/A
				cluster_ASD = 2;
				cluster_ASA = 3;
				cluster_ZSA = 3;

				XPR = normal(7, 3 * 3);
				cluster_shadowing = 3;

				if (NUM_TX == 1)   ///// 1x2 calibration case
				{
					d_rx = 0.5*Wavelength;
					d_tx = 10 * Wavelength;
				}
				else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx = 0.5
				{
					d_rx = 0.5*Wavelength;
					d_tx = 0.5 * Wavelength;
				}
			}

		}
		else
		{
			cout << "Frequency ERROR! - Set channel parameter" << endl;
		}

	}



}

void CHANNEL::Set_DELAY()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- delay
	///
	/////////////////////////////////////////////////

	double K_factor_dB = K_dB;

	D = 0.7705 - 0.0433 * K_factor_dB + 0.0002*K_factor_dB*K_factor_dB + 0.000017*K_factor_dB*K_factor_dB*K_factor_dB;

	//delay = new double[MAX_NUM_CLUSTERS];
	//delay_LOS = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		delay[i] = 0.;
		delay_LOS[i] = 0.;

	} /// ̸ ϴ Hmκп MAX_NUM_CLUSTER numpath  ʾƼ   // idx num_path Ŭ  delay Ⱚ  ̷  0 ʱȭϵߴ

	for (int i = 0; i < num_path; i++)
	{
		delay[i] = -r_tau * _DS * log(randnum.u());	
	}

	//cout << "delay = " << delay[0] << endl;

	for (int i = 0; i < num_path; i++)
	{
		double temp;
		if (i == (num_path - 1))
		{
			break;
		}

		for (int j = i + 1; j < num_path; j++)
		{
			temp = delay[i];

			if (delay[j]<temp)
			{
				temp = delay[j];
				delay[j] = delay[i];
				delay[i] = temp;
			}
		}
	} // sort delay, small -> large

	double temp_delay = delay[0]; //minimum

	for (int i = 0; i < num_path; i++)
	{

		delay[i] = delay[i] - temp_delay;

	}

	// LOS CASE -> scaled delay for delay spread, not to be used in cluster power generation (M.2412 Small scale generation Step 5)
	for (int i = 0; i < num_path; i++)
	{

		delay_LOS[i] = delay[i] / D;
		delay[i] = delay[i];

	}


}

void CHANNEL::Set_POWER()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- power
	///
	/////////////////////////////////////////////////

	//power = new double[MAX_NUM_CLUSTERS];
	//power_LOS = new double[MAX_NUM_CLUSTERS];
	//power_NLOS = new double[MAX_NUM_CLUSTERS];   /// LOS, NLOS separate for pre-calc


	double sum_power = 0.;
	double sum_power_LOS = 0.;
	double sum_power_NLOS = 0.;

	//double k_R = pow(10, K_factor / 10);
	double k_R = K_linear;  ///// 

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		power[i] = 0.;
		power_LOS[i] = 0.;
		power_NLOS[i] = 0.;

	} //// initialize


	for (int i = 0; i < num_path; i++)
	{
		
		power[i] = exp(-delay[i] * (r_tau - 1.) / (r_tau*_DS)) * pow(10., -1 * normal(0, cluster_shadowing) / 10.);
		
		power_LOS[i] = power[i];    // delay LOS -> cluster power generation
		power_NLOS[i] = power[i];

		sum_power += power[i];

		sum_power_LOS += power_LOS[i];
		sum_power_NLOS += power_NLOS[i];


	}

	//cout << "power = " << power[0] << endl;

	if (LOS == 1)
	{
		for (int i = 0; i < num_path; i++)
		{
			power[i] = power[i] / (sum_power*(k_R + 1));
		}
		power[0] = power[0] + k_R / (k_R + 1);
	}
	else if (LOS == 0)
	{
		for (int i = 0; i < num_path; i++)
		{
			power[i] = power[i] / sum_power;
		}
	}




	///// ================================================================== ////
	////  LOS, NLOS case for precalculation                                  ////
	////  ================================================================== ////
	for (int i = 0; i < num_path; i++)
	{
		power_LOS[i] = power_LOS[i] / (sum_power_LOS * (k_R + 1));
	}
	power_LOS[0] = power_LOS[0] + k_R / (k_R + 1);

	for (int i = 0; i < num_path; i++)
	{
		power_NLOS[i] = power_NLOS[i] / sum_power_NLOS;
	}
	////  ================================================================== ////


}

void CHANNEL::Find_Strong2Clusters()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter --  find the 2 strongest clusters
	///
	/////////////////////////////////////////////////

	strongest_power = power[0];
	strongest_power2 = power[1];
	strongest_power_idx = 0;
	strongest_power_idx2 = 1;

	if (power[0] < power[1])
	{
		strongest_power = power[1];
		strongest_power2 = power[0];
		strongest_power_idx = 1;
		strongest_power_idx2 = 0;
	}

	for (int i = 2; i < num_path; i++)
	{
		if (power[i] > strongest_power2)
		{
			if (power[i] > strongest_power)
			{
				strongest_power2 = strongest_power;
				strongest_power_idx2 = strongest_power_idx;
				strongest_power = power[i];
				strongest_power_idx = i;
			}
			else
			{
				strongest_power2 = power[i];
				strongest_power_idx2 = i;
			}
		}
	}
	
}

void CHANNEL::Set_AOAAOD(int _bs_idx, int _ue_idx)
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- AOA, AOD
	///
	/////////////////////////////////////////////////

	
	double K_factor_dB = K_dB;
	//double K_factor_dB = 10.;

	//double ASD = pow(10, normal(mu_ASD, sigma_ASD));
	//double ASA = pow(10, normal(mu_ASA, sigma_ASA)); 
	double C = 0;

	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 51 ~ 
	////////////////////////////////////////
	/////////////////////////////////////////////////////////// InH
	if (TYPE == 11) 
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

			if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
			{
				if (Propagation == 1) ///// LOS  Laplacian    num_clusters = 15
				{
					C = 1.434 * (0.9275 + 0.0439*K_factor_dB - 0.0071*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS   Laplacian    num_clusters = 19
				{
					C = 1.501;

				}
			}
			else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
			{
				if (Propagation == 1) ///// LOS  Gaussian    num_clusters = 15
				{
					C = 1.211 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS   Gaussian   num_clusters = 19
				{
					C = 1.273;

				}
			}
		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{
			if (Frequency >= 500000000 && Frequency <= 100000000000) //// 0.5GHz <= fc <= 100GHz
			{
				if (Propagation == 1) ///// LOS  Gaussian   num_clusters = 15
				{
					C = 1.211 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS  Gaussian    num_clusters = 19
				{
					C = 1.273;

				}
			}
		}

	}
	//////////////////////////////////////////////////////////////// Dense_Urban
	else if (TYPE == 12) 
	{
		////////////////////////////////////////////// UMi
		if (Configuration_Type == 3)
		{
			if (Channel_Model_Type == 0) /// Channel Model A
			{
				if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian  num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS   Gaussian    num_clusters = 19
					{
						C = 1.273;

					}
					else if (Propagation == 2)    ////OUT2IN Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
				else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian    num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS   Gaussian    num_clusters = 19
					{
						C = 1.273;

					}
					else if (Propagation == 2)  ////  OUT2IN  Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
			else if (Channel_Model_Type == 1) // Channel Model B
			{
				if (Frequency >= 500000000 && Frequency <= 100000000000) //// 0.5GHz <= fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS  Gaussian   num_clusters = 19
					{
						C = 1.273;

					}
					else if (Propagation == 2)  //// OUT2IN Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
		}
		////////////////////////////////////////////// UMa
		else
		{
			if (Channel_Model_Type == 0) /// Channel Model A
			{
				if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian  num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS   Gaussian    num_clusters = 20
					{
						C = 1.289;

					}
					else if (Propagation == 2)    ////OUT2IN Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
				else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian    num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS   Gaussian    num_clusters = 20
					{
						C = 1.289;

					}
					else if (Propagation == 2)  ////  OUT2IN  Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
			else if (Channel_Model_Type == 1) // Channel Model B
			{
				if (Frequency >= 500000000 && Frequency <= 100000000000) //// 0.5GHz <= fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS  Gaussian   num_clusters = 20
					{
						C = 1.289;

					}
					else if (Propagation == 2)  //// OUT2IN Gaussian   num_clusters = 12
					{
						C = 1.146 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
		}
	}
	///////////////////////////////////////////////////// RMa
	else if (TYPE == 13) 
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{
			if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
			{
				if (Propagation == 1) ///// LOS Laplacian?? Gaussian??   num_clusters = 11
				{
					C = 1.123 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS      num_clusters = 10
				{
					C = 1.090;

				}
				else if (Propagation == 2)    ////OUT2IN    num_clusters = 10
				{
					C = 1.090;
				}
			}
		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{
			if (Frequency >= 500000000 && Frequency <= 7000000000) //// 0.5GHz <= fc <= 7GHz
			{
				if (Propagation == 1) ///// LOS   num_clusters = 11
				{
					C = 1.123 * (1.1035 - 0.028*K_factor_dB - 0.002*K_factor_dB*K_factor_dB + 0.0001*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS    num_clusters = 10
				{
					C = 1.090;

				}
				else if (Propagation == 2)  //// OUT2IN   num_clusters = 10
				{
					C = 1.090;
				}
			}
		}

	}


	//cout << "C = " << C << endl;


	//AOA = new double[MAX_NUM_CLUSTERS];
	//AOD = new double[MAX_NUM_CLUSTERS];

	double first_AOD;
	double first_AOA;

	double X1_AOD;
	double Y1_AOD;
	double X1_AOA;
	double Y1_AOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		//double Y_AOA = sigma_AOA / 7. * randnum.n();
		//double Y_AOD = sigma_AOD / 7. * randnum.n();

		double Y_AOA = normal(0, pow(ASA / 7. , 2.));
		double Y_AOD = normal(0, pow(ASD / 7. , 2.));

		if (i == 0)
		{
			X1_AOD = X;
			Y1_AOD = Y_AOD;

			X1_AOA = X;
			Y1_AOA = Y_AOA;
		}

		if (power[i] == 0.)
		{
			AOD[i] = 0.;
			AOA[i] = 0.;

		}
		else
		{
			if ((TYPE == 11) && (Channel_Model_Type == 0) && (Frequency >= 500000000 && Frequency <= 6000000000)) //// 0.5GHz <= fc <= 6GHz)   //// Laplacian (InH_A && F<=6GHz)
			{
				AOD[i] = -1 * ASD * log(power[i] / strongest_power) / C;
				AOA[i] = -1 * ASA * log(power[i] / strongest_power) / C;
			}
			else     ///// Gaussian
			{
				AOD[i] = 2. * (ASD / 1.4) * sqrt(-1*log(power[i] / strongest_power)) / C;
				AOA[i] = 2. * (ASA / 1.4) * sqrt(-1*log(power[i] / strongest_power)) / C;
			}
		}

		if (i == 0)
		{
			first_AOD = AOD[0];
			first_AOA = AOA[0];
		}

		

		if (channel[_bs_idx][_ue_idx].self_ms.y > channel[_bs_idx][_ue_idx].self_bs.y)
		{
			if (channel[_bs_idx][_ue_idx].self_ms.x > channel[_bs_idx][_ue_idx].self_bs.x)
			{
				//cout << "case1" << endl;
				LOS_AOD_GCS = atan((channel[_bs_idx][_ue_idx].self_ms.y - channel[_bs_idx][_ue_idx].self_bs.y) / (channel[_bs_idx][_ue_idx].self_ms.x - channel[_bs_idx][_ue_idx].self_bs.x)) * (180. / pi);
				LOS_AOA_GCS = (180. + LOS_AOD_GCS) - 360.;
			}
			else
			{
				//cout << "case2" << endl;
				LOS_AOD_GCS = 180. - atan((channel[_bs_idx][_ue_idx].self_ms.y - channel[_bs_idx][_ue_idx].self_bs.y) / (channel[_bs_idx][_ue_idx].self_bs.x - channel[_bs_idx][_ue_idx].self_ms.x)) * (180. / pi);
				LOS_AOA_GCS = -1. * atan((channel[_bs_idx][_ue_idx].self_ms.y - channel[_bs_idx][_ue_idx].self_bs.y) / (channel[_bs_idx][_ue_idx].self_bs.x - channel[_bs_idx][_ue_idx].self_ms.x)) * (180. / pi);
			}
		}
		else ///// (ms[_ue_idx].loc.y < bs[_bs_idx].loc.y)
		{
			if (channel[_bs_idx][_ue_idx].self_ms.x > channel[_bs_idx][_ue_idx].self_bs.x)
			{
				//cout << "case3" << endl;
				LOS_AOD_GCS = -1. * atan((channel[_bs_idx][_ue_idx].self_bs.y - channel[_bs_idx][_ue_idx].self_ms.y) / (channel[_bs_idx][_ue_idx].self_ms.x - channel[_bs_idx][_ue_idx].self_bs.x)) * (180. / pi);
				LOS_AOA_GCS = 180. - atan((channel[_bs_idx][_ue_idx].self_bs.y - channel[_bs_idx][_ue_idx].self_ms.y) / (channel[_bs_idx][_ue_idx].self_ms.x - channel[_bs_idx][_ue_idx].self_bs.x)) * (180. / pi);
			}
			else
			{
				//cout << "case4" << endl;
				
				LOS_AOA_GCS = atan((channel[_bs_idx][_ue_idx].self_bs.y - channel[_bs_idx][_ue_idx].self_ms.y) / (channel[_bs_idx][_ue_idx].self_bs.x - channel[_bs_idx][_ue_idx].self_ms.x)) * (180. / pi);

				LOS_AOD_GCS = LOS_AOA_GCS - 180.;
			}
		}
		/*
		cout << "channel[_bs_idx][_ue_idx].self_ms.x = " << channel[_bs_idx][_ue_idx].self_ms.x << endl;
		cout << "channel[_bs_idx][_ue_idx].self_ms.y = " << channel[_bs_idx][_ue_idx].self_ms.y << endl;
		cout << "channel[_bs_idx][_ue_idx].self_bs.x = " << channel[_bs_idx][_ue_idx].self_bs.x << endl;
		cout << "channel[_bs_idx][_ue_idx].self_bs.y = " << channel[_bs_idx][_ue_idx].self_bs.y << endl;

		cout << "LOS_AOD_GCS = " << LOS_AOD_GCS << endl;
		cout << "LOS_AOA_GCS = " << LOS_AOA_GCS << endl;
		getchar();
		*/

		if (LOS_AOD_GCS > 180. || LOS_AOD_GCS < -180.)
		{
			cout << "LOS_AOD_GCS Error!  channel.cpp" << endl;
			getchar();
		}

		if (LOS_AOA_GCS > 180. || LOS_AOA_GCS < -180.)
		{
			cout << "LOS_AOA_GCS Error!  channel.cpp" << endl;
			getchar();
		}

		if (LOS == 1) // LOS
		{
			/*
			LOCATION vector;

			// bs_idx, ms_idx -> sector_idx
			double sector_idx;
			
			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);
			*/

			AOD[i] = X * AOD[i] + Y_AOD - (X1_AOD * first_AOD + Y1_AOD - LOS_AOD_GCS);
			AOA[i] = X * AOA[i] + Y_AOA - (X1_AOA * first_AOA + Y1_AOA - LOS_AOA_GCS);
		}
		else if (LOS == 0) // NLOS
		{
			AOD[i] = X * AOD[i] + Y_AOD + LOS_AOD_GCS;
			AOA[i] = X * AOA[i] + Y_AOA + LOS_AOA_GCS;
		}
	}
}

void CHANNEL::Set_ZOAZOD(int _bs_idx, int _ue_idx)
{

	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- ZOA, ZOD
	///
	/////////////////////////////////////////////////

	

	//double K_factor_dB = 10;
	double K_factor_dB = K_dB;
	//double ZSD = pow(10, normal(mu_ZSD, sigma_ZSD));
	//double ZSA = pow(10, normal(mu_ZSA, sigma_ZSA));
	double C = 1;

	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 51 ~ 
	if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

			if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
			{
				if (Propagation == 1) ///// LOS  Laplacian    num_clusters = 15
				{
					C = 1.1088 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS   Laplacian    num_clusters = 19
				{
					C = 1.1764;

				}
			}
			else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
			{
				if (Propagation == 1) ///// LOS  Gaussian    num_clusters = 15
				{
					C = 1.1088 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS   Gaussian   num_clusters = 19
				{
					C = 1.1764;

				}
			}
		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{
			if (Frequency >= 500000000 && Frequency <= 100000000000) //// 0.5GHz <= fc <= 100GHz
			{
				if (Propagation == 1) ///// LOS  Gaussian   num_clusters = 15
				{
					C = 1.1088 * (1.3086 + 0.0339*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS  Gaussian    num_clusters = 19
				{
					C = 1.184;

				}
			}
		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		///////////////////////////////////////////// UMi
		if (Configuration_Type == 3)
		{
			if (Channel_Model_Type == 0) /// Channel Model A
			{
				if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
				{
					if (Propagation == 1) ///// LOS    num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS      num_clusters = 19
					{
						C = 1.1764;

					}
					else if (Propagation == 2)    ////OUT2IN   num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
				else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS      num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS     num_clusters = 19
					{
						C = 1.1764;

					}
					else if (Propagation == 2)  ////  OUT2IN    num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
			else if (Channel_Model_Type == 1) // Channel Model B
			{
				if (Frequency >= 500000000 && Frequency <= 100000000000) //// 0.5GHz <= fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS     num_clusters = 12
					{
						C = 1.104 * (1.3086 + 0.0339*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS    num_clusters = 19
					{
						C = 1.184;

					}
					else if (Propagation == 2)  //// OUT2IN    num_clusters = 12
					{
						C = 1.104 * (1.3086 + 0.0339*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
		}
		///////////////////////////////////////////// UMa
		else
		{
			if (Channel_Model_Type == 0) /// Channel Model A
			{
				if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian  num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS   Gaussian    num_clusters = 20
					{
						C = 1.1918;

					}
					else if (Propagation == 2)    ////OUT2IN Gaussian   num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
				else if (Frequency > 6000000000 && Frequency <= 100000000000) /// 6GHz < fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian    num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS   Gaussian    num_clusters = 20
					{
						C = 1.1918;

					}
					else if (Propagation == 2)  ////  OUT2IN  Gaussian   num_clusters = 12
					{
						C = 1.04 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
			else if (Channel_Model_Type == 1) // Channel Model B
			{
				if (Frequency >= 500000000 && Frequency <= 100000000000) //// 0.5GHz <= fc <= 100GHz
				{
					if (Propagation == 1) ///// LOS  Gaussian   num_clusters = 12
					{
						C = 1.104 * (1.3086 + 0.0339*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

					}
					else if (Propagation == 0)   /////////// NLOS  Gaussian   num_clusters = 20
					{
						C = 1.178;

					}
					else if (Propagation == 2)  //// OUT2IN Gaussian   num_clusters = 12
					{
						C = 1.104 * (1.3086 + 0.0339*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);
					}
				}
			}
		}
	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{
			if (Frequency >= 500000000 && Frequency <= 6000000000) //// 0.5GHz <= fc <= 6GHz
			{
				if (Propagation == 1) ///// LOS Laplacian?? Gaussian??   num_clusters = 11
				{
					C = 1.013 * (1.35 + 0.0202*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS      num_clusters = 10
				{
					C = 0.9854;

				}
				else if (Propagation == 2)    ////OUT2IN    num_clusters = 10
				{
					C = 0.9854;
				}
			}
		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{
			if (Frequency >= 500000000 && Frequency <= 7000000000) //// 0.5GHz <= fc <= 7GHz
			{
				if (Propagation == 1) ///// LOS   num_clusters = 11
				{
					C = 1.031 * (1.3086 + 0.0339*K_factor_dB - 0.0077*K_factor_dB*K_factor_dB + 0.0002*K_factor_dB*K_factor_dB*K_factor_dB);

				}
				else if (Propagation == 0)   /////////// NLOS    num_clusters = 10
				{
					C = 0.957;

				}
				else if (Propagation == 2)  //// OUT2IN   num_clusters = 10
				{
					C = 0.957;
				}
			}
		}

	}


	//cout << "C_zoazod = " << C << endl;


	//ZOD = new double[MAX_NUM_CLUSTERS];
	//ZOA = new double[MAX_NUM_CLUSTERS];

	double first_ZOD;
	double first_ZOA;

	double X1_ZOD;
	double Y1_ZOD;
	double X1_ZOA;
	double Y1_ZOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_ZOA = normal(0, pow(ZSA / 7., 2.)); //
		double Y_ZOD = normal(0, pow(ZSD / 7., 2.));

		if (i == 0)
		{
			X1_ZOD = X;
			Y1_ZOD = Y_ZOD;

			X1_ZOA = X;
			Y1_ZOA = Y_ZOA;
		}

		if (power[i] == 0.)
		{
			ZOD[i] = 0.;
			ZOA[i] = 0.;

		}
		else
		{
			ZOD[i] = -1 * ZSD * log(power[i] / strongest_power) / C;
			ZOA[i] = -1 * ZSA * log(power[i] / strongest_power) / C;
		}
		if (i == 0)
		{
			first_ZOD = ZOD[0];
			first_ZOA = ZOA[0];
		}



		
		LOS_ZOA_GCS = 90. - (atan((bs_height - ms[_ue_idx].MS_HEIGHT_FINAL) / distance) * (180. / pi));
		LOS_ZOD_GCS = 180. - LOS_ZOA_GCS;

		/*
		cout << "distance = " << distance << endl;
		cout << "(bs_height - ms[_ue_idx].MS_HEIGHT_FINAL) = " << (bs_height - ms[_ue_idx].MS_HEIGHT_FINAL) << endl;
		cout << "LOS_ZOD_GCS = " << LOS_ZOD_GCS << endl;
		cout << "LOS_ZOA_GCS = " << LOS_ZOA_GCS << endl;
		getchar();
		*/

		if (LOS_ZOD_GCS > 180. || LOS_ZOD_GCS < 0.)
		{
			cout << "LOS_ZOD_GCS Error!  channel.cpp" << endl;
			getchar();
		}

		if (LOS_ZOA_GCS > 180. || LOS_ZOA_GCS < 0.)
		{
			cout << "LOS_ZOD_GCS Error!  channel.cpp" << endl;
			getchar();
		}




		if (LOS == 1) // LOS
		{
			/*
			LOCATION vector;

			// bs ms -> angle 
			double sector_idx;

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}
			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);
			*/

			ZOA[i] = (X * ZOA[i] + Y_ZOA) - (X1_ZOA * first_ZOA + Y1_ZOA - LOS_ZOA_GCS);
			ZOD[i] = (X * ZOD[i] + Y_ZOD) - (X1_ZOD * first_ZOD + Y1_ZOD - LOS_ZOD_GCS);

		}
		else if (LOS == 0) // NLOS
		{
			ZOA[i] = X * ZOA[i] + Y_ZOA + LOS_ZOA_GCS;
			ZOD[i] = X * ZOD[i] + Y_ZOD + LOS_ZOD_GCS + mu_offset_ZOD;

		}
		/*
		if (_ue_idx == 4)
		{
			cout << "LOS = " << LOS << endl;
			cout << "mu_offset_ZOD = " << mu_offset_ZOD << endl;
			cout << "X = " << X << endl;
			cout << "Y_ZOD = " << Y_ZOD << endl;
			cout << "X1_ZOD = " << X1_ZOD << endl;
			cout << "first_ZOD = " << first_ZOD << endl;
			cout << "Y1_ZOD = " << Y1_ZOD << endl;
			cout << "LOS_ZOD = " << LOS_ZOD << endl;
			cout << "ZOD[0] = " << ZOD[0] << endl;
			getchar();
		}
		*/
	}







	





}

void CHANNEL::Set_InitialPhase()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- initial phase
	///
	/////////////////////////////////////////////////
	
	/*
	random_phase_vv = new double *[MAX_NUM_CLUSTERS];
	random_phase_vh = new double *[MAX_NUM_CLUSTERS];
	random_phase_hv = new double *[MAX_NUM_CLUSTERS];
	random_phase_hh = new double *[MAX_NUM_CLUSTERS];

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		random_phase_vv[i] = new double[MAX_NUM_RAYS];
		random_phase_vh[i] = new double[MAX_NUM_RAYS];
		random_phase_hv[i] = new double[MAX_NUM_RAYS];
		random_phase_hh[i] = new double[MAX_NUM_RAYS];
	}
	*/

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		for (int j = 0; j < MAX_NUM_RAYS; j++)
		{

			random_phase_vv[i][j] = 360. * randnum.u() - 180.;
			random_phase_vh[i][j] = 360. * randnum.u() - 180.;
			random_phase_hv[i][j] = 360. * randnum.u() - 180.;
			random_phase_hh[i][j] = 360. * randnum.u() - 180.;

		}
	}

	random_phase_vv_LOS = 360. * randnum.u() - 180.;
	random_phase_hh_LOS = 360. * randnum.u() - 180.;
}

void CHANNEL::Set_SUBCLUSTER()
{
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////
	////// offset_angle 
	///////////////////////////////////

	/*
	offset_angle = new double *[MAX_NUM_CLUSTERS];

	for (int path_idx = 0; path_idx < MAX_NUM_CLUSTERS; path_idx++)
	{
		offset_angle[path_idx] = new double[MAX_NUM_RAYS];

	}
	*/

	for (int path_idx = 0; path_idx < MAX_NUM_CLUSTERS; path_idx++)
	{
		NUM_RAY_per_ClusterNUM[path_idx] = 0;

		for (int ray_idx = 0; ray_idx < MAX_NUM_RAYS; ray_idx++)
		{
			offset_angle[path_idx][ray_idx] = 0;
		}
	}

	NUM_PATH_for_channelcoeff = 0;  //// ð SSP ϹǷ, ʱȭ ʿ

	//////////////////////////////////////////////
	/////
	///// Set_subcluster 
	///// power  2 Ŭʹ 3 Ŭͷ  offset ش. ڵ   ƴ϶ 4 ߰ Ŭ(path) ߰ϴ , ray ٸ
	////
	/////////////////////////////////////////////

	if (cluster_DS < 0) ///// cluster_DS == N/A
	{
		delay[num_path] = delay[strongest_power_idx] + 1.28 * 3.91;
		delay[num_path + 1] = delay[strongest_power_idx] + 2.56 * 3.91;
		delay[num_path + 2] = delay[strongest_power_idx2] + 1.28 * 3.91;
		delay[num_path + 3] = delay[strongest_power_idx2] + 2.56 * 3.91;
	}
	else
	{
		delay[num_path] = delay[strongest_power_idx] + 1.28 * cluster_DS;
		delay[num_path + 1] = delay[strongest_power_idx] + 2.56 * cluster_DS;
		delay[num_path + 2] = delay[strongest_power_idx2] + 1.28 * cluster_DS;
		delay[num_path + 3] = delay[strongest_power_idx2] + 2.56 * cluster_DS;
	}

	power[strongest_power_idx] = power[strongest_power_idx] * 10. / 20.;
	power[strongest_power_idx2] = power[strongest_power_idx2] * 10. / 20.;
	power[num_path] = power[strongest_power_idx] * 6. / 20.;
	power[num_path + 1] = power[strongest_power_idx] * 4. / 20.;
	power[num_path + 2] = power[strongest_power_idx2] * 6. / 20.;
	power[num_path + 3] = power[strongest_power_idx2] * 4. / 20.;

	AOA[num_path] = AOA[strongest_power_idx];
	AOA[num_path + 1] = AOA[strongest_power_idx];
	AOA[num_path + 2] = AOA[strongest_power_idx2];
	AOA[num_path + 3] = AOA[strongest_power_idx2];

	AOD[num_path] = AOD[strongest_power_idx];
	AOD[num_path + 1] = AOD[strongest_power_idx];
	AOD[num_path + 2] = AOD[strongest_power_idx2];
	AOD[num_path + 3] = AOD[strongest_power_idx2];

	for (int path_idx = 0; path_idx < num_path + 4; path_idx++)
	{
		if (path_idx == strongest_power_idx || path_idx == strongest_power_idx2) /// Subcluster No.1 (1,2,3,4,5,6,7,8,19,20 rays)
		{
			//num_ray = 10;
			NUM_RAY_per_ClusterNUM[path_idx] = 10;
			offset_angle[path_idx][0] = 0.0447;
			offset_angle[path_idx][1] = -0.0447;
			offset_angle[path_idx][2] = 0.1413;
			offset_angle[path_idx][3] = -0.1413;
			offset_angle[path_idx][4] = 0.2492;
			offset_angle[path_idx][5] = -0.2492;
			offset_angle[path_idx][6] = 0.3715;
			offset_angle[path_idx][7] = -0.3715;
			offset_angle[path_idx][8] = 2.1551;
			offset_angle[path_idx][9] = -2.1551;
		}

		else if (path_idx == num_path || path_idx == num_path + 2) /// Subcluster No.2 (9,10,11,12,17,18 rays)
		{
			//num_ray = 6;
			NUM_RAY_per_ClusterNUM[path_idx] = 6;
			offset_angle[path_idx][0] = 0.5129;
			offset_angle[path_idx][1] = -0.5129;
			offset_angle[path_idx][2] = 0.6797;
			offset_angle[path_idx][3] = -0.6797;
			offset_angle[path_idx][4] = 1.5195;
			offset_angle[path_idx][5] = -1.5195;
		}

		else if (path_idx == num_path + 1 || path_idx == num_path + 3) /// Subcluster No.3 (13,14,15,16 rays)
		{
			//num_ray = 4;
			NUM_RAY_per_ClusterNUM[path_idx] = 4;
			offset_angle[path_idx][0] = 0.8844;
			offset_angle[path_idx][1] = -0.8844;
			offset_angle[path_idx][2] = 1.1481;
			offset_angle[path_idx][3] = -1.1481;
		}

		else /// Not 2 strong cluster
		{
			//num_ray = 20;
			NUM_RAY_per_ClusterNUM[path_idx] = 20;
			offset_angle[path_idx][0] = 0.0447;
			offset_angle[path_idx][1] = -0.0447;
			offset_angle[path_idx][2] = 0.1413;
			offset_angle[path_idx][3] = -0.1413;
			offset_angle[path_idx][4] = 0.2492;
			offset_angle[path_idx][5] = -0.2492;
			offset_angle[path_idx][6] = 0.3715;
			offset_angle[path_idx][7] = -0.3715;
			offset_angle[path_idx][8] = 0.5129;
			offset_angle[path_idx][9] = -0.5129;
			offset_angle[path_idx][10] = 0.6797;
			offset_angle[path_idx][11] = -0.6797;
			offset_angle[path_idx][12] = 0.8844;
			offset_angle[path_idx][13] = -0.8844;
			offset_angle[path_idx][14] = 1.1481;
			offset_angle[path_idx][15] = -1.1481;
			offset_angle[path_idx][16] = 1.5195;
			offset_angle[path_idx][17] = -1.5195;
			offset_angle[path_idx][18] = 2.1551;
			offset_angle[path_idx][19] = -2.1551;
		}
	}

	NUM_PATH_for_channelcoeff = num_path + 4;


}


void CHANNEL::Set_W_matrix()
{
	W_tx = MatrixXd::Zero(2 * BS_N*BS_M*BS_Ng*BS_Mg, 3);
	W_rx = MatrixXd::Zero(2 * MS_N*MS_M*MS_Ng*MS_Mg, 3);

	for (int i = 0; i < 2 * BS_N*BS_M*BS_Ng*BS_Mg; i++)
	{
		int ii = i + 1;     ///// ii = 1 ~ 2MNMgNg

		for (int j = 0; j < 3; j++)
		{
			if (j == 0)
			{
				W_tx(i, j) = 0;
			}
			else if (j == 1)
			{
				W_tx(i, j) = floor((ii - 1 - 2 * BS_N*floor((ii - 1) / (2 * BS_N))) / 2) * BS_d_H + floor((ii - 1 - 2 * BS_N*BS_Ng*floor((ii - 1) / (2 * BS_N*BS_Ng))) / (2 * BS_N)) * BS_d_gH;

			}
			else if (j == 2)
			{
				W_tx(i, j) = floor((ii - 1 - 2 * BS_N*BS_M*BS_Ng*floor((ii - 1) / (2 * BS_N*BS_M*BS_Ng))) / (2 * BS_N*BS_Ng)) * BS_d_V + floor((ii - 1 - 2 * BS_N*BS_M*BS_Ng*BS_Mg*floor((ii - 1) / (2 * BS_N*BS_M*BS_Ng*BS_Mg))) / (2 * BS_N*BS_M*BS_Ng)) * BS_d_gV;
			}
		}

		/*
		cout << "BS_M = " << BS_M << endl;
		cout << "BS_N = " << BS_N << endl;
		cout << "BS_Mg = " << BS_Mg << endl;
		cout << "BS_Ng = " << BS_Ng << endl;

		cout << "i = " << ii << endl;
		cout << "W_tx = " << "\t" << W_tx << endl;
		getchar();
		*/

	}

	for (int i = 0; i < 2 * MS_N*MS_M*MS_Ng*MS_Mg; i++)
	{
		int ii = i + 1;    ///// ii = 1 ~ 2MNMgNg

		for (int j = 0; j < 3; j++)
		{
			if (j == 0)
			{
				W_rx(i, j) = 0;
			}
			else if (j == 1)
			{
				W_rx(i, j) = floor((ii - 1 - 2 * MS_N*floor((ii - 1) / (2 * MS_N))) / 2) * MS_d_H + floor((ii - 1 - 2 * MS_N*MS_Ng*floor((ii - 1) / (2 * MS_N*MS_Ng))) / (2 * MS_N)) * MS_d_gH;
			}
			else if (j == 2)
			{
				W_rx(i, j) = floor((ii - 1 - 2 * MS_N*MS_M*MS_Ng*floor((ii - 1) / (2 * MS_N*MS_M*MS_Ng))) / (2 * MS_N*MS_Ng)) * MS_d_V + floor((ii - 1 - 2 * MS_N*MS_M*MS_Ng*MS_Mg*floor((ii - 1) / (2 * MS_N*MS_M*MS_Ng*MS_Mg))) / (2 * MS_N*MS_M*MS_Ng)) * MS_d_gV;
			}
		}
	}
	
	/*
	cout << "W_tx = " << "\t"<< W_tx << endl;
	cout << "W_rx = "<< "\t" << W_rx << endl;
	getchar();
	*/
}



void CHANNEL::Set_RMS_delay_spread()
{
	if (LOS == 0) //NLOS
	{
		delay_power = new double[MAX_NUM_CLUSTERS];
		power_ray_sum = new double[MAX_NUM_CLUSTERS];

		for (int i = 0; i < num_path; i++)
		{
			//delay_power[i] = num_ray * delay[i] * power[i];
			//power_ray_sum[i] = num_ray * power[i];

			delay_power[i] = delay[i] * power[i];
			power_ray_sum[i] = power[i];
		}

		double avg_delay;
		double delay_power_sum = 0;
		double power_sum = 0;


		for (int i = 0; i < num_path; i++)
		{
			delay_power_sum += delay_power[i];
			power_sum += power_ray_sum[i];
		}


		avg_delay = delay_power_sum / power_sum;


		RMS_delay_power = new double[MAX_NUM_CLUSTERS];


		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power[i] = (delay[i] - avg_delay) * (delay[i] - avg_delay) * power[i];
		}

		//double RMS_delay_spread = 0;
		double RMS_delay_power_sum = 0;

		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power_sum += RMS_delay_power[i];
		}

		RMS_delay_spread = sqrt(RMS_delay_power_sum / power_sum);
	}

	else if (LOS == 1) // LOS
	{
		delay_power = new double[MAX_NUM_CLUSTERS];
		power_ray_sum = new double[MAX_NUM_CLUSTERS];

		for (int i = 0; i < num_path; i++)
		{
			//delay_power[i] = num_ray * delay[i] * power[i];
			//power_ray_sum[i] = num_ray * power[i];

			delay_power[i] = delay_LOS[i] * power[i];
			power_ray_sum[i] = power[i];
		}

		double avg_delay;
		double delay_power_sum = 0;
		double power_sum = 0;


		for (int i = 0; i < num_path; i++)
		{
			delay_power_sum += delay_power[i];
			power_sum += power_ray_sum[i];
		}


		avg_delay = delay_power_sum / power_sum;


		RMS_delay_power = new double[MAX_NUM_CLUSTERS];


		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power[i] = (delay_LOS[i] - avg_delay) * (delay_LOS[i] - avg_delay) * power[i];
		}

		//double RMS_delay_spread = 0;
		double RMS_delay_power_sum = 0;

		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power_sum += RMS_delay_power[i];
		}

		RMS_delay_spread = sqrt(RMS_delay_power_sum / power_sum);

	}
	



}

void CHANNEL::Set_circular_angle_spread()
{
	//double mu_theta = 0;

	//3GPP 25.996 Annex A 
	
	AOA_power = new double[MAX_NUM_CLUSTERS];
	AOD_power = new double[MAX_NUM_CLUSTERS];
	power_ray_sum_circular = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < num_path; i++)
	{
		//delay_power[i] = num_ray * delay[i] * power[i];
		//power_ray_sum[i] = num_ray * power[i];

		AOA_power[i] = AOA[i] * power[i]; 
		AOD_power[i] = AOD[i] * power[i];
		power_ray_sum_circular[i] = power[i];
	}

	double mu_theta_AOA;
	double mu_theta_AOD;
	double AOA_power_sum = 0;
	double AOD_power_sum = 0;
	double power_sum = 0;

	// AOA AOD degree
	
	for (int i = 0; i < num_path; i++)
	{
		AOA_power_sum += AOA_power[i];
		AOD_power_sum += AOD_power[i];
		power_sum += power_ray_sum_circular[i];
	}

	mu_theta_AOA = AOA_power_sum / power_sum;  // ̰͵ degreeε
	mu_theta_AOD = AOD_power_sum / power_sum;

	/*
	cout << "A = " << mu_theta_AOA << endl;
	cout << "D = " << mu_theta_AOD << endl;
	cout << pi << endl;
	getchar();
	*/



	//Calibration  ߰ 
	if (mu_theta_AOA < -180.) // pi Ǿִµ degree  => -180 ~ 180 ִ°
	{
		mu_theta_AOA = 2. * 180. + mu_theta_AOA;
	}
	else if (mu_theta_AOA > 180.)
	{
		mu_theta_AOA = mu_theta_AOA - 2. * 180.;
	}
	else
	{
		mu_theta_AOA = mu_theta_AOA;

	}

	if (mu_theta_AOD < -180.)
	{
		mu_theta_AOD = 2. * 180. + mu_theta_AOD;
	}
	else if (mu_theta_AOD > 180.)
	{
		mu_theta_AOD = mu_theta_AOD - 2. * 180.;
	}
	else
	{
		mu_theta_AOD = mu_theta_AOD;

	}


	theta_n_m_mu_AOA = new double[MAX_NUM_CLUSTERS];
	theta_n_m_mu_AOD = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < num_path; i++)
	{
		if ((AOA[i] - mu_theta_AOA) < -180. ) // pi -> 180
		{
			theta_n_m_mu_AOA[i] = 2. * 180. + (AOA[i] - mu_theta_AOA);
		}
		else if (((AOA[i] - mu_theta_AOA) >= -180.) && ((AOA[i] - mu_theta_AOA) <= 180.))
		{
			theta_n_m_mu_AOA[i] = (AOA[i] - mu_theta_AOA);
		}
		else if ((AOA[i] - mu_theta_AOA) > 180.)
		{
			theta_n_m_mu_AOA[i] = 2. * 180. - (AOA[i] - mu_theta_AOA);
		}
		
	}

	for (int i = 0; i < num_path; i++)
	{
		if ((AOD[i] - mu_theta_AOD) < -180.)
		{
			theta_n_m_mu_AOD[i] = 2. * 180. + (AOD[i] - mu_theta_AOD);
		}
		else if (((AOD[i] - mu_theta_AOD) >= -180.) && ((AOD[i] - mu_theta_AOD) <= 180.))
		{
			theta_n_m_mu_AOD[i] = (AOD[i] - mu_theta_AOD);
		}
		else //if ((AOD[i] - mu_theta_AOD) > 180.)
		{
			theta_n_m_mu_AOD[i] = 2. * 180. - (AOD[i] - mu_theta_AOD);
		}

	}

	theta_power_AOA = new double[MAX_NUM_CLUSTERS];
	theta_power_AOD = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < num_path; i++)
	{
		theta_power_AOA[i] = theta_n_m_mu_AOA[i] * theta_n_m_mu_AOA[i] * power[i];
		theta_power_AOD[i] = theta_n_m_mu_AOD[i] * theta_n_m_mu_AOD[i] * power[i];
	}

	double theta_power_sum_AOA = 0;
	double theta_power_sum_AOD = 0;

	for (int i = 0; i < num_path; i++)
	{
		theta_power_sum_AOA += theta_power_AOA[i];
		theta_power_sum_AOD += theta_power_AOD[i];
	}

	circular_angle_spread_AOA = sqrt(theta_power_sum_AOA / power_sum);
	circular_angle_spread_AOD = sqrt(theta_power_sum_AOD / power_sum);


}

void CHANNEL::Kill_Memory(void)
{
	
	for (int path_idx = 0; path_idx < MAX_NUM_CLUSTERS; path_idx++)
	{
		delete[] random_phase_vv[path_idx];
		delete[] random_phase_vh[path_idx];
		delete[] random_phase_hv[path_idx];
		delete[] random_phase_hh[path_idx];
		delete[] offset_angle   [path_idx];
	}

	delete[] random_phase_vv;
	delete[] random_phase_vh;
	delete[] random_phase_hv;
	delete[] random_phase_hh;
	delete[] offset_angle;


	delete[] delay;
	delete[] delay_LOS;

	delete[] power;
	delete[] power_LOS;
	delete[] power_NLOS;

	delete[] AOA;
	delete[] AOD;

	if (CH_CAL == 0)
	{
		delete[] AOA_LOS;
		delete[] AOD_LOS;

		delete[] AOA_NLOS;
		delete[] AOD_NLOS;
	}


	if (TYPE == 6 || TYPE == 7 || TYPE == 8 || TYPE == 11 || TYPE == 12 || TYPE == 13 || TYPE == 14 || TYPE == 15 || TYPE == 16 || TYPE == 17 || TYPE == 18)
	{
		delete[] ZOA;
		delete[] ZOD;

		if (CH_CAL == 0)
		{
			delete[] ZOA_LOS;
			delete[] ZOD_LOS;

			delete[] ZOA_NLOS;
			delete[] ZOD_NLOS;
		}


	}

	delete[] delay_power;
	delete[] power_ray_sum;
	delete[] RMS_delay_power;

	delete[] AOA_power;
	delete[] AOD_power;
	delete[] power_ray_sum_circular;


	delete[] theta_n_m_mu_AOA;
	delete[] theta_n_m_mu_AOD;

	delete[] theta_power_AOA;
	delete[] theta_power_AOD;




	if (CHIR == NULL)
	{

	}
	else
	{
		/*
		for (int u = 0; u < NUM_TX; u++)
		{
			for (int s = 0; s < NUM_RX; s++)
			{
				delete[] CHIR[u][s];
			}
		}

		for (int u = 0; u < NUM_TX; u++)
		{
			delete[] CHIR[u];
		}

		delete[] CHIR;
		*/
	}






}


void CHANNEL::Precalculate(int _time_idx )  // ̸ ϴ CH Ʈ κ
{
	/*
	//// =============¿=================================================//
	//ofstream CHIR_Precalc("Precalc_CHIR.dat");
	//CHIR_Precalc << "time_idx" << "\t" << "LOS" << "\t" << "INDOOR" << "\t" << "V_ANGLE_MAPPING" << "\t" << "NUM_TX" << "\t" << "NUM_RX" << "\t" << "NUM_CLUSTER" << "\t" << "CHIR" << endl;
	//// =============¿=================================================//

	if (TYPE == 1 || TYPE == 2 || TYPE == 3 || TYPE == 4 || TYPE == 5)
	{
		//ms_idx = _ms_idx;

		double ray_AOD = 0.; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
		double ray_AOA = 0.;
		double doppler_freq = 0.;


		//for (int time = 0; time < precalculate_time; time++)
		//{
			for (int los = 0; los < 2; los++)
			{
				for (int indoor = 0; indoor < 2; indoor++)
				{
					for (int v = 0; v < anglev_mapping_value; v++)
					{
						for (int u = 0; u < NUM_TX; u++)
						{
							for (int s = 0; s < NUM_RX; s++)
							{
								for (int i = 0; i < num_path; i++)
								{
									for (int j = 0; j < num_ray; j++)
									{
										if (los == 0)
										{
											ray_AOD = AOD_NLOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_NLOS[i] + cluster_ASA * offset_angle[i][j];
										}
										else if (los == 1)
										{
											ray_AOD = AOD_LOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_LOS[i] + cluster_ASA * offset_angle[i][j];
										}

										// indoor  user_speed ٸ ġ
										doppler_freq = (user_speed * 1000. / 3600.) * cos((ray_AOA - (2. * 180. * randnum.u() - 180.))*pi / 180) / Wavelength;
										
										

										double xpr = pow(10, XPR / 10);


										// angle_v  ̾Ƽ 10  ϳ ϵ ġ
										//   angle_v = (10 20 30 40 ...)  ϳ  ..


										//double angle_v = atan2((bs_height - ms_height), distance) * 180 / pi;

										//double angle_v = atan2((bs_height - ms_height), min_distance) * 180 / pi;
										
										//double angle_v = 10. * (anglev_mapping_value + 1.);

										double angle_v = ((MAX_v_angle / anglev_mapping_value) * (v + 1)) * randnum.u();

										if (angle_v < ((MAX_v_angle / anglev_mapping_value) * v))
										{
											angle_v = angle_v + ((MAX_v_angle / anglev_mapping_value) * v);
										}


										///////////F_tx_v
										F_tx_v = 12. * (angle_v - ANGLE_tilt)*(angle_v - ANGLE_tilt) / (15. * 15.); // 15 = pi_3dB // vertical(elevation) antenna pattern
										if (F_tx_v > 20.)
										{
											F_tx_v = 20.;
										}

										////////////F_tx_h
										F_tx_h = 12. * ray_AOD*ray_AOD / (70. * 70.); // 70 = theta_3dB // horizental antenna pattern
										if (F_tx_h > 20.)
										{
											F_tx_h = 20.;
										}

										//////////F_rx_v
										F_rx_v = 0.;

										//////////F_rx_h
										F_rx_h = 0.;

										F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
										F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
										F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
										F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



										if (TYPE == 1)
										{
											F_vv = 1;
											F_vh = 1;
											F_hv = 1;
											F_hh = 1;
										}



										complex <double>jay(0, 1);
										complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv[i][j]) + sqrt(1 / xpr)*F_hv*exp(jay*random_phase_hv[i][j]) + sqrt(1 / xpr)*F_vh*exp(jay*random_phase_vh[i][j]) + F_hh*exp(jay*random_phase_hh[i][j]);


										if (los == 1)  // LOS
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_LOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));
											// d_tx  uniform distance between tx element (m.2135-1)
										}
										else           // NLOS
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_NLOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));
											// d_tx  uniform distance between tx element (m.2135-1)
										}


										if (los == 1)
										{
											double k_factor = pow(10, K_factor / 10);

											//CHIR_precalculate[time][los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));
											CHIR_precalc[los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));
											
											double ray_delay = delay[0];
											double ray_AOD_los = AOD_LOS[0];
											double ray_AOA_los = AOA_LOS[0];

											// ʿ
											doppler_freq = (user_speed * 1000. / 3600.) * cos(ray_AOA_los - (2. * 180. * randnum.u() - 180.)) / Wavelength;

											///////////F_tx_v
											F_tx_v = 12. * (angle_v - ANGLE_tilt)*(angle_v - ANGLE_tilt) / (15. * 15.);
											if (F_tx_v > 20.)
											{
												F_tx_v = 20.;
											}

											////////////F_tx_h
											F_tx_h = 12. * ray_AOD_los * ray_AOD_los / (70. * 70.);
											if (F_tx_h > 20.)
											{
												F_tx_h = 20.;
											}


											//////////F_rx_v
											F_rx_v = 0.;

											//////////F_rx_h
											F_rx_h = 0.;

											F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
											F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
											F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
											F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



											if (TYPE == 1)
											{
												F_vv = 1;
												F_vh = 1;
												F_hv = 1;
												F_hh = 1;
											}


											// LOS϶ delta(n-1) * .... ִ° ִ. i = 0 ϶ ִ°.. i  cluster index
											if (i == 0)  
											{
												//CHIR_precalculate[time][los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(time/1000.));
												CHIR_precalc[los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180.) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180.) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(_time_idx / 1000.));
											}
										}
									}
									//// =============¿=================================================//
									//CHIR_Precalc << time << "\t" << los << "\t" << indoor << "\t" << v << "\t" << u << "\t" << s << "\t" << i << "\t" << CHIR_precalc[time][los][indoor][v][u][s][i] << endl;
									//// =============¿=================================================//
								}
							}
						}
					}
				}
			}

			//cout << "CHIR Time = " << time + 1 << "ms" << endl;

		//}
	}



	///////////////////////////
	///
	/// 3D channel model
	///
	///////////////////////////
	else
	{
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//ms_idx = _ms_idx;

		double ray_AOD = 0.; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
		double ray_AOA = 0.;
		double doppler_freq = 0.;
		double ray_ZOA = 0.;
		double ray_ZOD = 0.;


		//for (int time = 0; time < precalculate_time; time++)
		//{
			for (int los = 0; los < 2; los++)
			{
				for (int indoor = 0; indoor < 2; indoor++)
				{
					for (int v = 0; v < anglev_mapping_value; v++)
					{
						for (int u = 0; u < NUM_TX; u++)
						{
							for (int s = 0; s < NUM_RX; s++)
							{
								for (int i = 0; i < num_path; i++)
								{
									for (int j = 0; j < num_ray; j++)
									{
										if (los == 0)
										{
											ray_AOD = AOD_NLOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_NLOS[i] + cluster_ASA * offset_angle[i][j];

											ray_ZOA = ZOA_NLOS[i] + cluster_ZSA * offset_angle[i][j];
											ray_ZOD = ZOD_NLOS[i] + (3 / 8) * pow(10, mu_ZSD) * offset_angle[i][j];
										}
										else if (los == 1)
										{
											ray_AOD = AOD_LOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_LOS[i] + cluster_ASA * offset_angle[i][j];

											ray_ZOA = ZOA_LOS[i] + cluster_ZSA * offset_angle[i][j];
											ray_ZOD = ZOD_LOS[i] + (3 / 8) * pow(10, mu_ZSD) * offset_angle[i][j];
										}

										// indoor  user_speed ٸ ġ
										doppler_freq = (user_speed * 1000. / 3600.) * (sin(ray_ZOA * pi / 180) * cos(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180))
											+ (user_speed * 1000. / 3600.) * (sin(ray_ZOA*pi / 180) * sin(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180))
											+ (user_speed * 1000. / 3600.) * (cos(ray_ZOA*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180));

										doppler_freq = doppler_freq / Wavelength;

										double xpr = pow(10, XPR / 10);

										///////////F_tx_v

										double ant_ZOD_v = 12. * (ray_ZOD - 90)*(ray_ZOD - 90) / (65. * 65.);
										double ant_AOD_v = 12. * (ray_AOD - 90)*(ray_AOD - 90) / (65. * 65.);

										F_tx_v = -MIN(-(ant_ZOD_v + ant_AOD_v), 30.);

										////////////F_tx_h

										double ant_ZOD_h = 12. * (ray_ZOD)*(ray_ZOD) / (65. * 65.);
										double ant_AOD_h = 12. * (ray_AOD)*(ray_AOD) / (65. * 65.);

										F_tx_h = -MIN(-(ant_ZOD_h + ant_AOD_h), 30.);

										//////////F_rx_v
										F_rx_v = 0.;

										//////////F_rx_h
										F_rx_h = 0.;

										F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
										F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
										F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
										F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);


										complex <double>jay(0, 1);
										complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv[i][j]) + sqrt(1 / xpr)*F_hv*exp(jay*random_phase_hv[i][j]) + sqrt(1 / xpr)*F_vh*exp(jay*random_phase_vh[i][j]) + F_hh*exp(jay*random_phase_hh[i][j]);

										if (los == 1)  // LOS
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_LOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));

											// d_tx  uniform distance between tx element (m.2135-1)
											//d_tx, d_rx  ̻ϴ. ڵ带  d_tx = 10 * wavelength * num_tx_antennaε // num_tx_antenna  ʴ(ڵ尡), ݿ ʿϴ // ߴ

											//⼭ d_tx location vector of tx antenna element (36.873) // m.2135-1 Ȱ ϵ سµ ٸ ִ(Ȯġ  κ..)
											//  exp ִ κ  κ ray_AOD  Ȯ
										}
										else           // NLOS 
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_NLOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));

											// d_tx  uniform distance between tx element (m.2135-1)
											//d_tx, d_rx  ̻ϴ. ڵ带  d_tx = 10 * wavelength * num_tx_antennaε // num_tx_antenna  ʴ(ڵ尡), ݿ ʿϴ // ߴ

											//⼭ d_tx location vector of tx antenna element (36.873) // m.2135-1 Ȱ ϵ سµ ٸ ִ(Ȯġ  κ..)
											//  exp ִ κ  κ ray_AOD  Ȯ
										}



										if (los == 1)
										{
											double k_factor = pow(10, K_factor / 10);

											//CHIR_precalculate[time][los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));

											CHIR_precalc[los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));

											double ray_delay = delay[0];
											double ray_AOD_los = AOD_LOS[0];
											double ray_AOA_los = AOA_LOS[0];

											double ray_ZOA_los = ZOA_LOS[0];
											double ray_ZOD_los = ZOD_LOS[0];

											// ʿ
											// indoor  user_speed ٸ ġ
											doppler_freq = user_speed * (sin(ray_ZOA * pi / 180) * cos(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180))
												+ user_speed * (sin(ray_ZOA*pi / 180) * sin(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180))
												+ user_speed * (cos(ray_ZOA*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180));

											doppler_freq = doppler_freq / Wavelength;

											///////////F_tx_v

											double ant_ZOD_v = 12. * (ray_ZOD - 90)*(ray_ZOD - 90) / (65. * 65.);
											double ant_AOD_v = 12. * (ray_AOD - 90)*(ray_AOD - 90) / (65. * 65.);

											F_tx_v = -MIN(-(ant_ZOD_v + ant_AOD_v), 30.);

											////////////F_tx_h

											double ant_ZOD_h = 12. * (ray_ZOD)*(ray_ZOD) / (65. * 65.);
											double ant_AOD_h = 12. * (ray_AOD)*(ray_AOD) / (65. * 65.);

											F_tx_h = -MIN(-(ant_ZOD_h + ant_AOD_h), 30.);

											//////////F_rx_v
											F_rx_v = 0.;

											//////////F_rx_h
											F_rx_h = 0.;

											F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
											F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
											F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
											F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



											// LOS϶ delta(n-1) * .... ִ° ִ. i =0 ϶ ִ°.. i  cluster index
											if (i == 0)
											{
												//CHIR_precalculate[time][los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(time/1000.));
												CHIR_precalc[los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180.) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180.) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(_time_idx / 1000.));
											}
										}
									}

									//// =============¿=================================================//
									//CHIR_Precalc << time << "\t" << los << "\t" << indoor << "\t" << v << "\t" << u << "\t" << s << "\t" << i << "\t" << CHIR_precalc[time][los][indoor][v][u][s][i] << endl;
									//// =============¿=================================================//
								}
							}
						}
					}
				}
			}

			//cout << "CHIR Time = " << time + 1 << "ms" << endl;
		//}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	}

	//// =============¿=================================================//
	//CHIR_Precalc.close();
	//// =============¿=================================================//
	*/

}


void CHANNEL::Set_AOAAOD_LOS()
{
	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - LOS case
	///
	/////////////////////////////////////////////////



	double sigma_AOD = pow(10, normal(mu_ASD, sigma_ASD));
	double sigma_AOA = pow(10, normal(mu_ASA, sigma_ASA));

	/*
	if (TYPE == 3)
	{
		sigma_AOA = sigma_AOA - 15.5;
	}  // for calibration &   
	*/


	double C;


	if (TYPE == 1)//InH
	{

			C = 1.434 * (0.9275 + 0.0439*K_factor - 0.0071*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 2)
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 3)
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 4)
	{

		C = 1.123 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	else if (TYPE == 5)
	{

			C = 1.211 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	else if (TYPE == 6)   //////// 
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);
	}
	else if (TYPE == 7)
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	else if (TYPE == 8)
	{
			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}

	AOA = new double[MAX_NUM_CLUSTERS];
	AOD = new double[MAX_NUM_CLUSTERS];

	AOA_LOS = new double[MAX_NUM_CLUSTERS];
	AOD_LOS = new double[MAX_NUM_CLUSTERS];

	double first_AOD;
	double first_AOA;

	double X1_AOD;
	double Y1_AOD;
	double X1_AOA;
	double Y1_AOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_AOA = normal(0, pow(sigma_AOA / 7., 2.));
		double Y_AOD = normal(0, pow(sigma_AOD / 7., 2.));

		if (i == 0)
		{
			X1_AOD = X;
			Y1_AOD = Y_AOD;

			X1_AOA = X;
			Y1_AOA = Y_AOA;
		}

		if (power[i] == 0.)
		{
			AOD[i] = 0.;
			AOA[i] = 0.;

		}
		else
		{
			if (TYPE == 1)
			{
				AOD[i] = -1 * sigma_AOD * log(power[i] / strongest_power) / C;
				AOA[i] = -1 * sigma_AOA * log(power[i] / strongest_power) / C;

			}
			else
			{
				AOD[i] = 2. * (sigma_AOD / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;
				AOA[i] = 2. * (sigma_AOA / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;

			}
		}

		if (i == 0)
		{
			first_AOD = AOD[0];
			first_AOA = AOA[0];
		}

		if (true) // LOS
		{

			LOCATION vector;

			// bs_idx, ms_idx  ε ϵ ؼ Ʒ ģ.
			double sector_idx;



			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);


			AOD[i] = X * AOD[i] + Y_AOD - (X1_AOD * first_AOD + Y1_AOD - angle); //
			AOA[i] = X * AOA[i] + Y_AOA - (X1_AOA * first_AOA + Y1_AOA - angle);

		}

		AOA_LOS[i] = AOA[i];
		AOD_LOS[i] = AOD[i];

	}

	delete[] AOA;
	delete[] AOD;


}


void CHANNEL::Set_AOAAOD_NLOS()
{
	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - NLOS case
	///
	/////////////////////////////////////////////////


	double sigma_AOD = pow(10, normal(mu_ASD, sigma_ASD));
	double sigma_AOA = pow(10, normal(mu_ASA, sigma_ASA));


	double C;


	if (TYPE == 1)//InH
	{

			C = 1.501;


	}
	else if (TYPE == 2)
	{

			C = 1.273;

	}
	else if (TYPE == 3)
	{

			C = 1.289;

	}
	else if (TYPE == 4)
	{

			C = 1.090;

	}
	else if (TYPE == 5)
	{

			C = 1.190;


	}
	else if (TYPE == 6)   //////// 
	{

			C = 1.273;

	}
	else if (TYPE == 7)
	{

			C = 1.289;

	}
	else if (TYPE == 8)
	{

			C = 1.289;


	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}


	AOA = new double[MAX_NUM_CLUSTERS];
	AOD = new double[MAX_NUM_CLUSTERS];

	AOA_NLOS = new double[MAX_NUM_CLUSTERS];
	AOD_NLOS = new double[MAX_NUM_CLUSTERS];

	double first_AOD;
	double first_AOA;

	double X1_AOD;
	double Y1_AOD;
	double X1_AOA;
	double Y1_AOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}


		double Y_AOA = normal(0, pow(sigma_AOA / 7., 2.));
		double Y_AOD = normal(0, pow(sigma_AOD / 7., 2.));

		if (i == 0)
		{
			X1_AOD = X;
			Y1_AOD = Y_AOD;

			X1_AOA = X;
			Y1_AOA = Y_AOA;
		}

		if (power[i] == 0.)
		{
			AOD[i] = 0.;
			AOA[i] = 0.;

		}
		else
		{
			if (TYPE == 1)
			{
				AOD[i] = -1 * sigma_AOD * log(power[i] / strongest_power) / C;
				AOA[i] = -1 * sigma_AOA * log(power[i] / strongest_power) / C;


			}
			else
			{
				AOD[i] = 2. * (sigma_AOD / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;
				AOA[i] = 2. * (sigma_AOA / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;

			}
		}

		if (i == 0)
		{
			first_AOD = AOD[0];
			first_AOA = AOA[0];
		}


		if (true) // NLOS
		{

			LOCATION vector;

			// bs_idx, ms_idx  ε ϵ ؼ Ʒ ģ.
			double sector_idx;


			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);


			AOD[i] = X * AOD[i] + Y_AOD + angle;
			AOA[i] = X * AOA[i] + Y_AOA + angle;

		}

		AOA_NLOS[i] = AOA[i];
		AOD_NLOS[i] = AOD[i];


	}

	delete[] AOA;
	delete[] AOD;

}


void CHANNEL::Set_ZOAZOD_LOS()
{

	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - LOS case
	///
	/////////////////////////////////////////////////

	double sigma_ZOD = pow(10, normal(mu_ZSD, sigma_ZSD));
	double sigma_ZOA = pow(10, normal(mu_ZSA, sigma_ZSA));

	double C = 1;

	if (TYPE == 6)   //////// 
	{

			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 7)
	{

			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 8)
	{
			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);


	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}


	ZOD = new double[MAX_NUM_CLUSTERS];
	ZOA = new double[MAX_NUM_CLUSTERS];

	ZOD_LOS = new double[MAX_NUM_CLUSTERS];
	ZOA_LOS = new double[MAX_NUM_CLUSTERS];

	double first_ZOD;
	double first_ZOA;

	double X1_ZOD;
	double Y1_ZOD;
	double X1_ZOA;
	double Y1_ZOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_ZOA = normal(0, pow(sigma_ZOA / 7., 2.)); //
		double Y_ZOD = normal(0, pow(sigma_ZOD / 7., 2.));

		if (i == 0)
		{
			X1_ZOD = X;
			Y1_ZOD = Y_ZOD;

			X1_ZOA = X;
			Y1_ZOA = Y_ZOA;
		}

		if (power[i] == 0.)
		{
			ZOD[i] = 0.;
			ZOA[i] = 0.;

		}
		else
		{

			ZOD[i] = -1 * sigma_ZOD * log(power[i] / strongest_power) / C;
			ZOA[i] = -1 * sigma_ZOA * log(power[i] / strongest_power) / C;

		}

		if (i == 0)
		{
			first_ZOD = ZOD[0];
			first_ZOA = ZOA[0];
		}




		if (true) // LOS
		{

			LOCATION vector;

			// bs ms  angle 
			double sector_idx;

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);

			ZOA[i] = (X * ZOA[i] + Y_ZOA) - (X1_ZOA * first_ZOA + Y1_ZOA - angle);
			ZOD[i] = (X * ZOD[i] + Y_ZOD) - (X1_ZOD * first_ZOD + Y1_ZOD - angle);




		}

		ZOA_LOS[i] = ZOA[i];
		ZOD_LOS[i] = ZOD[i];



	}

	delete[] ZOA;
	delete[] ZOD;



}


void CHANNEL::Set_ZOAZOD_NLOS()
{

	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - NLOS case
	///
	/////////////////////////////////////////////////

	double sigma_ZOD = pow(10, normal(mu_ZSD, sigma_ZSD));
	double sigma_ZOA = pow(10, normal(mu_ZSA, sigma_ZSA));

	double C = 1;

	if (TYPE == 6)   //////// 
	{
			C = 1.184;
	}
	else if (TYPE == 7)
	{
			C = 1.178;
	}
	else if (TYPE == 8)
	{
			C = 1.178;
	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}


	ZOD = new double[MAX_NUM_CLUSTERS];
	ZOA = new double[MAX_NUM_CLUSTERS];

	ZOD_NLOS = new double[MAX_NUM_CLUSTERS];
	ZOA_NLOS = new double[MAX_NUM_CLUSTERS];

	double first_ZOD;
	double first_ZOA;

	double X1_ZOD;
	double Y1_ZOD;
	double X1_ZOA;
	double Y1_ZOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_ZOA = normal(0, pow(sigma_ZOA / 7., 2.)); //
		double Y_ZOD = normal(0, pow(sigma_ZOD / 7., 2.));

		if (i == 0)
		{
			X1_ZOD = X;
			Y1_ZOD = Y_ZOD;

			X1_ZOA = X;
			Y1_ZOA = Y_ZOA;
		}

		if (power[i] == 0.)
		{
			ZOD[i] = 0.;
			ZOA[i] = 0.;

		}
		else
		{

			ZOD[i] = -1 * sigma_ZOD * log(power[i] / strongest_power) / C;
			ZOA[i] = -1 * sigma_ZOA * log(power[i] / strongest_power) / C;

		}

		if (i == 0)
		{
			first_ZOD = ZOD[0];
			first_ZOA = ZOA[0];
		}




		if (true) // NLOS
		{

			LOCATION vector;

			// bs ms  angle 
			double sector_idx;

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);



			if (Indoor == 1)
			{
				ZOA[i] = X * ZOA[i] + Y_ZOA + 90.;                    // indoor϶ angle 90  (36.873)
				ZOD[i] = X * ZOD[i] + Y_ZOA + angle + mu_offset_ZOD; //̰͸ ܷ ٸ //߿ ¾ޱ۵  ٸǷ ؾѴ
			}
			else
			{
				ZOA[i] = X * ZOA[i] + Y_ZOA + angle;
				ZOD[i] = X * ZOD[i] + Y_ZOA + angle + mu_offset_ZOD; //̰͸ ܷ ٸ //߿ ¾ޱ۵  ٸǷ ؾѴ
			}


		}

		ZOA_NLOS[i] = ZOA[i];
		ZOD_NLOS[i] = ZOD[i];

	}

	delete[] ZOA;
	delete[] ZOD;



}


void CHANNEL::Load_precalculate(double t, int _ms_idx, int _initial_time_offset)
{
	ms_idx = _ms_idx;


	int los_idx;
	int indoor_idx;
	int v_idx;
	
	

	if (LOS == 1)
	{
		los_idx = 1;
	}
	else
	{
		los_idx = 0;
	}

	if (Indoor == 1) //////// InH   ִ
	{
		indoor_idx = 1;
	}
	else
	{
		indoor_idx = 0;
	}


	if (TYPE == 1 || TYPE == 2 || TYPE == 3 || TYPE == 4 || TYPE == 5)
	{
		double angle_v = atan2((bs_height - ms_height), distance) * 180. / pi;

		//// 
		if (angle_v > MAX_v_angle - (MAX_v_angle/anglev_mapping_value))
		{
			v_idx = 2;
		}
		else if (angle_v < (MAX_v_angle / anglev_mapping_value))
		{
			v_idx = 0;
		}
		else
		{
			v_idx = 1;
		}
	}
	else
	{
		//v_idx = (int)(anglev_mapping_value * randnum.u());
		v_idx = 0;  // 3D channel model vertical angle , ZOA ϱ⶧
	}




	//if (TYPE == 1 || TYPE == 2 || TYPE == 3 || TYPE == 4 || TYPE == 5)
	//{
		//for (int u = 0; u < NUM_TX; u++)
		//{
			//for (int s = 0; s < NUM_RX; s++)
			//{
				//for (int i = 0; i < num_path; i++)
				//{
					//CHIR[u][s][i] = CHIR_precalc[(int)(_initial_time_offset + t) % precalculate_time][los_idx][indoor_idx][v_idx][u][s][i];
				//}
			//}
		//}
	//}

}



void CHANNEL::Update(double t, int _ms_idx)
{
	/*//////////////////////////////////////////////////////////
	// INPUT:
	// OUTPUT: CHIR
	// Notes: Channel coefficient generation - M.2412 page.59
	*///////////////////////////////////////////////////////////

	VectorXcd a_tx(2 * BS_N*BS_M*BS_Ng*BS_Mg);
	VectorXcd a_rx(2 * MS_N*MS_M*MS_Ng*MS_Mg);
	VectorXcd r_tx(3);
	VectorXcd r_rx(3);

	ms_idx = _ms_idx;

	if (ms_idx % 30 < 10)
	{
		alpha = 30.;
	}
	else if (ms_idx % 30 < 20)
	{
		alpha = 150.;
	}
	else
	{
		alpha = 270.;
	}

	for (int u = 0; u < NUM_TX; u++)
	{
		for (int s = 0; s < NUM_RX; s++)
		{
			for (int i = 0; i < NUM_PATH_for_channelcoeff; i++)
			{
				//cout << "NUM_PATH_for_channelcoeff = " << NUM_PATH_for_channelcoeff << endl;

				for (int j = 0; j < NUM_RAY_per_ClusterNUM[i]; j++)
				{
					//cout << "NUM_RAY_per_ClusterNUM = " << NUM_RAY_per_ClusterNUM[i] << endl;
					//getchar();


					complex <double>jay(0, 1);

					double ray_AOD = AOD[i] + cluster_ASD * offset_angle[i][j];
					double ray_AOA = AOA[i] + cluster_ASA * offset_angle[i][j];

					double ray_ZOA = ZOA[i] + cluster_ZSA * offset_angle[i][j];
					double ray_ZOD = ZOD[i] + (3 / 8) * pow(10, mu_ZSD) * offset_angle[i][j];



					///////////////  GCS -> LCS ??

					r_rx << sin(ray_ZOA)*cos(ray_AOA), sin(ray_ZOA)*sin(ray_AOA), cos(ray_ZOA);
					r_tx << sin(ray_ZOD)*cos(ray_AOD), sin(ray_ZOD)*sin(ray_AOD), cos(ray_ZOD);

					VectorXcd Wr_tx, Wr_rx;

					Wr_tx = W_tx * r_tx;
					Wr_rx = W_rx * r_rx;



					for (int vector_size = 0; vector_size < 2 * BS_N*BS_M*BS_Ng*BS_Mg; vector_size++)
					{
						a_tx(vector_size) = exp(jay * 2. * pi * Wr_tx(vector_size));
					}
					for (int vector_size = 0; vector_size < 2 * MS_N*MS_M*MS_Ng*MS_Mg; vector_size++)
					{
						a_rx(vector_size) = exp(jay * 2. * pi * Wr_rx(vector_size));
					}



					/*
					///////////F_tx

					double theta_prime_tx = Get_LCS_theta(alpha, beta, gamma, ray_ZOD, ray_AOD);
					double pi_prime_tx = Get_LCS_pi(alpha, beta, gamma, ray_ZOD, ray_AOD);


					double h_antgain_tx = -MIN(12. * (pi_prime_tx / 65.)*(pi_prime_tx / 65.), 30.);
					double v_antgain_tx = -MIN(12. * ((theta_prime_tx - 90) / 65.) * ((theta_prime_tx - 90) / 65.), 30.);


					double A_prime_tx = -MIN(-(h_antgain_tx + v_antgain_tx), 30.);


					complex <double> F_prime_tx_theta = sqrt(A_prime_tx) * cos(gamma);
					complex <double> F_prime_tx_pi    = sqrt(A_prime_tx) * sin(gamma);


					complex <double> cos_psi_tx = Get_cos_psi(alpha, beta, gamma, ray_ZOD, ray_AOD);
					complex <double> sin_psi_tx = Get_sin_psi(alpha, beta, gamma, ray_ZOD, ray_AOD);


					F_tx_v = cos_psi_tx * F_prime_tx_theta - sin_psi_tx * F_prime_tx_pi;
					F_tx_h = sin_psi_tx * F_prime_tx_theta + cos_psi_tx * F_prime_tx_pi;

					//////////F_rx_v
					F_rx_v = 0.;

					//////////F_rx_h
					F_rx_h = 0.;
					*/
					//////////////////////////////////////////////////////////////////////////////////

					F_tx_v = -MIN(12. * ((ray_ZOD - 90) / 65.) * ((ray_ZOD - 90) / 65.), 30.);
					//F_tx_v = -MIN(12. * ((ray_ZOD - 180) / 90.) * ((ray_ZOD - 180) / 90.), 25.);

					F_tx_h = -MIN(12. * (ray_AOD / 65.)*(ray_AOD / 65.), 30.);
					//F_tx_h = 0;

					//////////F_rx_v
					F_rx_v = -MIN(12. * ((ray_ZOA - 90) / 65.) * ((ray_ZOA - 90) / 65.), 30.);
					//F_rx_v = 0;

					//////////F_rx_h
					F_rx_h = -MIN(12. * (ray_AOA / 65.)*(ray_AOA / 65.), 30.);
					//F_rx_h = 0;

					F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
					F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
					F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
					F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);

					//cout << "F_vv = " << F_vv << F_vh << F_hh << F_hv << endl;
					//getchar();

					/////////////////////////////////////////



					VectorXd UT_velocity(3);

					UT_velocity << ms[ms_idx].speed*sin(ray_ZOA)*cos(ray_AOA), ms[ms_idx].speed*sin(ray_ZOA)*sin(ray_AOA), ms[ms_idx].speed*cos(ray_ZOA);


					RowVectorXcd r_rx_T = r_rx.transpose();
					RowVectorXcd a_tx_H = a_tx.adjoint();



					complex <double> doppler_freq = r_rx_T * UT_velocity;

					doppler_freq = doppler_freq / Wavelength;

					//cout << "doppler_freq = " << doppler_freq << endl;

					double xpr = pow(10, XPR / 10);

					//complex <double> a_rx_a_tx_H = a_tx.adjoint() * a_rx;



					MatrixXcd a_rx_a_tx_H;

					a_rx_a_tx_H = a_rx * a_tx.adjoint();  //// num_RX x 1  *   1 x num_TX


					//cout << "a_rx_a_tx_H = " << a_rx_a_tx_H << endl;


					complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv[i][j]) + sqrt(1 / xpr)*F_hv*exp(jay*random_phase_hv[i][j]) + sqrt(1 / xpr)*F_vh*exp(jay*random_phase_vh[i][j]) + F_hh*exp(jay*random_phase_hh[i][j]);



					//CHIR[u][s][i] += sqrt(power[i] / NUM_RAY_per_ClusterNUM[i]) * antenna_gain * a_rx_a_tx_H(s, u) * exp(jay * 2. * pi * doppler_freq * t / 1000.);  ///// ð ִ°   ѹ ϵ....


					CHIR[u][s][i] += sqrt(power[i] / NUM_RAY_per_ClusterNUM[i]) * antenna_gain * a_rx_a_tx_H(s, u);





					/*
					if (ms_idx == 4)
					{
					cout << "sqrt(power[i] / NUM_RAY_per_ClusterNUM[i]) =" << sqrt(power[i] / NUM_RAY_per_ClusterNUM[i]);
					cout << "antenna_gain = " << antenna_gain << endl;
					cout << " " << endl;
					cout << "ZOD[i] = " << ZOD[i] << endl;
					cout << "(mu_ZSD) = " << mu_ZSD << endl;
					cout << "offset_angle[i][j] = " << offset_angle[i][j] << endl;
					cout << "ray_ZOD = " << ray_ZOD << endl;
					cout << "ray_AOD = " << ray_AOD << endl;
					cout << "Wr_tx =" << Wr_tx << endl;
					cout << "W_tx = " << W_tx << endl;
					cout << "r_tx = " << r_tx << endl;
					cout << "a_rx = " << a_rx << endl;
					cout << "a_tx = " << a_tx << endl;
					cout << "a_rx_a_tx_H(u, s) = " << a_rx_a_tx_H(u, s) << endl;
					cout << "exp(jay * 2. * pi * doppler_freq * t / 1000.) = " << exp(jay * 2. * pi * doppler_freq * t / 1000.) << endl;
					cout << "CHIR[u][s][i] = " << CHIR[u][s][i] << endl;
					getchar();
					}
					*/

					//cout << "CHIR[u][s][i] = " << CHIR[u][s][i] << endl;

				}
			}
		}
	}

	if (LOS == 1)
	{
		complex <double> jay(0, 1);

		//double k_factor = pow(10, K_factor / 10);

		for (int u = 0; u < NUM_TX; u++)
		{
			for (int s = 0; s < NUM_RX; s++)
			{
				for (int i = 0; i < NUM_PATH_for_channelcoeff; i++)
				{
					CHIR[u][s][i] *= sqrt(1 / (K_linear + 1));
				}
			}
		}

		double ray_delay = delay[0];
		double ray_AOD = AOD[0];
		double ray_AOA = AOA[0];

		double ray_ZOA = ZOA[0];
		double ray_ZOD = ZOD[0];


		//////////////////////////// GCS -> LCS ??? //////////////

		r_rx << sin(ray_ZOA)*cos(ray_AOA), sin(ray_ZOA)*sin(ray_AOA), cos(ray_ZOA);
		r_tx << sin(ray_ZOD)*cos(ray_AOD), sin(ray_ZOD)*sin(ray_AOD), cos(ray_ZOD);

		VectorXcd Wr_tx, Wr_rx;

		Wr_tx = W_tx * r_tx;
		Wr_rx = W_rx * r_rx;

		for (int vector_size = 0; vector_size < 2 * BS_N*BS_M*BS_Ng*BS_Mg; vector_size++)
		{
			a_tx(vector_size) = exp(jay * 2. * pi * Wr_tx(vector_size));
		}
		for (int vector_size = 0; vector_size < 2 * MS_N*MS_M*MS_Ng*MS_Mg; vector_size++)
		{
			a_rx(vector_size) = exp(jay * 2. * pi * Wr_rx(vector_size));
		}

		/*
		///////////F_tx

		double theta_prime_tx = Get_LCS_theta(alpha, beta, gamma, ray_ZOD, ray_AOD);
		double pi_prime_tx = Get_LCS_pi(alpha, beta, gamma, ray_ZOD, ray_AOD);

		double h_antgain_tx = -MIN(12. * (pi_prime_tx / 65.)*(pi_prime_tx / 65.), 30.);
		double v_antgain_tx = -MIN(12. * ((theta_prime_tx - 90) / 65.) * ((theta_prime_tx - 90) / 65.), 30.);

		double A_prime_tx = -MIN(-(h_antgain_tx + v_antgain_tx), 30.);

		complex <double> F_prime_tx_theta = sqrt(A_prime_tx) * cos(gamma);
		complex <double> F_prime_tx_pi = sqrt(A_prime_tx) * sin(gamma);

		complex <double> cos_psi_tx = Get_cos_psi(alpha, beta, gamma, ray_ZOD, ray_AOD);
		complex <double> sin_psi_tx = Get_sin_psi(alpha, beta, gamma, ray_ZOD, ray_AOD);

		F_tx_v = cos_psi_tx * F_prime_tx_theta - sin_psi_tx * F_prime_tx_pi;
		F_tx_h = sin_psi_tx * F_prime_tx_theta + cos_psi_tx * F_prime_tx_pi;


		//////////F_rx_v
		F_rx_v = 0.;

		//////////F_rx_h
		F_rx_h = 0.;
		*/

		//////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////

		F_tx_v = -MIN(12. * ((ray_ZOD - 90) / 65.) * ((ray_ZOD - 90) / 65.), 30.);
		F_tx_h = -MIN(12. * (ray_AOD / 65.)*(ray_AOD / 65.), 30.);

		//////////F_rx_v
		F_rx_v = -MIN(12. * ((ray_ZOA - 90) / 65.) * ((ray_ZOA - 90) / 65.), 30.);

		//////////F_rx_h
		F_rx_h = -MIN(12. * (ray_AOA / 65.)*(ray_AOA / 65.), 30.);

		F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
		F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
		F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
		F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);

		//cout << "F_vv = " << F_vv << F_vh << F_hh << F_hv << endl;
		//getchar();

		/////////////////////////////////////////
		///////////////////////////////////////////


		/////////////////////////////////////////

		VectorXd UT_velocity(3);

		UT_velocity << ms[ms_idx].speed*sin(ray_ZOA)*cos(ray_AOA), ms[ms_idx].speed*sin(ray_ZOA)*sin(ray_AOA), ms[ms_idx].speed*cos(ray_ZOA);

		RowVectorXcd r_rx_T = r_rx.transpose();
		RowVectorXcd a_tx_H = a_tx.adjoint();

		complex <double> doppler_freq = r_rx_T * UT_velocity;

		doppler_freq = doppler_freq / Wavelength;

		double xpr = pow(10, XPR / 10);

		//complex <double> a_rx_a_tx_H = a_tx.adjoint() * a_rx;



		MatrixXcd a_rx_a_tx_H;

		a_rx_a_tx_H = a_rx * a_tx.adjoint();



		complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv_LOS) + F_hh*exp(jay*random_phase_hh_LOS);

		for (int u = 0; u < NUM_TX; u++)
		{
			for (int s = 0; s < NUM_RX; s++)
			{
				//CHIR[u][s][0] += sqrt(K_linear / (K_linear + 1)) * antenna_gain * a_rx_a_tx_H(s,u) * exp(jay * 2. * pi * doppler_freq * t / 1000.); //// ð ִ°   ѹ ϵ....

				CHIR_LOS[u][s][0] += sqrt(K_linear / (K_linear + 1)) * antenna_gain * a_rx_a_tx_H(s, u);
			}
		}

	}





	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////




}


void CHANNEL::Update_per_time(double t, int _ms_idx)
{
	/*//////////////////////////////////////////////////////////
	// INPUT:
	// OUTPUT: CHIR
	// Notes: Channel coefficient generation - M.2412 page.59
	*///////////////////////////////////////////////////////////

	VectorXcd r_rx(3);

	ms_idx = _ms_idx;



	for (int u = 0; u < NUM_TX; u++)
	{
		for (int s = 0; s < NUM_RX; s++)
		{
			for (int i = 0; i < NUM_PATH_for_channelcoeff; i++)
			{
				for (int j = 0; j < NUM_RAY_per_ClusterNUM[i]; j++)
				{
					complex <double>jay(0, 1);

					double ray_AOD = AOD[i] + cluster_ASD * offset_angle[i][j];
					double ray_AOA = AOA[i] + cluster_ASA * offset_angle[i][j];

					double ray_ZOA = ZOA[i] + cluster_ZSA * offset_angle[i][j];
					double ray_ZOD = ZOD[i] + (3 / 8) * pow(10, mu_ZSD) * offset_angle[i][j];


					r_rx << sin(ray_ZOA)*cos(ray_AOA), sin(ray_ZOA)*sin(ray_AOA), cos(ray_ZOA);


					VectorXd UT_velocity(3);

					UT_velocity << ms[ms_idx].speed*sin(ray_ZOA)*cos(ray_AOA), ms[ms_idx].speed*sin(ray_ZOA)*sin(ray_AOA), ms[ms_idx].speed*cos(ray_ZOA);


					RowVectorXcd r_rx_T = r_rx.transpose();



					complex <double> doppler_freq = r_rx_T * UT_velocity;

					doppler_freq = doppler_freq / Wavelength;



					//CHIR[u][s][i] += sqrt(power[i] / NUM_RAY_per_ClusterNUM[i]) * antenna_gain * a_rx_a_tx_H(s, u) * exp(jay * 2. * pi * doppler_freq * t / 1000.);  ///// ð ִ°   ѹ ϵ....

					CHIR[u][s][i] *= exp(jay * 2. * pi * doppler_freq * t / 1000.);
				}
			}
		}
	}

	if (LOS == 1)
	{
		complex <double> jay(0, 1);



		double ray_delay = delay[0];
		double ray_AOD = AOD[0];
		double ray_AOA = AOA[0];

		double ray_ZOA = ZOA[0];
		double ray_ZOD = ZOD[0];


		//////////////////////////// GCS -> LCS ??? //////////////

		r_rx << sin(ray_ZOA)*cos(ray_AOA), sin(ray_ZOA)*sin(ray_AOA), cos(ray_ZOA);






		VectorXd UT_velocity(3);

		UT_velocity << ms[ms_idx].speed*sin(ray_ZOA)*cos(ray_AOA), ms[ms_idx].speed*sin(ray_ZOA)*sin(ray_AOA), ms[ms_idx].speed*cos(ray_ZOA);

		RowVectorXcd r_rx_T = r_rx.transpose();


		complex <double> doppler_freq = r_rx_T * UT_velocity;

		doppler_freq = doppler_freq / Wavelength;



		for (int u = 0; u < NUM_TX; u++)
		{
			for (int s = 0; s < NUM_RX; s++)
			{
				//CHIR[u][s][0] += sqrt(K_linear / (K_linear + 1)) * antenna_gain * a_rx_a_tx_H(s,u) * exp(jay * 2. * pi * doppler_freq * t / 1000.); //// ð ִ°   ѹ ϵ....

				CHIR_LOS[u][s][0] *= exp(jay * 2. * pi * doppler_freq * t / 1000.);

				CHIR[u][s][0] += CHIR_LOS[u][s][0];
			}
		}

	}





	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////




}










/*
double Get_LCS_theta(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi)
{
	return acos(cos(beta)*cos(gamma)*cos(GCS_theta) + (sin(beta)*cos(gamma)*cos(GCS_pi - alpha) - sin(gamma)*sin(GCS_pi - alpha))*sin(GCS_theta)) * 180. / pi;
}

double Get_LCS_pi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi)
{

	double real;
	double imag;

	real = (cos(beta)*sin(GCS_theta)*cos(GCS_pi - alpha) - sin(beta)*cos(GCS_theta));
	imag = (cos(beta)*sin(gamma)*cos(GCS_theta) + (sin(beta)*sin(gamma)*cos(GCS_pi - alpha) + cos(gamma)*sin(GCS_pi - alpha))*sin(GCS_theta));

	complex <double> result = (real, imag);

	return arg(result) * 180. / pi;

}
*/
double Get_cos_psi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi)
{
	double A = cos(beta)*cos(gamma)*sin(GCS_theta) - (sin(beta)*cos(gamma)*cos(GCS_pi - alpha) - sin(gamma)*sin(GCS_pi - alpha))*cos(GCS_theta);

	double B = cos(beta)*cos(gamma)*cos(GCS_theta) - (sin(beta)*cos(gamma)*cos(GCS_pi - alpha) - sin(gamma)*sin(GCS_pi - alpha))*sin(GCS_theta);

	double cos_psi = A / sqrt(1 - B*B);

	return cos_psi;
}

double Get_sin_psi(double alpha, double beta, double gamma, double GCS_theta, double GCS_pi)
{
	double A = sin(beta)*cos(gamma)*sin(GCS_pi - alpha) + sin(gamma)*cos(GCS_pi - alpha);

	double B = cos(beta)*cos(gamma)*cos(GCS_theta) - (sin(beta)*cos(gamma)*cos(GCS_pi - alpha) - sin(gamma)*sin(GCS_pi - alpha))*sin(GCS_theta);

	double sin_psi = A / sqrt(1 - B*B);

	return sin_psi;
}